{{- /*
  Usage: `callout "type"`, where `type` is one of info (default), danger, or warning
*/ -}}

{{- $css_class := .Get 0 | default "info" -}}

<div class="bd-callout bd-callout-{{ $css_class }}">
{{ .Inner | markdownify }}
</div>
