{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_lp/competency_path

    Competency path template.

    Classes required for JS:
    * competencydialogue

    Context variables required for this template:
    * framework - The competency framework
      * id - competency id
      * name - competency idnumber

    * ancestors - array of nodes
      * id - competency id
      * name - competency idnumber
      * first - true if node is in first position
      * last - true if node is in last position

    * pluginbaseurl - base url of plugin tool_lp

    Example context (json):
    {
        "framework": {
            "id": "1",
            "name": "Framework"
        },
        "ancestors": [
            {
                "id": "1",
                "name": "C1"
            },
            {
                "id": "1",
                "name": "C2",
                "last": true
            }
        ]
    }
}}
<nav id="competency-path-{{uniqid}}">
    <small>
        {{#showlinks}}
        <a href="{{pluginbaseurl}}/competencies.php?competencyframeworkid={{framework.id}}&pagecontextid={{pagecontextid}}" >
            {{{framework.name}}}
        </a>
        {{/showlinks}}
        {{^showlinks}}
        {{{framework.name}}}
        {{/showlinks}}
        /
        {{#ancestors}}
            {{#showlinks}}
            <a href="{{pluginbaseurl}}/competencies.php?competencyid={{id}}">
                {{{name}}}
            </a>
            {{/showlinks}}
            {{^showlinks}}
                {{{name}}}
            {{/showlinks}}
            {{^last}}<span> / </span>{{/last}}
        {{/ancestors}}
    </small>
</nav>
{{#js}}
require(['tool_lp/competencydialogue'], function(Compdialogue) {
    Compdialogue.init();
});
{{/js}}
