{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template forumreport_summary/filter_dates_popover

    Summary report dates filter.

    Example context (json):
    {
        "filterdatesform": "<div>HTML for date filters</div>"
    }
}}

<div id="filter-dates-popover" class="popover filter-dates-popover mt-3 hidden" data-openfilter="false">
    <h3 class="popover-header">{{# str}} filter:datesname, forumreport_summary {{/ str}}</h3>
    <div class="popover-body" data-region="filter-dates">
        {{{filterdatesform}}}
        <div id="dates-filter-warning" class="form-control-feedback text-danger hidden float-end">
            {{# str}} filter:datesorderwarning, forumreport_summary {{/ str}}
        </div>
        <br>&nbsp;
        <button type="button" class="filter-save float-end btn btn-link p-0" aria-label="{{# str}} save {{/ str}}">
            <strong>{{# str}} save {{/ str}}</strong>
        </button>
    </div>
</div>
