<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for core component 'bulkusers'
 *
 * @package    core
 * @subpackage bulkusers
 * @copyright  1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['addall'] = 'Add all';
$string['addsel'] = 'Add to selection';
$string['allfilteredusers'] = 'All filtered ({$a->count}/{$a->total})';
$string['allselectedusers'] = 'All selected ({$a->count}/{$a->total})';
$string['allusers'] = 'All users ({$a})';
$string['available'] = 'Available';
$string['confirmmessage'] = 'Do you really want to send the message above to all these users?<br />{$a}';
$string['deleteduser'] = 'Deleted user';
$string['nofilteredusers'] = 'No users found (0/{$a})';
$string['noselectedusers'] = 'No users selected';
$string['removeall'] = 'Remove all';
$string['removesel'] = 'Remove from selection';
$string['selected'] = 'Selected';
$string['selectedlist'] = 'Selected user list...';
$string['selectedlist_help'] = 'Users can be added or removed from the selected users list by clicking on the users\' names then clicking on an appropriate button. Multiple users may be selected by holding down the Apple or Ctrl key whilst clicking on the users\' names.';
$string['users'] = 'Users';
$string['users_help'] = 'All users who passed the active filters are listed in the available box. If no filter is set, all site users are listed.';
$string['usersfound'] = '{$a} user(s) found.';
$string['usersinlist'] = 'Users in list';
$string['usersselected'] = '{$a} user(s) selected.';
