{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/cm/cmname

    Convenience mustache to displays a course module inplace editable from the format renderer.

    Format plugins are free to implement an alternative inplace editable for activities.

    Example context (json):
    {
        "url": "#",
        "pluginname": "File",
        "textclasses": "",
        "modname": "resource",
        "activityicon": {
            "icon": "../../../pix/help.svg",
            "iconclass": "",
            "purpose": "content"
        },
        "activityname": {
            "displayvalue" : "<a href=\"#\">Moodle</a>",
            "value" : "Moodle",
            "itemid" : "1",
            "component" : "core_unknown",
            "itemtype" : "unknown",
            "edithint" : "Edit this",
            "editlabel" : "New name for this",
            "type" : "text",
            "options" : "",
            "linkeverything": 0
        }
    }
}}
{{#url}}
    {{! Icon }}
    {{#activityicon}}
        {{$ core_courseformat/local/content/cm/cmicon }}
            {{> core_courseformat/local/content/cm/cmicon }}
        {{/ core_courseformat/local/content/cm/cmicon }}
    {{/activityicon}}

    {{! Name & Badge}}
    <div class="activity-name-area activity-instance d-flex flex-column me-2">
        <div class="activitytitle {{textclasses}} modtype_{{modname}} position-relative align-self-start">
            <div class="activityname">
                {{#activityname}}
                    {{$ core/inplace_editable }}
                        {{> core/inplace_editable }}
                    {{/ core/inplace_editable }}
                {{/activityname}}
                {{#activitybadge}}
                    {{$ core_courseformat/local/content/cm/activitybadge }}
                        {{> core_courseformat/local/content/cm/activitybadge }}
                    {{/ core_courseformat/local/content/cm/activitybadge }}
                {{/activitybadge}}
            </div>
        </div>
    </div>

{{/url}}
