{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_lti/external_registration_return

    This template provides the layout for the external tool configuration page in the
    LTI module.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    *

}}
<div id="external-registration-return-container">
    <div class="registration-loading-container loading-screen">
        {{> mod_lti/loader }}
        <p class="loading-text">{{#str}} loadinghelp, moodle {{/str}}</p>
    </div>
</div>
{{#js}}
    require(['mod_lti/external_registration'], function(registration) {
        registration.init();
    });
{{/js}}
