// stylelint-disable scss/dollar-variable-default

// Local docs variables
$bd-purple:        #563d7c;
$bd-purple-bright: lighten(saturate($bd-purple, 5%), 15%);
$bd-purple-light:  lighten(saturate($bd-purple, 5%), 45%);
$bd-dark:          #2a2730;
$bd-download:      #ffe484;
$bd-info:          #5bc0de;
$bd-warning:       #f0ad4e;
$bd-danger:        #d9534f;
