{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_feedback/main_action_menu
    Tertiary nav for the feedback module
    Classes required for JS:
        * none
    Data attributes required for JS:
        * data-region
    Context variables required for this template:
        * see mod/feedback/classes/output/edit_action_bar.php
        * see mod/feedback/classes/output/edit_template_action_bar.php
        * see mod/feedback/classes/output/responses_action_bar.php
        * see mod/feedback/classes/output/standard_action_bar.php
    Example context (json):
    {
        "left": {
            "actionlink": {
                "disabled": false,
                "url": "#",
                "id": "test-id",
                "classes": "btn btn-link",
                "attributes": [
                    {
                        "name": "title",
                        "value": "Action link's title"
                    }
                ],
                "icon": {
                    "attributes": [
                        {"name": "src", "value": "https://raw.githubusercontent.com/moodle/moodle/master/pix/t/check.png"},
                        {"name": "alt", "value": "Action link's icon"}
                    ]
                },
                "text": "This is an action link"
            },
            "urlselect": {
                "id": "url_select_test",
                "action": "https://example.com/post",
                "formid": "url_select_form",
                "sesskey": "sesskey",
                "label": "core/url_select",
                "helpicon": {
                    "title": "Help with something",
                    "text": "Help with something",
                    "url": "http://example.org/help",
                    "linktext": "",
                    "icon":{
                        "extraclasses": "",
                        "attributes": [
                            {"name": "src", "value": "../../../pix/help.svg"},
                            {"name": "alt", "value": "Help icon"}
                        ]
                    }
                },
                "showbutton": "Go",
                "options": [{
                    "name": "Group 1", "isgroup": true, "options":
                    [
                        {"name": "Item 1", "isgroup": false, "value": "1"},
                        {"name": "Item 2", "isgroup": false, "value": "2"}
                    ]},
                    {"name": "Group 2", "isgroup": true, "options":
                    [
                        {"name": "Item 3", "isgroup": false, "value": "3"},
                        {"name": "Item 4", "isgroup": false, "value": "4"}
                    ]}],
                "disabled": false,
                "title": "Some cool title"
            },
            "singleselect": {
                "name": "lang",
                "method": "get",
                "action": "http://localhost/stable_master/mod/scorm/player.php",
                "classes": "langmenu",
                "label": "Zombies are coming...",
                "disabled": false,
                "title": null,
                "formid": "randomid",
                "id": "single_select5833dd4f4b08d108",
                "params": [
                    {
                        "name": "scoid",
                        "value": "12"
                    },
                    {
                        "name": "cm",
                        "value": "15"
                    },
                    {
                        "name": "mode",
                        "value": "review"
                    },
                    {
                        "name": "currentorg",
                        "value": "eXeMapADrive4823c6301cf72b22b72"
                    }
                ],
                "options": [
                    {
                        "value": "en",
                        "name": "English ‎(en)‎",
                        "selected": true,
                        "optgroup": false
                    },
                    {
                        "value": "ar",
                        "name": "Muhaaaa..",
                        "selected": false,
                        "optgroup": false
                    }
                ],
                "labelattributes": [],
                "helpicon": false
            }
        }
    }
}}
<div class="container-fluid tertiary-navigation">
    <div class="d-flex">
        {{#left}}
            {{#actionlink}}
                <div class="navitem">
                    {{> core/action_link}}
                </div>
            {{/actionlink}}
            {{#urlselect}}
                <div class="navitem">
                    {{> core/url_select}}
                </div>
            {{/urlselect}}
            {{#singleselect}}
                <div class="navitem">
                    {{> core/single_select}}
                </div>
            {{/singleselect}}
        {{/left}}
        {{#addselect}}
            <div class="navitem">
                {{> core/action_menu }}
            </div>
        {{/addselect}}
        {{#actionsselect}}
            <div class="navitem ms-sm-auto">
                {{> core/action_menu }}
            </div>
        {{/actionsselect}}
    </div>
</div>
