#page-report-progress-index #completion-progress th,
#page-report-progress-index #completion-progress td {
    padding: 2px 2px;
    font-weight: normal;
    border-right: 1px solid #eee;
}

#page-report-progress-index .progress-actions {
    text-align: center;
    list-style: none;
}

#page-report-progress-index .completion_pagingbar {
    margin: 1em 0;
    text-align: center;
}

#page-report-progress-index .completion_prev {
    display: inline;
    margin-right: 2em;
}

#page-report-progress-index .completion_pagingbar p {
    display: inline;
}

#page-report-progress-index .completion_next {
    display: inline;
    margin-left: 2em;
}

#page-report-progress-index .completion-header,
#page-report-progress-index .completion-progresscell {
    text-align: center;
}

/* Custom CSS for rotated header.. */
#page-report-progress-index .rotated-text-container {
    display: inline-block;
    width: 16px;
}

#page-report-progress-index .rotated-text {
    display: inline-block;
    white-space: nowrap;
    /*rtl:remove*/
    transform: translate(0, 100%) rotate(-90deg);
    /*rtl:raw:
    transform: translate(0, 100%) rotate(90deg);
    */
    transform-origin: 0 0;
    vertical-align: middle;
}

#page-report-progress-index .rotated-text:after {
    content: "";
    float: left;
    margin-top: 100%;
}

#page-report-progress-index .modicon {
    padding-top: 5px;
}
#completion-progress .icon {
    margin-left: 0.5rem;
}
