.que.calculatedmulti .answer .specificfeedback {
    display: inline;
    padding: 0 0.7em;
    background: #fff3bf;
}

.que.calculatedmulti .answer .specificfeedback * {
    display: inline;
    background: #fff3bf;
}

.que.calculatedmulti .answer .specificfeedback script {
    display: none;
}


.que.calculatedmulti .answer div.r0,
.que.calculatedmulti .answer div.r1 {
    display: flex;
    margin: 0.25rem 0;
    align-items: flex-start;
}

.que.calculatedmulti .answer div.r0 input,
.que.calculatedmulti .answer div.r1 input {
    margin: 0.3rem 0.5rem;
    width: 14px;
    flex-shrink: 0;
}
