<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component qbank_usage, language 'en'
 *
 * @package    qbank_usage
 * @copyright  2021 Catalyst IT Australia Pty Ltd
 * @author     Safat Shahin <safatshahin@catalyst-au.net>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['pluginname'] = 'Question usage';
$string['notused'] = 'Never';
$string['privacy:metadata'] = 'The Question usage question bank plugin does not store any personal data.';
$string['questionusage'] = 'Usage';
$string['questionusage_help'] = 'The number of quizzes in which the question is used, with a link to open a window listing the quizzes and the number of attempts.';
$string['questionlastused'] = 'Last used';
$string['questionlastused_help'] = 'The date when the question was last attempted.';
$string['usageheader'] = 'Question usage';

// Table.
$string['modulename'] = 'Activity name';
$string['coursename'] = 'Course name';
$string['versions'] = 'Version';
$string['state'] = 'State';
$string['attempts'] = 'Attempts';
$string['questionusageversion'] = 'v{$a}';
