{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_lesson/edit_action_buttons

    Actions panel for essay grading.

    Classes required for JS:
    * none

    Context variables required for this template:
    * see mod/lesson/classes/output/edit_action_buttons.php

    Example context (json):
    {
        "edit": {
            "button": {
                "id" : "id1",
                "method" : "get",
                "url" : "#",
                "formid": "1",
                "primary" : true,
                "tooltip" : "This is a tooltip",
                "label" : "Edit",
                "attributes": [
                    {
                      "name": "data-attribute",
                      "value": "yeah"
                    }
                ]
            }
        },
        "gradeessays": {
            "button": {
                "id" : "id2",
                "method" : "get",
                "url" : "#",
                "formid": "2",
                "primary" : false,
                "tooltip" : "This is a tooltip",
                "label" : "Grade essays",
                "attributes": [
                    {
                      "name": "data-attribute",
                      "value": "yeah"
                    }
                ]
            }
        }
    }

}}
<div class="container-fluid tertiary-navigation">
    <div class="d-flex">
        {{#edit}}
            {{#button}}
                <div class="navitem">
                    {{>core/single_button}}
                </div>
            {{/button}}
        {{/edit}}
        {{#editcontents}}
            {{#button}}
                <div class="navitem">
                    {{>core/single_button}}
                </div>
            {{/button}}
        {{/editcontents}}
        {{#gradeessays}}
            {{#button}}
                <div class="navitem">
                    {{>core/single_button}}
                </div>
            {{/button}}
        {{/gradeessays}}
    </div>
</div>
{{#notification}}
    {{> core/notification }}
{{/notification}}
