<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

namespace gradepenalty_duedate;

/**
 * Constants for the gradepenalty_duedate plugin.
 *
 * @package     gradepenalty_duedate
 * @copyright   2025 Catalyst IT Australia
 * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class constants {
    /**
     * @var int The minimum overdueby value.
     */
    public const OVERDUEBY_MIN = 1;

    /**
     * @var int The maximum overdueby value.
     */
    public const OVERDUEBY_MAX = YEARSECS;

    /**
     * @var int The minimum penalty value.
     */
    public const PENALTY_MIN = 0;

    /**
     * @var int The maximum penalty value.
     */
    public const PENALTY_MAX = 100;
}
