{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_drawer

    This template will render the message drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {}

}}
{{< core/drawer}}
    {{$tabindex}}-1{{/tabindex}}
    {{$role}}dialog{{/role}}
    {{$drawerattributes}}
        aria-modal="true"
        aria-labelledby="message-drawer-header-{{uniqid}}"
    {{/drawerattributes}}
    {{$drawercontent}}
        <div id="message-drawer-{{uniqid}}" class="message-app" data-region="message-drawer" role="region" tabindex="0">
            <h2 class="visually-hidden" id="message-drawer-header-{{uniqid}}">{{#str}} messagingcategory,admin {{/str}}</h2>
            <div class="closewidget text-end pe-2">
                <a class="text-dark btn-link" data-action="closedrawer" href="#"
                   title="{{#cleanstr}} closebuttontitle {{/cleanstr}}" aria-label="{{#cleanstr}} closebuttontitle {{/cleanstr}}"
                >
                    {{#pix}} i/window_close, core {{/pix}}
                </a>
            </div>
            <div class="header-container position-relative" data-region="header-container">
                {{> core_message/message_drawer_view_contacts_header }}
                {{> core_message/message_drawer_view_conversation_header }}
                {{> core_message/message_drawer_view_overview_header }}
                {{> core_message/message_drawer_view_search_header }}
                {{> core_message/message_drawer_view_settings_header }}
            </div>
            <div class="body-container position-relative" data-region="body-container">
                {{> core_message/message_drawer_view_contact_body }}
                {{> core_message/message_drawer_view_contacts_body }}
                {{> core_message/message_drawer_view_conversation_body }}
                {{> core_message/message_drawer_view_group_info_body }}
                {{> core_message/message_drawer_view_overview_body }}
                {{> core_message/message_drawer_view_search_body }}
                {{> core_message/message_drawer_view_settings_body }}
            </div>
            <div class="footer-container position-relative" data-region="footer-container">
                {{> core_message/message_drawer_view_conversation_footer }}
                {{> core_message/message_drawer_view_overview_footer }}
            </div>
        </div>
    {{/drawercontent}}
{{/core/drawer}}

{{#js}}
require(['jquery', 'core_message/message_drawer'], function($, MessageDrawer) {
    var root = $('#message-drawer-{{uniqid}}');
    MessageDrawer.init(root, '{{uniqid}}', false);
});
{{/js}}
