<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component qbank_previewquestion, language 'en'
 *
 * @package    qbank_previewquestion
 * @copyright  2021 Catalyst IT Australia Pty Ltd
 * @author     Safat Shahin <safatshahin@catalyst-au.net>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['alwayslatest'] = 'Always latest';
$string['newerversion'] = 'This preview is using an older version of the question. {$a->restartbutton}';
$string['pluginname'] = 'Preview question';
$string['privacy:metadata'] = 'The Preview question question bank plugin does not store any personal data.';
// Tag related errors.
$string['tagclosebutton'] = 'Close';
$string['tagerror'] = 'No question was found with following tags: {$a}. Please change or remove tags filtering.';
$string['tagsnotfound'] = 'Tags not found';
// Form string(s).
$string['previewoptions'] = 'Preview options';
$string['questionversion'] = 'Question version';
$string['restartnow'] = 'Use latest version';
$string['theoptionsyouselectonlyaffectthepreview'] = 'These settings are for testing the question. The options you select only affect the preview.';
// Preview title.
$string['versiontitle'] = 'Version {$a}';
$string['versiontitlelatest'] = 'Version {$a} (latest)';

