{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_analytics/insights_report

    Template for the insights report.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * none

    Example context (json):
    {
        "modelname": "Not engaging courses",
        "noactions": {
            "message": "Users have not executed any actions on the generated insights.",
            "announce": "true"
        }
    }
}}

<div class="box">
    <h3>{{#str}}actionsexecutedbyusersfor, tool_analytics, {{modelname}}{{/str}}</h3>

    {{#contextselect}}
        <div class="mt-3">
            {{> core/single_select }}
        </div>
    {{/contextselect}}

    {{#noactions}}
        <div class="mt-3 mb-1">
            {{> core/notification_info}}
        </div>
    {{/noactions}}
    {{^noanalysables}}
        <div class="row">
            <div class="col-xl-6">
                {{{separatedchart}}}
            </div>
            <div class="col-xl-6">
                {{{groupedchart}}}
            </div>
        </div>
    {{/noanalysables}}
</div>
