{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template qbank_columnsortorder/action_handle

    Render a single action handle for customising a column.

    Example context (json):
    {
        "action": "move",
        "target": "",
        "title": "Move column A",
        "pixicon": "i/dragdrop",
        "pixcomponent": "core",
        "popup": true
    }
}}
<span class="qbank_columnsortorder-action-handle {{action}}" tabindex="0" role="button"{{#popup}} aria-haspopup="true"{{/popup}}
      data-action="{{action}}"
      data-target="{{target}}"
      {{#dragtype}}data-drag-type="{{dragtype}}" {{/dragtype}}
      title="{{title}}">
        {{#pixicon}}
            {{#pix}}{{pixicon}}, {{pixcomponent}}{{/pix}}
        {{/pixicon}}
        {{#icon}}
            <i class="fa fa-{{{icon}}} me-1" aria-hidden="true"></i>
        {{/icon}}
        {{#text}}
            {{text}}
        {{/text}}
</span>
