{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_data/view_footer

    The mod_data sticky footer content.

    Example context (json):
    {
        "selectall": "<button>Select all</button>",
        "deleteselected": "<button>Cancel selection</button>",
        "pagination": "1, 2, 3...",
        "addentrybutton": {
            "disabled": false,
            "url": "#",
            "id": "test-id",
            "classes": "btn btn-link",
            "attributes": [
                {
                    "name": "title",
                    "value": "Add entry"
                }
            ],
            "text": "Add"
        }
    }
}}
{{#selectall}}
<div class="navitem">
    {{{selectall}}}
    {{{deleteselected}}}
</div>
{{/selectall}}
<div class="navitem submit mx-auto">
    {{{pagination}}}
</div>
{{#addentrybutton}}
<div class="navitem">
    {{> core/action_link}}
</div>
{{/addentrybutton}}
