H5PEditor.language.core = {
  missingTranslation: '[Vertaling ontbreekt :key]',
  loading: 'Laden, wachten alstublieft ...',
  selectLibrary: 'Selecteer de library die je wilt gebruiken voor je content.',
  unknownFieldPath: 'Kon ":path" niet vinden.',
  notImageField: '":path" is geen afbeelding.',
  notImageOrDimensionsField: '":path" is geen afbeelding of een veld voor afmetingen.',
  requiredProperty: 'De :property is vereist en moet een waarde hebben.',
  onlyNumbers: 'De :property waarde mag alleen cijfers bevatten.',
  illegalDecimalNumber: ':property can only contain numbers with max :decimals decimals.',
  exceedsMax: 'De :property waarde is hoger dan het maximum :max.',
  listExceedsMax: 'De lijst is groter dan het maximum van :max items.',
  belowMin: 'De :property waarde is lager dan het minimum :min.',
  listBelowMin: 'De lijst moet ten minste :min items bevatten voor het correct werken van de content.',
  outOfStep: 'De :property waarde kan alleen worden gewijzigd in stappen van :step.',
  add: 'Toevoegen',
  addFile: 'Voeg bestand toe',
  removeFile: 'Verwijder bestand',
  confirmRemoval: 'Weet je zeker dat je dit :type wilt verwijderen?',
  removeImage: 'Verwijder afbeelding',
  confirmImageRemoval: 'Hiermee wordt je afbeelding verwijderd. Weet je zeker dat je wilt doorgaan?',
  changeFile: 'Wijzig bestand',
  changeLibrary: 'Content type wijzigen?',
  semanticsError: 'Semantics fout: :error',
  missingProperty: 'Veld :index mist :property eigenschap.',
  expandCollapse: 'Uitvouwen/Invouwen',
  addEntity: 'Voeg :entity toe',
  tooLong: 'Veldwaarde is te lang, mag :max letters of minder bevatten.',
  invalidFormat: 'Veldwaarde bevat een ongeldig format of karakters die verboden zijn.',
  confirmChangeLibrary:
    'Hierdoor verlies je al het werk dat je met het huidige content type hebt gedaan. Weet je zeker dat je het content type wilt wijzigen?',
  commonFields: 'Tekst overschrijven en vertalingen',
  commonFieldsDescription: 'Hier kun je instellingen bewerken of in dit content type gebruikte teksten vertalen.',
  uploading: 'Uploaden, wachten alstublieft ...',
  noFollow: 'Kan veld ":path" niet volgen.',
  editCopyright: 'Copyright bewerken',
  close: 'Sluiten',
  tutorial: 'Tutorial beschikbaar',
  editMode: 'Bewerkmodus',
  listLabel: 'Lijst',
  uploadError: 'Fout bij uploaden bestand',
  fileToLarge: 'Het bestand dat je probeert te uploaden kan te groot zijn.',
  unknownFileUploadError: 'Onbekende fout bij uploaden bestand',
  noSemantics: 'Fout, kon het content type formulier niet laden.',
  editImage: 'Afbeelding bewerken',
  saveLabel: 'Opslaan',
  cancelLabel: 'Annuleren',
  resetToOriginalLabel: 'Terugzetten naar oorspronkelijk',
  loadingImageEditor: 'Afbeeldingseditor laden, wachten alstublieft ...',
  selectFiletoUpload: 'Selecteer bestand om te uploaden',
  or: 'of',
  enterAudioUrl: 'Voer audio bron URL in',
  enterVideoUrl: 'Enter video URL',
  enterAudioTitle: 'Plak link of andere URL van audiobron',
  enterVideoTitle: 'Plak YouTube-link of andere URL van videobron',
  uploadAudioTitle: 'Upload audiobestand',
  uploadVideoTitle: 'Upload videobestand',
  addVideoDescription:
    'H5P supports all external video sources formatted as mp4, webm or ogv, like Vimeo Pro, and has support for YouTube and Panopto links.',
  insert: 'Invoegen',
  cancel: 'Annuleren',
  height: 'hoogte',
  width: 'breedte',
  textField: 'tekstveld',
  numberField: 'getalveld',
  orderItemUp: 'Oplopend sorteren',
  orderItemDown: 'Aflopend sorteren',
  expandAllContent: 'Expand all content',
  collapseAllContent: 'Collapse all content',
  removeItem: 'Item verwijderen',
  hubPanelLabel: 'Selecteer content type',
  importantInstructions: 'Belangrijke instructies',
  showImportantInstructions: 'Toon instructies',
  hideImportantInstructions: 'Verberg belangrijke instructies',
  hide: 'Verberg',
  example: 'Voorbeeld',
  createContentTabLabel: 'Creëer inhoud',
  uploadTabLabel: 'Uploaden',
  uploadPlaceholder: 'Geen bestand gekozen',
  uploadInstructionsTitle: 'Een H5P bestand uploaden.',
  uploadInstructionsContent:
    'Je kunt beginnen met voorbeelden van <a href="https://h5p.org/content-types-and-applications" target="blank">H5P.org</a>.',
  uploadFileButtonLabel: 'Upload een bestand',
  uploadFileButtonChangeLabel: 'Wijzig bestand',
  uploadingThrobber: 'Uploaden...',
  uploadSuccess: ':title is met succes geüpload!',
  unableToInterpretError: 'Kon reactie niet interpreteren.',
  unableToInterpretSolution: 'Controleer je foutenlog.',
  h5pFileWrongExtensionTitle: 'Het geselecteerde bestand kon niet worden geüpload',
  h5pFileWrongExtensionContent: 'Alleen bestanden met de .h5p extensie toegestaan.',
  h5pFileValidationFailedTitle: 'Kon het H5P bestand niet valideren.',
  h5pFileValidationFailedContent:
    'Zorg ervoor dat de geüploade H5P geldige H5P content bevat. H5P bestanden die alleen bibliotheken bevatten, dienen te worden geüpload via de H5P Bibliotheken pagina.',
  h5pFileUploadServerErrorTitle: 'Het H5P bestand kon niet worden geüpload',
  h5pFileUploadServerErrorContent:
    'Er is een onverwachte fout opgetreden. Controleer het server log voor meer informatie.',
  contentTypeSectionAll: 'Alle Content Types',
  searchResults: 'Zoekresultaten',
  contentTypeSearchFieldPlaceholder: 'Zoeken naar Content Types',
  contentTypeInstallButtonLabel: 'Installeren',
  contentTypeInstallingButtonLabel: 'Bezig met installeren',
  contentTypeUseButtonLabel: 'Gebruik',
  contentTypeDetailButtonLabel: 'Details',
  contentTypeUpdateButtonLabel: 'Updaten',
  contentTypeUpdatingButtonLabel: 'Bezig met updaten',
  contentTypeGetButtonLabel: 'Ophalen',
  contentTypeBackButtonLabel: 'Terug',
  contentTypeIconAltText: 'Icoon',
  contentTypeInstallSuccess: ':contentType succesvol geïnstalleerd!',
  contentTypeUpdateSuccess: ':contentType succesvol geüpdatet!',
  contentTypeInstallError: ':contentType kon niet worden geïnstalleerd. Neem contact op met je site beheerder.',
  contentTypeLicensePanelTitle: 'Licentie',
  contentTypeDemoButtonLabel: 'Content Demo',
  numResults: ':num resultaten',
  show: 'Toon',
  recentlyUsedFirst: 'Recent gebruikt eerst',
  popularFirst: 'Populairste eerst',
  newestFirst: 'Nieuwste eerst',
  aToZ: 'A tot Z',
  noResultsFound: 'Geen resultaten gevonden',
  noResultsFoundDesc: 'Er is geen content type dat overeenkomt met je zoekcriteria.',
  readMore: 'Meer lezen',
  readLess: 'Minder lezen',
  contentTypeOwner: 'Door :owner',
  contentTypeUnsupportedApiVersionTitle: 'Dit content type vereist een nieuwere core versie',
  contentTypeUnsupportedApiVersionContent: 'Neem contact op met je site beheerder voor de benodigde updates',
  contentTypeUpdateAvailable: 'Update beschikbaar',
  contentTypeRestricted: 'Beperkt content type',
  contentTypeRestrictedDesc: 'Het gebruik van dit content type is beperkt door een site beheerder.',
  contentTypeNotInstalled: 'Content type niet geïnstalleerd',
  contentTypeNotInstalledDesc: 'Je hebt geen toestemming om content types te installeren.',
  theContentType: 'het content type',
  currentMenuSelected: 'huidige selectie',
  errorCommunicatingHubTitle: 'Kan niet communiceren met hub.',
  errorCommunicatingHubContent: 'Er is een fout opgetreden. Probeer het opnieuw.',
  warningNoContentTypesInstalled: 'Je hebt geen geïnstalleerde content types.',
  warningChangeBrowsingToSeeResults:
    'Klik <em>Alle</em> om de lijst op te halen van alle content types die je kunt installeren.',
  warningUpdateAvailableTitle: 'Er is een nieuwe versie van :contentType beschikbaar.',
  warningUpdateAvailableBody: 'Update naar de laatste versie voor een verbeterde ervaring.',
  licenseDescription:
    'Sommige van de kenmerken van deze licentie zijn hieronder weergegeven. Klik op het info icoon hierboven om de oorspronkelijke tekst van de licentie te lezen.',
  licenseModalTitle: 'Details van licentie',
  licenseModalSubtitle: 'Selecteer een licentie om informatie te krijgen over het correcte gebruik ervan',
  licenseUnspecified: 'Niet gespecificeerd',
  licenseCanUseCommercially: 'Mag commercieel gebruiken',
  licenseCanModify: 'Mag aanpassen',
  licenseCanDistribute: 'Mag distribueren',
  licenseCanSublicense: 'Mag sublicenties verstrekken',
  licenseCanHoldLiable: 'Mag aansprakelijk houden',
  licenseCannotHoldLiable: 'Mag niet aansprakelijk houden',
  licenseMustIncludeCopyright: 'Moet copyright bijvoegen',
  licenseMustIncludeLicense: 'Moet licentie bijvoegen',
  licenseFetchDetailsFailed: 'Details van licentie ophalen mislukt',
  imageLightboxTitle: 'Afbeeldingen',
  imageLightBoxProgress: ':num van :total',
  nextImage: 'Volgende afbeelding',
  previousImage: 'Vorige afbeelding',
  screenshots: 'Screenshots',
  reloadButtonLabel: 'Opnieuw laden',
  videoQuality: 'Videokwaliteitslabel',
  videoQualityDescription:
    'Dit label helpt de gebruiker bij het vaststelllen van de huidige kwaliteit van de video. Bijv. 1080p, 720p, HD of Mobile',
  videoQualityDefaultLabel: 'Kwaliteit :index',
  noContentTypesAvailable: 'Geen content types beschikbaar',
  noContentTypesAvailableDesc:
    'Je site heeft problemen bij het verbinden met H5P.org en het tonen van de beschikbare content types.',
  contentTypeCacheOutdated: 'Content types lijst is verlopen',
  contentTypeCacheOutdatedDesc:
    'Je site heeft problemen bij het verbinden met H5P.org om te controleren op updates voor content types. Het kan zijn dat je geen content types kunt updaten of installeren.',
  tryAgain: 'Probeer nogmaals',
  getHelp: 'Vraag hulp',
  untitled: 'Zonder titel :libraryTitle',
  title: 'Titel',
  metadata: 'Metadata',
  addTitle: 'Voeg titel toe',
  usedForSearchingReportsAndCopyrightInformation: 'Gebruikt voor zoeken, rapportage en copyright informatie',
  metadataSharingAndLicensingInfo: 'Metadata (deel- en licentie-info)',
  fillInTheFieldsBelow: 'Vul de velden hieronder in',
  saveMetadata: 'Bewaar metadata',
  addAuthor: 'Bewaar auteur',
  confirmRemoveAuthor: 'Weet je zeker dat je deze auteur wilt verwijderen?',
  addNewChange: 'Voeg nieuwe wijziging toe',
  confirmDeleteChangeLog: 'Weet je zeker dat je deze invoer van het wijzigingslog wilt verwijderen?',
  changelogDescription:
    'Sommige licenties vereisen dat wijzigingen aan het originele werk, of afgeleiden daarvan, worden gelogd en getoond. U kunt hier uw wijzigingen vanwege licentieredenen loggen of alleen om uzelf en anderen te informeren over wijzigingen aan deze inhoud.',
  logThisChange: 'Log deze wijziging',
  newChangeHasBeenLogged: 'Nieuwe wijziging is gelogd',
  loggedChanges: 'Gelogde wijzigingen',
  noChangesHaveBeenLogged: 'Er zijn geen wijzigingen gelogd',
  errorHeader: 'Er is een fout opgetreden',
  errorCalculatingMaxScore:
    'Kon de maximumscore voor deze inhoud niet berekenen. De maximumscore lijkt 0 te zijn. Neem contact op met uw beheerder als dit niet juist is.',
  maxScoreSemanticsMissing: 'Kon de verwachte semantics in de content niet vinden.',
  copyButton: 'Kopiëren',
  copiedButton: 'Gekopieerd',
  pasteButton: 'Plakken',
  pasteAndReplaceButton: 'Plak en vervang',
  pasteContent: 'Plak inhoud',
  confirmPasteContent:
    'Hiermee vervang je de huidige inhoud door de inhoud van het klembord. De huidige inhoud gaat verloren. Weet je zeker dat je wilt doorgaan?',
  confirmPasteButtonText: 'Vervang inhoud',
  copyToClipboard: 'Kopieer H5P inhoud naar het klembord',
  copiedToClipboard: 'Inhoud is gekopieerd naar het klembord',
  pasteFromClipboard: 'Plak H5P inhoud van het klembord',
  pasteAndReplaceFromClipboard: 'Vervang bestaande inhoud met H5P inhoud van het klembord',
  pasteNoContent: 'Geen H5P inhoud op het klembord',
  pasteError: 'Kan niet plakkken van klembord',
  pasteContentNotSupported: 'De inhoud in het H5P klembord wordt niet ondersteund in deze context',
  pasteContentRestricted: 'De inhoud in het H5P klembord bevat afgeschermde content',
  pasteTooOld:
    'De inhoud in het H5P klembord is van een lagere versie (:clip) dan wat wordt ondersteund in deze context (:local), probeer indien mogelijk de inhoud die u wilt plakken te upgraden, kopieer het opnieuw en plak het hier opnieuw.',
  pasteTooNew:
    'De inhoud in het H5P klembord is van een hogere versie (:clip) dan wat wordt ondersteund in deze context (:local), probeer indien mogelijk deze content eerst te upgraden, en probeer dan opnieuw de inhoud hier te plakken.',
  ok: 'OK',
  avTablistLabel: 'Invoegen m.b.v.',
  tabTitleBasicFileUpload: 'Bestandsupload',
  tabTitleInputLinkURL: 'Link/URL',
  errorTooHighVersion: 'Parameters bevat %used terwijl alleen %supported of vroeger wordt ondersteund.',
  errorNotSupported: 'Parameters bevat %used wat niet wordt ondersteund.',
  errorParamsBroken: 'Parameters zijn corrupt.',
  libraryMissing: 'Ontbrekende vereiste library %lib.',
  scriptMissing: 'Kon upgrade-script voor %lib niet laden.',
  language: 'Taal',
  noLanguagesSupported: 'Geen taal ondersteuning',
  changeLanguage: 'Wijzig taal naar :language?',
  thisWillPotentially:
    'Mogelijk stelt dit alle tekst en vertalingen opnieuw in. Je kunt dit niet ongedaan maken. De content zelf wordt niet gewijzigd. Wil je doorgaan?',
  notAllTextsChanged: 'Niet alle teksten zijn gewijzigd, er is slechts gedeeltelijke dekking voor :language.',
  contributeTranslations:
    'Als je de vertaling voor :language wilt completeren, kun je hier meer leren over <a href=":url" target="_new">het bijdragen van vertalingen aan H5P</a>',
  unknownLibrary: "Jammer genoeg is het geselecteerde content type '%lib' niet geïnstalleerd op dit systeem.",
  proceedButtonLabel: 'Doorgaan naar opslaan',
  enterFullscreenButtonLabel: 'In volledig scherm werken',
  exitFullscreenButtonLabel: 'Volledig scherm afsluiten',
  a11yTitleShowLabel: 'Toon label voor AT',
  a11yTitleHideLabel: 'Verberg label voor AT',
  reuseSuccess: ':title is met succes geïmporteerd van de H5P Hub.',
  noContentHeader: 'No Fitting Content?',
  noContentSuggestion: 'Create one yourself!',
  tutorials: 'Tutorials',
  contentSectionAll: 'All shared content',
  popularContent: 'Popular Content',
  allPopular: 'All Popular',
  newOnTheHub: 'New on the Hub',
  allNew: 'All New',
  filterBy: 'Filter by',
  filter: 'Filter',
  filters: {
    level: {
      dropdownLabel: 'Level',
      dialogHeader: 'Select level of education',
      dialogButtonLabel: 'Filter level of education',
    },
    language: {
      dropdownLabel: 'Language',
      dialogHeader: 'Select language(s)',
      dialogButtonLabel: 'Filter languages',
      searchPlaceholder: 'Type to search for languages',
    },
    reviewed: {
      dropdownLabel: 'Reviewed',
      dialogHeader: 'Reviewed Content',
      dialogButtonLabel: 'Filter',
      optionLabel: 'Show only reviewed content',
    },
    contentTypes: {
      dropdownLabel: 'Content types',
      dialogHeader: 'Select Content type(s)',
      dialogButtonLabel: 'Filter Content Types',
      searchPlaceholder: 'Type to search for content types',
    },
    disciplines: {
      dropdownLabel: 'Discipline',
      dialogHeader: 'Select Your Discipline',
      dialogButtonLabel: 'Filter Disciplines',
      searchPlaceholder: 'Type to search for disciplines',
    },
    licenses: {
      dropdownLabel: 'License',
      dialogHeader: 'Select preferred rights of use',
      dialogButtonLabel: 'Filter licenses',
      options: { modified: 'Can be modified', commercial: 'Allows commercial use' },
    },
  },
  clearFilters: 'Clear all filters',
  contentSearchFieldPlaceholder: 'Search for Content',
  loadingContentTitle: 'We are loading content for you...',
  loadingContentSubtitle: 'Please wait',
  by: 'By',
  dropdownButton: 'Open dropdown',
  paginationNavigation: 'Pagination navigation',
  page: 'Page',
  currentPage: 'Current page',
  nextPage: 'Go to next page',
  previousPage: 'Go to previous page',
  contentPreviewButtonLabel: 'Preview',
  contentDownloadButtonLabel: 'Get Content',
  reuseContentTabLabel: 'Get Shared Content',
  contentPublisherPanelHeader: 'Publisher Info',
  noContentFoundDesc: 'There is no content that matches your search criteria.',
  h5pType: 'H5P Type',
  level: 'Level',
  size: 'Size',
  failedFetchingData: 'Failed fetching data',
  filterErrorMessage: 'Something went wrong. Please reload the page.',
  in: 'in',
  navigateToParent: 'Navigate to parent',
};
