{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_lti/repost_crosssite

    This template re-executes a cross site request but from the site domain, avoiding the cross site issue.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    *

    Example context (json):
    {
        "url":"https://some.tool.example/mod/lti/auth.php",
        "params": {
            "response_type": "id_token"
        }
    }

}}
<form action="{{url}}" method="POST" id="autopostme">
    {{#params}}
    <input type="hidden" name="{{key}}" value="{{value}}">
    {{/params}}
    <input type="hidden" name="repost" value="true">
</form>

{{#js}}
        document.getElementById("autopostme").submit();
{{/js}}
