{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/notification_preferences_component

    The notification preferences page

    Classes required for JS:
    * None

    Data attibutes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * displayname   The component display name
    * colspan       The number of columns the row should span
    * hasnotifications Whether any notifications exist
    * notifications The list of notifications

    Example context (json):
    {
        "displayname": "System",
        "colspan": 2,
        "hasnotifications": 1,
        "notifications": [
            {
                "displayname": "Notices about minor problems",
                "preferencekey": "message_provider_moodle_notices",
                "processors": [
                    {
                        "displayname": "Popup notification",
                        "name": "popup",
                        "locked": 0,
                        "userconfigured": 1,
                        "enabled": 0,
                        "enabledlabel": "Sending Assignment enabled status"
                    }
                ]
            }
        ]
    }
}}
{{#hasnotifications}}
    <tr>
        <th colspan="{{{colspan}}}">{{{displayname}}}</th>
    </tr>
    {{#notifications}}
        {{> message/notification_preferences_component_notification }}
    {{/notifications}}
{{/hasnotifications}}
