<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'report_event', language 'en', branch 'MOODLE_27_STABLE'
 *
 * @package    report_eventlist
 * @copyright  2014 Adrian Greeve <adrian@moodle.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['abstractclass'] = 'Abstract class';
$string['action'] = 'Action';
$string['affectedtable'] = 'Affected table';
$string['all'] = 'All';
$string['clear'] = 'Clear';
$string['component'] = 'Component';
$string['core'] = 'Core';
$string['coresubsystem'] = 'Subsystem ({$a})';
$string['create'] = 'create';
$string['crud'] = 'Database query type';
$string['delete'] = 'delete';
$string['details'] = 'Details';
$string['dname'] = 'Name';
$string['edulevel'] = 'Education level';
$string['errorinvalidevent'] = 'The event provided is not a valid event.';
$string['errorinvaliddirectory'] = 'The event directory does not exist or is invalid.';
$string['eventcode'] = 'Event Code';
$string['eventexplanation'] = 'Explanation of the event';
$string['eventname'] = 'Event name';
$string['filter'] = 'Filter';
$string['legacyevent'] = 'Legacy event';
$string['name'] = 'Name';
$string['objecttable'] = 'Object table';
$string['other'] = 'Other';
$string['otherinformation'] = 'Other information:';
$string['othereventparameters'] = 'Other event parameters';
$string['parentevent'] = 'Parent Event';
$string['participating'] = 'Participating';
$string['pluginname'] = 'Events list';
$string['read'] = 'read';
$string['relatedobservers'] = 'Plugins observing this event';
$string['since'] = 'Since';
$string['teaching'] = 'Teaching';
$string['typedeclaration'] = 'Other event parameters';
$string['update'] = 'update';
$string['yes'] = 'yes';
$string['privacy:metadata'] = 'The Events list plugin does not store any personal data.';
