{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_policy/page_nopermission

    Template for showing to the user the policy docs to agree.

    Classes required for JS:
    * policyactions

    Data attributes required for JS:
    -

    Context variables required for this template:
    * pluginbaseurl
    * haspermissionagreedocs - status of the capability of the user to accept policy documents
    * messagetitle - Message title to display, with the error.
    * messagedesc - Message description to display, with the error.
    * supportname - name of the support contact; displayed when the user does not have permission to accept policy documents
    * supportemail - email of the support contact; displayed when the user does not have permission to accept policy documents
    * behalfuser - If behalfid is defined and valid, full name of the behalf user with a link to his/her profile; null otherwise
    * policies - Array with all the links to current policies (for showing them if needed).

    Example context (json):
    {
        "haspermissionagreedocs": false,
        "messagetitle": "You don't have permission",
        "messagedesc": "Sorry you don't have permission to accept",
        "policies": [
            "Policy 1",
            "Policy 2"
        ],
        "supportname": "Admin",
        "supportemail": "moodlesite/user/contactsitesupport.php"
    }
}}

{{^haspermissionagreedocs}}
    <h3>{{{messagetitle}}}</h3>
    <p class="mt-1 mb-1">{{{messagedesc}}}</p>

    <div class="policies">
        <ul>
        {{# policies }}
            <li>{{{.}}}</li>
        {{/ policies }}
        </ul>
    </div>

    {{#supportemail}}
        <p>{{#str}}nopermissiontoagreedocscontact, tool_policy{{/str}}</p>
        <p>{{{supportemail}}}</p>
    {{/supportemail}}
{{/haspermissionagreedocs}}

{{#js}}
// Initialise the JS for the modal window which displays the policy versions.
require(['tool_policy/policyactions'], function(ActionsMod) {
    ActionsMod.init('[data-action="view"]');
});
{{/js}}
