{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_grades/weight_field

    Weight field.

    Example context (json):
    {
        "itemname": "Awesome test",
        "id": "2",
        "value": "100"
    }
}}
<label class="accesshide" for="weight_{{id}}">
    {{#str}}extracreditvalue, grades, {{itemname}}{{/str}}
</label>
<input type="text" name="weight_{{id}}" id="weight_{{id}}" value="{{value}}" size="4" class="form-control">
