{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_user/participantsfilter

    Template for the form containing one or more filter rows.

    Example context (json):
    {
        "filtertypes": [
            {
                "name": "status",
                "title": "Status",
                "values": [
                    {
                        "value": 1,
                        "title": "Active"
                    },
                    {
                        "value": 0,
                        "title": "Suspended"
                    }
                ]
            }
        ]
    }
}}
<form method="dialog">
{{> core/datafilter/filter }}
</form>

{{#js}}
    require(['core_user/participants_filter'], function(ParticipantsFilter) {
        ParticipantsFilter.init('core-filter-{{uniqid}}');
    });
{{/js}}
