<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'course', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package   core_courseformat
 * @copyright 2021 Ferran Recio <ferran@moodle.com>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['addsection'] = 'Add section';
$string['availability_show'] = 'Show on course page';
$string['availability_show_help'] = 'Available to students (subject to any access restrictions which may be set).';
$string['availability_hide'] = 'Hide on course page';
$string['availability_hide_help'] = 'Not available to students.';
$string['availability_stealth'] = 'Make available but don\'t show on course page';
$string['availability_stealth_help'] = 'Available to students if you provide a link. Activities will still appear in the gradebook and other reports.';
$string['bulkedit'] = 'Bulk actions';
$string['bulkeditingcourse'] = 'Bulk edit course: {$a}';
$string['bulkcancel'] = 'Close bulk actions';
$string['bulkselection'] = '{$a} selected';
$string['bulkselection_plural'] = '{$a} selected';
$string['cm_add_feedback'] = '{$a->modname} added.';
$string['cm_complete_feedback'] = '{$a->name} marked as done.';
$string['cm_complete_feedback_batch'] = 'Selected {$a->count} activities marked as done.';
$string['cm_delete_feedback'] = '{$a->name} deleted.';
$string['cm_delete_feedback_batch'] = 'Selected {$a->count} activities deleted.';
$string['cm_duplicate_feedback'] = '{$a->name} duplicated.';
$string['cm_duplicate_feedback_batch'] = 'Selected {$a->count} activities duplicated.';
$string['cm_hide_feedback'] = '{$a->name} is now hidden.';
$string['cm_hide_feedback_batch'] = 'The selected {$a->count} activities are now hidden.';
$string['cm_moveleft_feedback'] = '{$a->name} moved left.';
$string['cm_moveright_feedback'] = '{$a->name} moved right.';
$string['cm_nogroups_feedback'] = '{$a->name} group mode changed to No groups.';
$string['cm_separategroups_feedback'] = '{$a->name} group mode changed to Separate groups.';
$string['cm_show_feedback'] = '{$a->name} is now shown.';
$string['cm_show_feedback_batch'] = 'The selected {$a->count} activities are now shown.';
$string['cm_stealth_feedback'] = '{$a->name} is now available but not shown on the course page.';
$string['cm_stealth_feedback_batch'] = 'The selected {$a->count} activities are now available but not shown on the course page.';
$string['cm_uncomplete_feedback'] = '{$a->name} marked as undone.';
$string['cm_uncomplete_feedback_batch'] = 'Selected {$a->count} activities marked as undone.';
$string['cm_visiblegroups_feedback'] = '{$a->name} group mode changed to Visible groups.';
$string['cmavailability'] = 'Activity availability';
$string['cmdelete_info'] = 'This will delete {$a->name} and any user data it contains.';
$string['cmdelete_subsectiontitle'] = 'Delete subsection?';
$string['cmdelete_title'] = 'Delete activity?';
$string['cmmove_info'] = 'Move {$a} after';
$string['cmmove_subsectiontitle'] = 'Move subsection';
$string['cmmove_title'] = 'Move activity';
$string['cmsdelete'] = 'Delete activities';
$string['cmsdelete_info'] = 'This will delete {$a->count} activities and any user data they contain.';
$string['cmsdelete_title'] = 'Delete selected activities?';
$string['cmsduplicate'] = 'Duplicate activities';
$string['cmsmove'] = 'Move activities';
$string['cmsmove_title'] = 'Move selected activities';
$string['cmsmove_info'] = 'Move {$a} activities after';
$string['courseindex'] = 'Course index';
$string['courseindexoptions'] = 'Course index options';
$string['maxsectionaddmessage'] = 'You have reached the maximum number of sections allowed for a course.';
$string['nobulkaction'] = 'No bulk actions available';
$string['orphansectionwarning'] = 'This section and its content are not part of the course structure and are not visible to students. To use any of this content, move it to a different section.';
$string['preference:coursesectionspreferences'] = 'Section user preferences for course {$a}';
$string['privacy:metadata:preference:coursesectionspreferences'] = 'Section user preferences like collapsed and expanded.';
$string['section_add_feedback'] = 'Section added.';
$string['section_hide_feedback'] = 'Course section {$a->name} hidden.';
$string['section_hide_feedback_batch'] = 'Selected {$a->count} course sections hidden.';
$string['section_show_feedback'] = 'Course section {$a->name} shown.';
$string['section_show_feedback_batch'] = 'Selected {$a->count} course sections shown.';
$string['section_delete_feedback'] = 'Course section {$a->name} deleted.';
$string['section_delete_feedback_batch'] = 'Selected {$a->count} course sections deleted.';
$string['sectionaddmax'] = 'You have reached the maximum number of sections allowed for a course...';
$string['sectionavailability_title'] = 'Section availability';
$string['sectiondelete_info'] = 'This will delete {$a->name} and all the activities it contains.';
$string['sectiondelete_title'] = 'Delete section?';
$string['sectionsavailability'] = 'Sections availability';
$string['sectionsavailability_title'] = 'Sections availability';
$string['sectionsdelete'] = 'Delete sections';
$string['sectionmove_info'] = 'Move {$a} after';
$string['sectionmove_title'] = 'Move section';
$string['sectionsdelete_info'] = 'This will delete {$a->count} sections and all the activities they contain.';
$string['sectionsdelete_title'] = 'Delete selected sections?';
$string['sectionsmove'] = 'Move sections';
$string['sectionsmove_info'] = 'Move {$a} sections after';
$string['sectionsmove_title'] = 'Move selected sections';
$string['selectcm'] = 'Select activity {$a}';
$string['selectsection'] = 'Select section {$a}';
