// Import FonAwesome.

$fa-version: "6.3.0" !default;

@import "fontawesome/brands";
@import "fontawesome/regular";
@import "fontawesome/solid";
@import "fontawesome/v4-shims";
@import "fontawesome/fontawesome";
