{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_assign/override_delete_modal

    Example context (json):
    {
    }

}}
{{< core/modal }}
    {{$title}}{{#str}} confirm {{/str}}{{/title}}
    {{$body}}
        <div class="container">
            <form>
                <div class="form-group row">
                    <p>{{confirmmessage}}</p>
                </div>

                {{#showpenaltyrecalculation}}
                    <div class="form-group row">
                        <div class="form-check form-check-inline">
                            <input class="form-check-input" type="checkbox" id="recalculatepenalties" name="recalculatepenalties"/>

                            <label class="form-check-label" for="recalculatepenalties">
                                {{#str}} overriderecalculatepenalty, assign {{/str}}
                            </label>
                        </div>
                    </div>
                {{/showpenaltyrecalculation}}
            </form>
        </div>
    {{/body}}
    {{$footer}}
        <button type="button" class="btn btn-secondary" data-action="cancel">{{#str}} cancel {{/str}}</button>
        <button type="button" class="btn btn-danger" data-action="delete">{{#str}} continue {{/str}}</button>
    {{/footer}}
{{/ core/modal }}