{{!
    This file is part of Moodle - https://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_generator/parsingresult

    Display the result of parsing or executing a behat testing scenario.

    Example context (json):
    {
        "isvalid": true,
        "generalerror": "Error message",
        "haslines": true,
        "scenarios": [
            {
                "type": "Scenario",
                "name": "Scenario title",
                "hassteps": true,
                "steps": [
                    {
                        "text": "Step text",
                        "isvalid": true,
                        "isexecuted": true,
                        "hasarguments": true,
                        "arguments": "Argument text",
                        "error": "Error message"
                    }
                ]
            }
        ]
    }
}}
{{^isvalid}}
<div class="alert alert-danger mb-3" role="alert">
    <span class="fa fa-times"></span>
    {{generalerror}}
</div>
{{/isvalid}}
{{#haslines}}
    <p>{{#str}} testscenario_steps, tool_generator {{/str}}</p>
{{/haslines}}
{{#scenarios}}
<h3>{{type}}{{#name}}: {{name}} {{/name}}</h3>
<div>
    {{#scenarioerror}}
    <div class="alert alert-danger mb-3" role="alert">
        <span class="fa fa-times"></span>
        {{scenarioerror}}
    </div>
    {{/scenarioerror}}
    {{^hassteps}}
    <div class="alert alert-info mb-3" role="alert">
        <span class="fa fa-info-circle"></span>
        {{#str}} testscenario_scenarionosteps, tool_generator {{/str}}
    </div>
    {{/hassteps}}
    {{#hassteps}}
    <ul class="list-group mb-3">
        {{#steps}}
        <li class="list-group-item">
            <div>
                {{text}}
                {{^isvalid}}
                <span class="badge rounded-pill text-bg-danger">
                    <span class="fa fa-times"></span>
                    {{#str}} error {{/str}}
                </span>
                {{/isvalid}}
                {{#isvalid}}
                    {{#isexecuted}}
                    <span class="badge rounded-pill text-bg-success">
                        <span class="fa fa-check "></span>
                        {{#str}} success {{/str}}
                    </span>
                    {{/isexecuted}}
                {{/isvalid}}
            </div>
            {{#hasarguments}}
            <div class="alert alert-info mb-0 mt-2" role="alert" style="overflow: auto;">
                <pre class="mb-0">{{arguments}}</pre>
            </div>
            {{/hasarguments}}
            {{^isvalid}}
            <div class="alert alert-danger mb-0 mt-2" role="alert" style="overflow: auto;">
                {{error}}
            </div>
            {{/isvalid}}
        </li>
        {{/steps}}
    </ul>
    {{/hassteps}}
</div>
{{/scenarios}}
