#page-admin-tool-langimport-index .langimport {
    margin: auto;
    float: none;
    width: 100%;
}

#page-admin-tool-langimport-index #menuuninstalllang,
#page-admin-tool-langimport-index #menupack {
    height: 300px;
}
