H5PEditor.language.core = {
  missingTranslation: '[Mangler oversettelse :key]',
  loading: 'Laster, vennligst vent...',
  selectLibrary: 'Velg biblioteket du ønsker å bruke for innholdet ditt.',
  unknownFieldPath: 'Kan ikke finne ":path".',
  notImageField: '":path" er ikke et bilde.',
  notImageOrDimensionsField: '":path" er verken et bilde- eller dimensjonsfelt.',
  requiredProperty: '":property" er påkrevd og må ha en verdi.',
  onlyNumbers: '":property" kan bare innholde tall.',
  illegalDecimalNumber: ':property kan bare inneholde tall med maksimalt :decimals desimaler.',
  exceedsMax: '":property" overstiger maksverdien på :max.',
  listExceedsMax: 'Antallet elementer i listen overstiger maksantallet på :max elementer.',
  belowMin: '":property" er mindre enn minimumsverdien på :min.',
  listBelowMin: 'Listen trenger minst :min elementer for at innholdet skal virke.',
  outOfStep: '":property" kan bare endres i steg på :step.',
  add: 'Legg til',
  addFile: 'Legg til fil',
  removeFile: 'Fjern fil',
  confirmRemoval: 'Er du sikker på at du ønsker å fjerne denne :type?',
  removeImage: 'Fjern bilde',
  confirmImageRemoval: 'Er du sikker på at du ønsker å fjerne dette bildet?',
  changeFile: 'Bytt fil',
  changeLibrary: 'Endre innholdstype',
  semanticsError: 'Semantikkfeil: :error',
  missingProperty: 'Feltet :index mangler :property attributten sin.',
  expandCollapse: 'Utvid/Slå sammen',
  addEntity: 'Legg til :entity',
  tooLong: 'Feltets verdi er for lang, den må være på :max tegn eller mindre.',
  invalidFormat: 'Feltets verdi har ugyldig format eller bruker ulovlige tegn.',
  confirmChangeLibrary:
    'Ved å gjøre dette mister du alt arbeid gjort med nåværende innholdstype. Er du sikker på at du ønsker å bytte innholdstype?',
  commonFields: 'Innstillinger og tekster',
  commonFieldsDescription: 'Her kan du redigere innstillinger eller oversette tekster som brukes i dette innholdet.',
  uploading: 'Laster opp fil, vennligst vent...',
  noFollow: 'Kunne ikke følge feltet ":path".',
  editCopyright: 'Rediger opphavsrett',
  close: 'Lukk',
  tutorial: 'Veiledning',
  editMode: 'Redigeringsmodus',
  listLabel: 'Liste',
  uploadError: 'Filopplasting feilet',
  fileToLarge: 'Filen du prøver å laste opp kan være for stor.',
  unknownFileUploadError: 'Ukjent filopplastingsfeil',
  noSemantics: 'Feil: kunne ikke laste skjemaet for innholdstypen.',
  editImage: 'Rediger bilde',
  saveLabel: 'Lagre',
  cancelLabel: 'Avbryt',
  resetToOriginalLabel: 'Tilbakestill bilde',
  loadingImageEditor: 'Laster inn bilderedigering, vennligst vent...',
  selectFiletoUpload: 'Velg fil som skal lastes opp',
  or: 'eller',
  enterAudioUrl: 'Skriv inn nettadresse til lydkilde',
  enterVideoUrl: 'Skriv inn nettadresse til video',
  enterAudioTitle: 'Lim inn link eller annen lydkilde URL',
  enterVideoTitle: 'Lim inn YouTube lenke eller annen videokilde URL',
  uploadAudioTitle: 'Last opp lydfil',
  uploadVideoTitle: 'Last opp videofil',
  addVideoDescription:
    'H5P støtter alle eksterne videokilder på formatene mp4, webm eller ogv, slik som Vimeo Pro, og har støtte for YouTube-lenker og Panopto-lenker.',
  insert: 'Sett inn',
  cancel: 'Avbryt',
  height: 'Høyde',
  width: 'Bredde',
  textField: 'Tekstfelt',
  numberField: 'Nummerfelt',
  orderItemUp: 'Flytt element opp',
  orderItemDown: 'Flytt element ned',
  expandAllContent: 'Expand all content',
  collapseAllContent: 'Collapse all content',
  removeItem: 'Fjern element',
  hubPanelLabel: 'Velg innholdstype',
  importantInstructions: 'Viktige instruksjoner',
  showImportantInstructions: 'Vis instruksjoner',
  hideImportantInstructions: 'Skjul viktige instruksjoner',
  hide: 'Skjul',
  example: 'Eksempel',
  createContentTabLabel: 'Lag innhold',
  uploadTabLabel: 'Last opp',
  uploadPlaceholder: 'Ingen fil er valgt',
  uploadInstructionsTitle: 'Last opp en .h5p fil.',
  uploadInstructionsContent:
    'Du kan starte med eksempler fra <a href="https://h5p.org/content-types-and-applications" target="blank">H5P.org</a>.',
  uploadFileButtonLabel: 'Last opp en fil',
  uploadFileButtonChangeLabel: 'Bytt fil',
  uploadingThrobber: 'Laster opp...',
  uploadSuccess: ':title ble lastet opp!',
  unableToInterpretError: 'Kan ikke tolke respons.',
  unableToInterpretSolution: 'Vennligst sjekk feilloggen din.',
  h5pFileWrongExtensionTitle: '.h5p-fil ble ikke funnet.',
  h5pFileWrongExtensionContent: 'Du må laste opp en fil med fil-endelsen .h5p',
  h5pFileValidationFailedTitle: 'Kunne ikke validere H5P-filen.',
  h5pFileValidationFailedContent:
    'Pass på at .h5p filen du laster opp inneholder valid H5P innhold. H5P filer som bare inneholder biblioteker skal lastes opp på bibliotekssiden og kan ikke lastes opp som innhold.',
  h5pFileUploadServerErrorTitle: 'H5P-filen kunne ikke bli lastet opp.',
  h5pFileUploadServerErrorContent: 'En uventet feil har oppstått. Sjekk din serverlogg for detaljer.',
  contentTypeSectionAll: 'Alle innholdstyper',
  searchResults: 'Søkeresultater',
  contentTypeSearchFieldPlaceholder: 'Søk etter innholdstyper',
  contentTypeInstallButtonLabel: 'Installer',
  contentTypeInstallingButtonLabel: 'Installerer',
  contentTypeUseButtonLabel: 'Bruk',
  contentTypeDetailButtonLabel: 'Detaljer',
  contentTypeUpdateButtonLabel: 'Oppdater',
  contentTypeUpdatingButtonLabel: 'Oppdaterer',
  contentTypeGetButtonLabel: 'Hent',
  contentTypeBackButtonLabel: 'Tilbake',
  contentTypeIconAltText: 'Ikon',
  contentTypeInstallSuccess: ':contentType ble installert!',
  contentTypeUpdateSuccess: ':contentType ble oppdatert!',
  contentTypeInstallError: ':contentType kunne ikke bli installert. Kontakt administrator for siden.',
  contentTypeLicensePanelTitle: 'Lisens',
  contentTypeDemoButtonLabel: 'Innholdsdemonstrasjon',
  numResults: ':num treff',
  show: 'Vis',
  recentlyUsedFirst: 'Nylig brukt først',
  popularFirst: 'Populære først',
  newestFirst: 'Nyeste først',
  aToZ: 'A til Å',
  noResultsFound: 'Ingen resultater',
  noResultsFoundDesc: 'Det finnes ingen innholdstype som samsvarer med søkekriteriene dine.',
  readMore: 'Vis mer',
  readLess: 'Vis mindre',
  contentTypeOwner: 'Av :owner',
  contentTypeUnsupportedApiVersionTitle: 'Denne innholdstypen krever en ny versjon av H5P Core',
  contentTypeUnsupportedApiVersionContent: 'Kontakt din systemadministrator for nødvendige oppdateringer.',
  contentTypeUpdateAvailable: 'Oppdatering tilgjengelig',
  contentTypeRestricted: 'Begrenset tilgang til innholdstype',
  contentTypeRestrictedDesc: 'Bruken av denne innholdstypen er begrenset av en administrator.',
  contentTypeNotInstalled: 'Inneholdstype ikke installert',
  contentTypeNotInstalledDesc: 'Du har ikke tilgang til å installere innholdstyper.',
  theContentType: 'Innholdstypen',
  currentMenuSelected: 'Gjeldende valg',
  errorCommunicatingHubTitle: 'Klarte ikke å kommunisere med H5P Hub.',
  errorCommunicatingHubContent: 'En feil har oppstått. Vennligst prøv igjen.',
  warningNoContentTypesInstalled: 'Du har ingen installerte innholdstyper.',
  warningChangeBrowsingToSeeResults: 'Klikk <em>Alle</em> for å få en liste over alle innholdstyper du kan installere.',
  warningUpdateAvailableTitle: 'En ny versjon av :contentType er tilgjengelig.',
  warningUpdateAvailableBody: 'Oppdater til siste versjon for en best mulig opplevelse.',
  licenseDescription:
    'Noen av egenskapene til denne lisensen indikeres under. Klikk på informasjonsikonet for å lese lisensteksten.',
  licenseModalTitle: 'License Details',
  licenseModalSubtitle: 'Velg en lisens for å se en oversikt over riktig bruk',
  licenseUnspecified: 'Uspesifisert',
  licenseCanUseCommercially: 'Kan bruke kommersielt',
  licenseCanModify: 'Kan endre',
  licenseCanDistribute: 'Kan distribuere',
  licenseCanSublicense: 'Kan viderelisensiere',
  licenseCanHoldLiable: 'Kan holde ansvarlig',
  licenseCannotHoldLiable: 'Kan ikke holde ansvarlig',
  licenseMustIncludeCopyright: 'Må inkludere opphavsrettsinformasjon',
  licenseMustIncludeLicense: 'Må inkludere lisens',
  licenseFetchDetailsFailed: 'Feil ved henting av lisens-detaljer',
  imageLightboxTitle: 'Bilder',
  imageLightBoxProgress: ':num av :total',
  nextImage: 'Neste bilde',
  previousImage: 'Forrige bilde',
  screenshots: 'Skjermbilder',
  reloadButtonLabel: 'Last på nytt',
  videoQuality: 'Navn på videokvalitet',
  videoQualityDescription:
    'Dette navnet hjelper brukeren med å identifisere den aktuelle videokvaliteten. F.eks. 1080p, 720p, HD eller for mobil',
  videoQualityDefaultLabel: 'Kvalitet :index',
  noContentTypesAvailable: 'Ingen innholdstyper er tilgjengelig',
  noContentTypesAvailableDesc:
    'Nettstedet ditt har problemer med å koble til H5P.org og kan ikke liste tilgjengelige innholdstyper.',
  contentTypeCacheOutdated: 'Innholdstype-listen er utdatert',
  contentTypeCacheOutdatedDesc:
    'Nettstedet ditt har problemer med å koble til H5P.org for å se etter innholdstype-oppdateringer. Det kan hende at du ikke får oppdatert eller installert nye innholdstyper.',
  tryAgain: 'Prøv på nytt',
  getHelp: 'Få hjelp',
  untitled: 'Uten tittel :libraryTitle',
  title: 'Tittel',
  metadata: 'Metadata',
  addTitle: 'Legg til tittel',
  usedForSearchingReportsAndCopyrightInformation: 'Brukes til søk, rapporter og copyright-informasjon',
  metadataSharingAndLicensingInfo: 'Metadata (deling- og lisensinformasjon)',
  fillInTheFieldsBelow: 'Fyll inn feltene nedenfor',
  saveMetadata: 'Lagre metadata',
  addAuthor: 'Lagre forfatter',
  confirmRemoveAuthor: 'Er du sikker på at du ønsker å fjerne denne forfatteren?',
  addNewChange: 'Legg til endring',
  confirmDeleteChangeLog: 'Er du sikker på at du ønsker å slette denne loggoppføringen?',
  changelogDescription:
    'Enkelte lisenser krever at endringer gjort på det originale verket, eller verk avledet av dette, loggføres og vises. Du kan loggføre endringene dine her for lisenshensyn, slik at du og andre skal kunne holde styr på endringene som er gjort på dette innholdet.',
  logThisChange: 'Loggfør denne endringen',
  newChangeHasBeenLogged: 'Ny endring er blitt loggført',
  loggedChanges: 'Loggførte endringer',
  noChangesHaveBeenLogged: 'Ingen endring er blitt loggført',
  errorHeader: 'En feil har oppstått',
  errorCalculatingMaxScore:
    'Kunne ikke kalkulerer maks poengsum for denne innholdstypen. Maks poengsum er antatt til å være 0. Kontakt administrator dersom dette ikke er korrekt.',
  maxScoreSemanticsMissing: 'Kunne ikke finne den forventede semantikken i den valgte innholdstypen.',
  copyButton: 'Kopier',
  copiedButton: 'Kopiert',
  pasteButton: 'Lim inn',
  pasteAndReplaceButton: 'Lim inn og erstatt',
  pasteContent: 'Erstatt innhold',
  confirmPasteContent:
    'Dette vil erstatte det eksisterende innholdet med innholdet på din utklippstavle. Det eksisterende innholdet vil gå tapt. Er du sikker på at du ønsker å fortsette? ',
  confirmPasteButtonText: 'Erstatt innhold',
  copyToClipboard: 'Kopier H5P-innhold til utklippstavlen',
  copiedToClipboard: 'Innholdet er kopiert til utklippstavlen',
  pasteFromClipboard: 'Lim inn H5P-innhold fra utklippstavlen',
  pasteAndReplaceFromClipboard: 'Erstatt eksisterende innhold med H5P-innhold fra utklippstavlen',
  pasteNoContent: 'Utklippstavlen har ikke H5P-innhold',
  pasteError: 'Kan ikke lime inn fra utklippstavlen',
  pasteContentNotSupported: 'Innholdet på utklippstavlen støttes ikke i denne sammenhengen',
  pasteContentRestricted: 'Innholdet på utklippstavlen er begrenset på denne siden.',
  pasteTooOld:
    'Innholdet på utklippstavlen har lavere versjon (:clip) enn det som er støttet i denne sammenhengen (:local). Om det er mulig, kan du oppdatere innholdet du ønsker å kopiere, for så å forsøke og kopiere og lime det inn på nytt.',
  pasteTooNew:
    'Innholdet på utklippstavlen har høyere versjon (:clip) enn det som er støttet i denne sammenhengen (:local). Om det er mulig, kan du oppdatere innholdet du ønsker å kopiere til, for så å forsøke og kopiere og lime det inn på nytt.',
  ok: 'OK',
  avTablistLabel: 'Sett inn via',
  tabTitleBasicFileUpload: 'Filopplasting',
  tabTitleInputLinkURL: 'Lenke/URL',
  errorTooHighVersion: 'Parameterene innholder %used mens kun %supported eller tidligere er støttet.',
  errorNotSupported: 'Parameterene inneholder %used som ikke er støttet.',
  errorParamsBroken: 'Parameterene er ødelagt.',
  libraryMissing: 'Mangler påkrevd bibliotek %lib.',
  scriptMissing: 'Kunne ikke laste oppgraderingsskript for %lib.',
  language: 'Språk',
  noLanguagesSupported: 'Ingen språk støttet',
  changeLanguage: 'Bytt språk til :language?',
  thisWillPotentially:
    'Dette kan potensielt tilbakestille all teksten og oversettelsene. Du kan ikke omgjøre dette. Innholdet vil ikke bli endret. Vil du fortsette?',
  notAllTextsChanged: 'Ikke alle tekstene ble endret, det er kun delvis støtte for :language.',
  contributeTranslations:
    'Hvis du vil fullføre oversettelsen for :language kan du lære mer om <a href=":url" target="_new">bidra med oversettelser til H5P</a>',
  unknownLibrary: "Dessverre var ikke den valgte '%lib'-innholdstypen installert på dette systemet.",
  proceedButtonLabel: 'Fortsett for å lagre',
  enterFullscreenButtonLabel: 'Åpne i fullskjerm',
  exitFullscreenButtonLabel: 'Avslutt fullskjerm',
  noContentHeader: 'Ikke noe innhold som passer?',
  noContentSuggestion: 'Lag innhold selv!',
  tutorials: 'Veiledninger',
  contentSectionAll: 'Alt delt innhold',
  popularContent: 'Populært innhold',
  allPopular: 'Alt populært innhold',
  newOnTheHub: 'Nytt på H5P Hub',
  allNew: 'Alt nytt',
  filterBy: 'Filtrer etter',
  filter: 'Filtrer',
  filters: {
    level: {
      dropdownLabel: 'Nivå',
      dialogHeader: 'Velg utdanningsnivå',
      dialogButtonLabel: 'Filtrer etter utdanningsnivå',
    },
    language: {
      dropdownLabel: 'Språk',
      dialogHeader: 'Velg språk',
      dialogButtonLabel: 'Filtrer etter språk',
      searchPlaceholder: 'Velg ett eller flere språk',
    },
    reviewed: {
      dropdownLabel: 'Gjennomgått',
      dialogHeader: 'Gjennomgått innhold',
      dialogButtonLabel: 'Filtrer',
      optionLabel: 'Vis bare gjennomgått innhold',
    },
    contentTypes: {
      dropdownLabel: 'Innholdstyper',
      dialogHeader: 'Velg innholdstype(r)',
      dialogButtonLabel: 'Filtrer på innholdstyper',
      searchPlaceholder: 'Velg en eller flere innholdstyper',
    },
    disciplines: {
      dropdownLabel: 'Disiplin',
      dialogHeader: 'Velg din disiplin',
      dialogButtonLabel: 'Filtrer disipliner',
      searchPlaceholder: 'Velg en eller flere disipliner',
    },
    licenses: {
      dropdownLabel: 'Lisens',
      dialogHeader: 'Velg rettigheter for bruk',
      dialogButtonLabel: 'Filtrer på lisenser',
      options: { modified: 'Kan endres', commercial: 'Tillater kommersiell bruk' },
    },
  },
  clearFilters: 'Nullstill alle filtre',
  contentSearchFieldPlaceholder: 'Søk etter innhold',
  loadingContentTitle: 'Vi laster innhold for deg...',
  loadingContentSubtitle: 'Vennligst vent',
  by: 'Av',
  dropdownButton: 'Åpne nedtrekksmeny',
  paginationNavigation: 'Sidenavigasjon',
  page: 'Side',
  currentPage: 'Gjeldende side',
  nextPage: 'Gå til neste side',
  previousPage: 'Gå til forrige side',
  contentPreviewButtonLabel: 'Forhåndsvis',
  contentDownloadButtonLabel: 'Hent innhold',
  reuseContentTabLabel: 'Finn delt innhold',
  contentPublisherPanelHeader: 'Informasjon om publiserer',
  noContentFoundDesc: 'Det finnes ikke noe innhold som samsvarer med ditt søk.',
  h5pType: 'H5P-type',
  level: 'Nivå',
  size: 'Størrelse',
  failedFetchingData: 'Henting av data mislyktes',
  filterErrorMessage: 'Noe gikk galt. Vennligst last siden på nytt.',
  in: 'i',
  navigateToParent: 'Naviger til overordnet nivå',
  a11yTitleShowLabel: 'Vis etikett for tilgjengelighet',
  a11yTitleHideLabel: 'Skjul etikett for tilgjengelighet',
  reuseSuccess: ':title ble vellykket importert fra H5P Hub.',
};
