{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more comments.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_dataprivacy/data_registry

    Data registry main page.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * none

    Example context (json):
    {
    }
}}
<div class="data-registry">
    <div class="top-nav d-flex">
        {{#defaultsbutton}}
            {{> core/action_link}}
        {{/defaultsbutton}}
        {{#actions}}
            {{> core/action_menu}}
        {{/actions}}
    </div>
    {{#info}}
        <div class="mt-1">
            {{> core/notification_info}}
        </div>
    {{/info}}
    {{#nosystemdefaults}}
        <div class="mt-1">
            {{> core/notification_warning}}
        </div>
    {{/nosystemdefaults}}

    <div class="container-fluid mt-2">
        <div class="row">
            <div class="col-md-4 ps-0 nav-pills context-tree">
                {{#tree}}
                    {{> tool_dataprivacy/context_tree_node}}
                {{/tree}}
            </div>
            <div class="col-md-8" id="context-form-container">
            </div>
        </div>
    </div>
</div>
