{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_data/template_editor_resetmodal

    Reset template modal content.

    Example context (json):
    {
        "resetallname": "resetallcheck"
    }
}}
<p>{{#str}} resettemplateconfirm, mod_data, {{templatename}} {{/str}}</p>
<div class="form-check">
    <input type="checkbox" class="form-check-input" id="modalcheckbox" name="{{resetallname}}">
    <label class="form-check-label" for="modalcheckbox">{{#str}} resetalltemplates, mod_data {{/str}}</label>
</div>
