<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings used by the plagiarism component
 *
 * @package    core_plagiarism
 * @copyright  2010 Dan Marsden <dan@danmarsden.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['availableplugins'] = 'Available plugins';
$string['configplagiarismplugins'] = 'Please choose the plagiarism plugin you would like to configure';
$string['enableplagiarism'] = 'Enable plagiarism plugins';
$string['configenableplagiarism'] = 'This will allow administrators to configure plagiarism plugins (if installed)';
$string['manageplagiarism'] = 'Manage plagiarism plugins';
$string['nopluginsinstalled'] = 'No plagiarism plugins are installed.';
$string['plagiarism'] = 'Plagiarism';
$string['privacy:metadata:plagiarism'] = 'The plagiarism subsystem acts as a channel, passing requests from plugins to the various plagiarism plugins.';
