<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Lang strings
 *
 * @package    report
 * @subpackage completion
 * @copyright  2009 Catalyst IT Ltd
 * @author     Aaron Barnes <aaronb@catalyst.net.nz>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['clicktomarkusercomplete'] = 'Click to mark user complete';
$string['completion:view'] = 'View course completion report';
$string['completiondate'] = 'Completion date';
$string['id'] = 'ID';
$string['name'] = 'Name';
$string['nocapability'] = 'Can not access user completion report';
$string['page-report-completion-x'] = 'Any completion report';
$string['page-report-completion-index'] = 'Course completion report';
$string['page-report-completion-user'] = 'User course completion report';
$string['pluginname'] = 'Course completion';
$string['privacy:metadata'] = 'The Course completion report only shows data stored in other locations.';
$string['eventreportviewed'] = 'Completion report viewed';
$string['eventuserreportviewed'] = 'Completion user report viewed';
