{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_grades/grades/grader/gradingpanel/scale

    Scale-based grading template for use in the grading panel.

    Context variables required for this template:

    Example context (json):
    {
        "value": 1,
        "selected": true,
        "title": "Motivational"
    }
}}
<form>
  <div class="mb-3">
    <label for="core_grades-grade-{{uniqid}}">{{#str}}gradenoun, moodle{{/str}}</label>
    <select class="form-control" name="grade" id="core_grades-grade-{{uniqid}}" aria-describedby="core_grades-help-{{uniqid}}">
        <option value="-1">{{#str}} nograde, moodle{{/str}}</option>
    {{#options}}
        <option value="{{value}}" {{#selected}}selected{{/selected}}>{{title}}</option>
    {{/options}}
    </select>
    <small id="core_grades-help-{{uniqid}}" class="form-text text-muted">{{#str}}grade_help, core_grades{{/str}}</small>
  </div>
</form>
