.ai-drawer {
    position: fixed;
    top: 60px;
    bottom: 0;
    right: calc(-315px + -10px);
    width: 315px;
    background-color: #f8f9fa;
    z-index: 1016;
    transition: right 0.2s ease, top 0.2s ease, bottom 0.2s ease, visibility 0.2s ease, transform 0.5s ease;
    visibility: hidden;
}
.ai-drawer.show {
    right: 0;
    visibility: visible;
}

.ai-drawer-header {
    padding: 0;
    height: 60px;
    display: flex;
    align-items: center;
}

.ai-drawer-header .ai-drawer-button {
    margin-left: auto;
    margin-right: 5px;
}

.ai-drawer-body {
    position: relative;
    height: calc(100vh - 120px);
    display: flex;
    flex-direction: column;
    flex-wrap: nowrap;
    padding: 0.4rem;
    overflow-y: auto;
    scrollbar-width: thin;
    scrollbar-color: #6a737b #f8f9fa;
}

.course-assist-response-content {
    font-size: 0.875em;
}

#ai-features .icon {
    min-width: 16px;
}

.course-assist-response-header .icon {
    width: 30px;
    height: 30px;
}

.course-assist-controls .icon,
.ai-course-assist-sparkles-icon .icon {
    margin-right: 0;
}

.course-assist-controls .ai-action {
    color: unset;
}

/* Toggle visibility of sparkles icon on hover. */
.course-assist-controls .ai-action .ai-course-assist-sparkles-icon,
.course-assist-controls .ai-action:not([disabled]):hover .ai-course-assist-sparkles-icon.white,
.course-assist-controls.show .ai-action:not([disabled]) .ai-course-assist-sparkles-icon.white {
    display: inline-block;
}

.course-assist-controls .ai-action .ai-course-assist-sparkles-icon.white,
.course-assist-controls .ai-action:not([disabled]):hover .ai-course-assist-sparkles-icon,
.course-assist-controls.show .ai-action:not([disabled]) .ai-course-assist-sparkles-icon {
    display: none;
}
