{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_bigbluebuttonbn/guest_links

    This template renders the mobile page.

    Example context (json):
    {
        "guestaccessenabled": 1,
        "guestjoinurl": "http://moodle.org/",
        "guestpassword": "password"
    }
}}
<button class="btn btn-link m-1" type="button" data-action="show-guest-access">
    {{#pix}}t/add, core{{/pix}}{{#str}}guestaccess_add, mod_bigbluebuttonbn{{/str}}
</button>
{{#js}}
    require(['mod_bigbluebuttonbn/guest_access_modal'], function(GuestLinkAccess) {
     GuestLinkAccess.init({
        id: "{{ bigbluebuttonbnid }}",
        groupid: "{{ groupid }}"
    });
    });
{{/js}}
