{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_course/activitychooserbutton

    Displays a add activity or resource button.

    Example context (json):
    {
        "hasactionlinks": false,
        "sectionnum": 0,
        "sectionname": "Section example",
        "modid": 1,
        "activityname": "Activity example",
        "sectionreturn": 0
    }
}}

{{! Single button to add a new activity or resource. }}
{{^hasactionlinks}}
    {{> core_course/addresourceoractivitybutton }}
{{/hasactionlinks}}

{{! Dropdown with "add a new activity or resource" item and any other action links. }}
{{#hasactionlinks}}
<div class="dropdown">
    <button class="btn add-content d-flex justify-content-center align-items-center p-1 icon-no-margin"
            type="button"
            id="dropdownMenuButton-{{#modid}}{{modid}}{{/modid}}{{^modid}}{{uniqid}}{{/modid}}"
            data-bs-toggle="dropdown"
            data-action="open-addingcontent"
            {{#modid}}
            data-beforemod="{{modid}}"
            aria-label="{{#str}}insertcontentbefore, core, { "activityname": {{#quote}} {{activityname}} {{/quote}} } {{/str}}"
            {{/modid}}
            {{^modid}}
            aria-label="{{#str}}insertcontentsection, core, { "sectionname": {{#quote}} {{sectionname}} {{/quote}} } {{/str}}"
            {{/modid}}
            aria-haspopup="true"
            aria-expanded="false"
            tabindex="0"
            title="{{#str}}addcontent, core{{/str}}"
    >
        {{#pix}} t/add, core {{/pix}}
    </button>
    <div class="dropdown-menu" aria-labelledby="dropdownMenuButton-{{#modid}}{{modid}}{{/modid}}{{^modid}}{{uniqid}}{{/modid}}">
        {{> core_course/addresourceoractivitybutton }}
        {{#actionlinks}}
            {{> core/action_link}}
        {{/actionlinks}}
    </div>
</div>
{{/hasactionlinks}}
