<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component aiplacement_editor, language 'en'.
 *
 * @package    aiplacement_editor
 * @copyright  2024 Matt Porritt <matt.porritt@moodle.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['generatecontent'] = 'Generate AI content';
$string['generateimage'] = 'AI generate image';
$string['generateimagesetting'] = 'Enable generate image';
$string['generateimagesetting_desc'] = 'Enable or disable the generation of images from a text prompt.';
$string['generatetext'] = 'AI generate text';
$string['generatetextsetting'] = 'Enable generate text';
$string['generatetextsetting_desc'] = 'Enable or disable the generation of text from a text prompt.';
$string['noeditor'] = 'Text editor placement is not available for this context.';
$string['pluginname'] = 'Text editor placement';
$string['privacy:metadata'] = 'The Text editor placement plugin does not store any personal data.';
$string['editor:generate_image'] = 'Generate AI images in text editor';
$string['editor:generate_text'] = 'Generate AI text in text editor';
