{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_grades/grade_category_visibility_toggle

    Template for the grade category visibility toggle element.

    Context variables required for this template:
    * category - The grade category identifier

    Example context (json):
    {
        "category": "gc2"
    }
}}
<a aria-expanded="true" role="button" data-category={{category}} data-target=".{{category}}[data-hidden='false']" class="btn btn-icon me-1 toggle-category" href="#">
    <span class="collapsed text-nowrap" title="{{#str}} collapse, core {{/str}}">
        <i class="icon fa fa-chevron-down fa-fw" title="{{#str}} collapse, core {{/str}}" role="img" aria-label="{{#str}} collapse, core {{/str}}"></i>
    </span>
    <span class="expanded text-nowrap" title="{{#str}} expand, core {{/str}}">
        <i class="icon fa fa-chevron-right fa-fw" title="{{#str}} expand, core {{/str}}" role="img" aria-label="{{#str}} expand, core {{/str}}"></i>
    </span>
</a>
