{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_calendar/export_calendar_url

    Template for the URL for the exported calendar.

    Example context (json):
    {
        "calendarexporturl": "https://www.moodle.org"
    }
}}
<div class="card">
    <div class="card-body alert-info">
        <div class="mb-3">
            <label for="calendarexporturl">{{#str}}calendarexporturl, calendar{{/str}}</label>
            <div class="input-group">
                <input type="text" id="calendarexporturl" class="form-control" value="{{calendarexporturl}}" readonly />
                <button id="copyexporturl" {{!
                    }} data-action="copytoclipboard"{{!
                    }} data-clipboard-target="#calendarexporturl"{{!
                    }} data-clipboard-success-message="{{#str}}calendarurlcopiedtoclipboard, core_calendar{{/str}}"{{!
                    }} class="btn btn-secondary"{{!
                    }} aria-label="{{#str}}copycalendarurl, calendar{{/str}}"{{!
                }} disabled>
                    {{#str}}copyurl, calendar{{/str}}
                </button>
            </div>
        </div>
    </div>
</div>
{{#js}}
require(['core_calendar/export'], function(calendarExport) {
    calendarExport.init();
});
{{/js}}
