{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/movesection

    Displays the course index.

    Example context (json):
    {
        "sectionid": 23,
        "sectiontitle": "Section title",
        "sections": [
            {
                "title": "General",
                "id": 42,
                "number": 1,
                "sectionurl": "#",
                "indexcollapsed": 0
            },
            {
                "title": "City of God or Cidade de Deus",
                "id": "43",
                "number": "2",
                "sectionurl": "#",
                "indexcollapsed": 1
            }
        ]
    }

}}
{{#information}}
<p data-for="sectionname">
    {{{information}}}:
</p>
{{/information}}
<nav class="collapse-list" id="destination-selector" role="tree">
    {{#sections}}
    <div
        class="collapse-list-item"
        role="treeitem"
    >
        <a href="#"
            class="collapse-list-link text-truncate"
            data-for="section"
            data-id="{{id}}"
            data-number="{{number}}"
        >
            {{{title}}}
        </a>
    </div>
    {{/sections}}
</nav>
