{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
  }}
{{!
  @template qbank_customfields/preview_display

  Example context (json):
    {"categories":[
          {  "catname":"Other fields",
             "fields":[
                {"name":"checkit", "value":"No"},
                {"name":"datetime", "value":"1 October 2021"}
             ]
          }
      ]}
}}
<h3>{{#str}} customfield, qbank_customfields {{/str}}</h3>
{{#categories}}
    <h4>{{catname}}</h4>
    {{#fields}}
        <div class="customfield customfield_{{type}} customfield_{{shortname}}">
            <span class="customfieldname">{{{name}}}</span><span class="customfieldseparator">: </span><span class="customfieldvalue">{{{value}}}</span>
        </div>
    {{/fields}}
{{/categories}}
