<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'userkey', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package   core
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['adduserkey'] = 'Add user key';
$string['createnewkey'] = 'Create a new user key';
$string['createuserkey'] = 'Create user key';
$string['deletekeyconfirm'] = 'Do you really want to delete this user key?';
$string['deleteuserkey'] = 'Delete user key';
$string['edituserkey'] = 'Edit user key';
$string['keyiprestriction'] = 'Key IP restriction';
$string['keyiprestriction_help'] = 'Enter a specific IP address, or a range of IP addresses that will be the only IP addresses allowed to access this data. Leave empty to disable IP restriction (not recommended).';
$string['keymanager'] = 'Key manager';
$string['keyvaliduntil'] = 'Key valid until';
$string['keyvaliduntil_help'] = 'Select an optional date after which the key will no longer be valid (recommended for added security).';
$string['keyvalue'] = 'Key value';
$string['newuserkey'] = 'New user key';
$string['privacy:metadata:user_private_key:script'] = 'The script which is responsible for the user key.';
$string['privacy:metadata:user_private_key:value'] = 'The value of the key.';
$string['privacy:metadata:user_private_key:userid'] = 'The user associated with the key.';
$string['privacy:metadata:user_private_key:instance'] = 'The instance of the script.';
$string['privacy:metadata:user_private_key:iprestriction'] = 'The IP address range that this key can be used from.';
$string['privacy:metadata:user_private_key:validuntil'] = 'The date that the private key is valid until';
$string['privacy:metadata:user_private_key:timecreated'] = 'The date and time that the key was created.';
$string['privacy:metadata:user_private_key'] = 'Private keys for the user.';
$string['userkey'] = 'User key';
$string['userkey_help'] = 'Select a saved key that will give users access to the data published by this export plugin, without having to log into Moodle. Select "Create a new user key" to generate a new key when submitting this form.';
$string['userkeys'] = 'User keys';
