<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
/**
 * Strings for component 'favourites', language 'en', branch 'master'
 *
 * @package   core_favourites
 * @copyright 2018 Jake Dallimore <jrhdallimore@gmail.com>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
$string['privacy:metadata:favourite'] = 'Stores references to items which have been starred.';
$string['privacy:metadata:favourite:component'] = 'The component which the starred item belongs to e.g. core_user';
$string['privacy:metadata:favourite:itemid'] = 'The identifier for the item being starred';
$string['privacy:metadata:favourite:itemtype'] = 'The starred item type e.g. course';
$string['privacy:metadata:favourite:ordering'] = 'A number used to order starred items of the same type';
$string['privacy:metadata:favourite:timecreated'] = 'The time when the item was starred';
$string['privacy:metadata:favourite:timemodified'] = 'The time when the starred item was last modified';
$string['privacy:metadata:favourite:userid'] = 'The ID of the user who starred the item';
