{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_lesson/override_action_menu

    A select menu and button for user and group overrides.

    Classes required for JS:
    * none

    Context variables required for this template:
    * see mod/lesson/classes/output/override_action_menu.php

    Example context (json):
    {
        "urlselect": {
            "id": "url_select_test",
            "action": "https://example.com/post",
            "formid": "url_select_form",
            "sesskey": "sesskey",
            "label": "core/url_select",
            "helpicon": {
                "title": "Help with something",
                "text": "Help with something",
                "url": "http://example.org/help",
                "linktext": "",
                "icon":{
                    "extraclasses": "",
                    "attributes": [
                        {"name": "src", "value": "../../../pix/help.svg"},
                        {"name": "alt", "value": "Help icon"}
                    ]
                }
            },
            "showbutton": "Go",
            "options": [{
                "name": "Group 1", "isgroup": true, "options":
                [
                    {"name": "Item 1", "isgroup": false, "value": "1"},
                    {"name": "Item 2", "isgroup": false, "value": "2"}
                ]},
                {"name": "Group 2", "isgroup": true, "options":
                [
                    {"name": "Item 3", "isgroup": false, "value": "3"},
                    {"name": "Item 4", "isgroup": false, "value": "4"}
                ]}],
            "disabled": false,
            "title": "Some cool title"
        },
        "addoverride": {
            "link": "https://moodle.org",
            "text": "Add override"
        },
        "headinglevel": 3,
        "heading": "Some heading"
    }

}}
<div class="container-fluid tertiary-navigation">
    <div class="d-flex">
        {{#urlselect}}
            <div class="navitem">
                {{>core/url_select}}
            </div>
        {{/urlselect}}
        {{#addoverride}}
            <div class="navitem">
                <a class="btn btn-primary" href="{{link}}">{{text}}</a>
            </div>
        {{/addoverride}}
    </div>
</div>
<h{{headinglevel}}>{{heading}}</h{{headinglevel}}>
