{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_calendar/month_navigation

    Calendar month navigation.

    The purpose of this template is to render the navigation to switch to previous and next months.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Example context (json):
    {
    }
}}
<div id="month-navigation-{{uniqid}}-{{calendarinstanceid}}" class="controls">
    <div class="calendar-controls">
        <a{{!
            }} href="{{previousperiodlink}}"{{!
            }} class="arrow_link previous"{{!
            }} {{#viewinginblock}}title="{{#str}}monthprevwithname, calendar, {{previousperiodname}}{{/str}}" {{/viewinginblock}} {{!
            }} data-year="{{previousperiod.year}}"{{!
            }} data-month="{{previousperiod.mon}}"{{!
            }} data-drop-zone="nav-link" {{!
            }} aria-label="{{#str}}monthprevwithname, calendar, {{previousperiodname}}{{/str}}" {{!
        }}>
            <span class="arrow" aria-hidden="true">{{{larrow}}}</span>
            &nbsp;
            <span class="arrow_text">{{previousperiodname}}</span>
        </a>
        <span class="hide"> | </span>
        {{#viewinginblock}}
            <h4 class="current">{{periodname}}</h4>
        {{/viewinginblock}}
        {{^viewinginblock}}
            <h2 class="current">{{periodname}}</h2>
        {{/viewinginblock}}
        <span class="hide"> | </span>
        <a{{!
            }} href="{{nextperiodlink}}"{{!
            }} class="arrow_link next"{{!
            }} {{#viewinginblock}}title="{{#str}}monthnextwithname, calendar, {{nextperiodname}}{{/str}}" {{/viewinginblock}} {{!
            }} data-year="{{nextperiod.year}}"{{!
            }} data-month="{{nextperiod.mon}}"{{!
            }} data-drop-zone="nav-link" {{!
            }} aria-label="{{#str}}monthnextwithname, calendar, {{nextperiodname}}{{/str}}" {{!
        }}>
            <span class="arrow_text">{{nextperiodname}}</span>
            &nbsp;
            <span class="arrow" aria-hidden="true">{{{rarrow}}}</span>
        </a>
    </div>
</div>
{{#js}}
require(['jquery', 'core_calendar/month_navigation_drag_drop'], function($, DragDrop) {
    var root = $('#month-navigation-{{uniqid}}-{{calendarinstanceid}}');
    DragDrop.init(root);
});
{{/js}}
