.h5p-hub article,.h5p-hub aside,.h5p-hub details,.h5p-hub figcaption,.h5p-hub figure,.h5p-hub footer,.h5p-hub header,.h5p-hub hgroup,.h5p-hub menu,.h5p-hub nav,.h5p-hub section{display:block}.h5p-hub ol,.h5p-hub ul{list-style:none}.h5p-hub ul.ul,.h5p-hub .h5p-hub-ul{list-style:disc;margin:.833em 0}.h5p-hub ul.ul li,.h5p-hub .h5p-hub-ul li{margin-left:2.499em}.h5p-hub blockquote,.h5p-hub q{quotes:none}.h5p-hub blockquote:before,.h5p-hub blockquote:after,.h5p-hub q:before,.h5p-hub q:after{content:'';content:none}.h5p-hub table{border-collapse:collapse;border-spacing:0}.h5p-hub dl{margin-top:0;margin-bottom:0}.h5p-hub dd,.h5p-hub dt{padding:0;margin:0}.h5p-hub dt{font-weight:700}.h5p-hub button{border:0;padding:0;margin:0}.h5p-hub{font-family:'Open Sans', sans-serif;font-size:.917em;color:#1f2227}.h5p-hub .h1,.h5p-hub .h2,.h5p-hub .h3,.h5p-hub .h4,.h5p-hub .h5,.h5p-hub .h6,.h5p-hub h1,.h5p-hub h2,.h5p-hub h3,.h5p-hub h4,.h5p-hub h5,.h5p-hub h6{font-family:inherit;color:inherit;font-weight:300;line-height:1.1}.h5p-hub .h1,.h5p-hub .h2,.h5p-hub .h3,.h5p-hub h1,.h5p-hub h2,.h5p-hub h3{margin-top:.833em;margin-bottom:.4165em}.h5p-hub .h4,.h5p-hub .h5,.h5p-hub .h6,.h5p-hub h4,.h5p-hub h5,.h5p-hub h6{margin-top:.4165em;margin-bottom:.4165em;font-weight:600}.h5p-hub h1,.h5p-hub .h1{font-size:1.733em}.h5p-hub h2,.h5p-hub .h2{font-size:1.458em}.h5p-hub h3,.h5p-hub .h3{font-size:1.250em}.h5p-hub h4,.h5p-hub .h4{font-size:1.042em}.h5p-hub h5,.h5p-hub .h5{font-size:0.938em}.h5p-hub h6,.h5p-hub .h6{font-size:0.875em}.h5p-hub ul{padding:0;margin:0}.h5p-hub hr{border:0;height:0;margin:.833em 0;border-bottom:1px solid #ececec}.h5p-hub small,.h5p-hub .small,.h5p-hub .h5p-hub-small{color:#697585;line-height:1.7em;font-size:.95em}.h5p-hub .link,.h5p-hub a{font-family:'Open Sans', sans-serif;color:#1f2227;font-size:1em;font-weight:400;cursor:pointer;text-decoration:underline;background-color:transparent;border:0;padding:0}.h5p-hub .dl-horizontal dt{float:left;width:5em;overflow:hidden;clear:left;text-align:right;text-overflow:ellipsis;white-space:nowrap}.h5p-hub .dl-horizontal dd{margin-left:5.833em}.h5p-hub .additional-information{font-style:italic;color:#697585}.h5p-hub .hidden{display:none}.h5p-hub .h5p-hub-hidden{display:none}.h5p-hub .h5p-hub-button{font-family:'Open Sans', sans-serif;font-size:.95em;font-weight:600;text-align:center;text-decoration:none;line-height:1em;min-width:4em;padding:0.708em 1.5em;border-radius:1.375em;background-color:#f3f3f3;color:#1f2227;display:inline-block;cursor:pointer;border:2px solid transparent;transition:background-color .35s ease}.h5p-hub .h5p-hub-button[disabled],.h5p-hub .h5p-hub-button[aria-disabled]{cursor:not-allowed}.h5p-hub .h5p-hub-button:focus{outline:0;box-shadow:0.06em 0 0.6em 0.1em #7bc1f9}.h5p-hub .h5p-hub-button.h5p-hub-button-primary{background-color:#0a78d1;color:#fff;border-color:#0a78d1}.h5p-hub .h5p-hub-button.h5p-hub-button-primary:hover{background-color:#096ab9;border-color:#096ab9}.h5p-hub .h5p-hub-button.h5p-hub-button-primary:active{background-color:#085ca0;border-color:#085ca0}.h5p-hub .h5p-hub-button.h5p-hub-button-inverse-primary{background-color:#fff;color:#0a78d1;border-color:#0a78d1}.h5p-hub .h5p-hub-button.h5p-hub-button-inverse-primary:hover{color:#085ca0;border-color:#085ca0}.h5p-hub .h5p-hub-button.h5p-hub-button-inverse-primary:active{color:#054070;border-color:#054070}.h5p-hub .h5p-hub-button:before{font-weight:normal}.h5p-hub .h5p-hub-img-responsive{max-width:100%;object-fit:contain}.h5p-hub #h5p-hub-reuse-view .h5p-hub-img-responsive{background-color:#d0d0d1}@-webkit-keyframes spin{100%{transform:rotate(360deg)}}@keyframes spin{100%{transform:rotate(360deg)}}.h5p-hub .h5p-hub-button [class^="icon-"]{margin-right:.4165em;display:inline-block;font-weight:normal;font-size:0.9em}.h5p-hub .h5p-hub-icon-hub-icon:before{font-family:'h5p-hub';content:""}.h5p-hub .h5p-hub-icon-arrow-thick:before{font-family:'h5p-hub';content:""}.h5p-hub .h5p-hub-icon-arrow-thin:before{font-family:'h5p-hub';content:""}.h5p-hub .h5p-hub-icon-accordion-arrow:before{font-family:'h5p-hub';font-weight:normal;content:""}.h5p-hub .h5p-hub-icon-search:before{font-family:'h5p-hub';content:""}

.h5p-hub .h5p-hub-client-drop-down{background-color:white;color:#697585;font-style:italic;font-weight:600;cursor:pointer;position:relative}.h5p-hub .h5p-hub-client-drop-down .h5p-hub-icon-hub-icon{color:#474f5a;padding:0.833em 1.55em}.h5p-hub .h5p-hub-client-drop-down .h5p-hub-icon-hub-icon:before{font-style:normal;font-weight:normal;margin-right:1em}.h5p-hub .h5p-hub-client-drop-down .h5p-hub-icon-hub-icon:after{font-family:'h5p-hub';content:"";display:inline-block;speak:none;font-style:normal;font-weight:normal;font-variant:normal;text-transform:none;margin-right:.9163em;margin-right:1.8em;display:block;font-size:0.9em;line-height:3.5em;position:absolute;right:0;top:0}.h5p-hub .h5p-hub-client-drop-down .h5p-hub-description{color:transparent;width:1px;height:1px;overflow:hidden;display:inline-block;position:absolute}.h5p-hub .h5p-hub-client-drop-down .h5p-hub-selected{color:#474f5a}

.h5p-hub .h5p-hub-panel.h5p-hub-section-content-types .h5p-hub-tab-button[aria-selected="true"]{border-bottom:3px solid #0a78d1}.h5p-hub .h5p-hub-panel.h5p-hub-section-reuse .h5p-hub-tab-button[aria-selected="true"]{border-bottom:3px solid #db6f28}.h5p-hub .h5p-hub-panel.h5p-hub-section-upload .h5p-hub-tab-button[aria-selected="true"]{border-bottom:3px solid #ced6e3}.h5p-hub .h5p-hub-tab-panel{border:none}.h5p-hub .h5p-hub-tab-panel [role="tablist"]{background-color:#474f5a;list-style:none;border-bottom:none;padding-left:1em;margin:0}.h5p-hub .h5p-hub-tab-panel [role="tablist"]>span{display:block;height:0.3em;background:#0a78d1;position:relative;transition:all .5s ease}.h5p-hub .h5p-hub-tab-panel [role="tab"]{display:inline-block;text-decoration:none;text-align:center;transition:all .5s ease;color:#f3f3f3;outline-color:#1a93f4;border-bottom:3px solid transparent;transition:border-bottom 0.4s}.h5p-hub .h5p-hub-tab-panel [role="tab"]:nth-child(1) ~ span,.h5p-hub .h5p-hub-tab-panel [role="tab"]:nth-child(1)[aria-selected="true"] ~ span{height:0.27em;width:200px;right:0}.h5p-hub .h5p-hub-tab-panel [role="tab"]:nth-child(2) ~ span,.h5p-hub .h5p-hub-tab-panel [role="tab"]:nth-child(2)[aria-selected="true"] ~ span{height:0.27em;width:4em;right:-230px}.h5p-hub .h5p-hub-tab-panel [role="tab"]>a{display:block;color:#fff;text-decoration:none;padding:.833em}.h5p-hub .h5p-hub-tab-panel [role="tab"]>a:before{font-family:'h5p-hub';content:"";display:inline-block;speak:none;font-style:normal;font-weight:normal;font-variant:normal;text-transform:none;margin-right:.833em}.h5p-hub .h5p-hub-tab-panel [role="tab"]>a:focus:before{border-radius:50%;box-shadow:0 0 0.6em 0.1em #7bc1f9}.h5p-hub .h5p-hub-tab-panel [role="tab"][aria-selected='true']>a:before{content:""}

.h5p-hub .h5p-hub-no-content{display:flex;box-shadow:0 -0.4em 0.4em -0.2em rgba(79,87,99,0.2) inset}.h5p-hub .h5p-hub-no-content .h5p-hub-no-results-img{width:12.7em;height:4.95em;padding:3em 2em}@media (max-width: 576px){.h5p-hub .h5p-hub-no-content .h5p-hub-no-results-img{width:8em;padding:3em 1em}}.h5p-hub .h5p-hub-no-content .h5p-hub-right-text{padding:3em 2em}.h5p-hub .h5p-hub-no-content .h5p-hub-right-text .h5p-hub-header{font-size:1.2em;font-weight:bold;margin-bottom:0.4em}.h5p-hub .h5p-hub-no-content .h5p-hub-right-text .h5p-hub-suggestion-text{color:#4d4d4d;margin-bottom:0.3em}@media (max-width: 576px){.h5p-hub .h5p-hub-no-content .h5p-hub-right-text{padding:3em 1em}}.h5p-hub .h5p-hub-no-content .h5p-hub-right-text .h5p-hub-url{color:#186df7;font-weight:bold}

.h5p-hub-list{overflow:auto}.h5p-hub-list ol{margin:1.25em 0;padding:0;list-style:none}

.h5p-hub ul.h5p-hub-list-of-numbers{margin-top:1.5em;display:flex;flex-direction:row;border-radius:0.15146em;justify-content:center}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-list-element{user-select:none;-ms-user-select:none;padding:0.17039em}@media (max-width: 576px){.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-list-element{padding:0.25em;margin-bottom:1em}}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-list-element a{padding:0.25em 0.75em;font-size:1em;font-weight:bold;text-decoration:none;outline:none;border-radius:0.15146em;border:0.07573em solid #ffffff;display:inline-block}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-list-element a.h5p-hub-highlight{background-color:RGBA(209, 107, 23, 0.1);border:0.07573em solid #d16b17}@media (hover: hover){.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-list-element a:hover{background-color:RGBA(209, 107, 23, 0.05);border:0.07573em solid #ffffff}}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-list-element a:focus{box-shadow:0 0 0.4em 0.01em RGBA(26, 147, 244, 0.7)}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-list-element.disabled{pointer-events:none}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-list-element.disabled a{color:#b3b3b3}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-list-element.disabled a:hover{background-color:#ffffff}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-previous-arrow::after{font-family:'h5p-hub';content:"";transform:rotate(90deg);font-style:normal;font-size:0.7em;display:inline-block}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-next-arrow::before{font-family:'h5p-hub';content:"";transform:rotate(-90deg);font-style:normal;font-size:0.7em;display:inline-block}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-dots{padding:0.17039em;pointer-events:none;user-select:none;-ms-user-select:none}.h5p-hub ul.h5p-hub-list-of-numbers .h5p-hub-dots .h5p-hub-dots-text{padding:0.25em 0.75em}



.h5p-hub .h5p-hub-content-item.h5p-hub-tabular{font-size:16px;display:flex;padding:0.5em 1em;border-bottom:1px solid #ececec;align-items:center;cursor:pointer}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-left{align-self:flex-start}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-middle{flex-grow:10;flex-shrink:10;margin-left:1em;overflow:hidden;min-width:0}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-content-icon{width:7em;height:5.25em;object-fit:contain;background-color:#d0d0d1}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-headline{line-height:1.2}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-title{font-size:1.1em;font-weight:bold;line-height:normal;letter-spacing:-0.5px}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-title.h5p-hub-reviewed:after{font-size:0.8em;font-family:'h5p-hub';font-weight:normal;content:'\e90b';color:#359d94;position:relative;top:-0.2em;margin-left:0.3em}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-by{font-size:1em;color:#666;margin:0 0.3em}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-owner{font-size:1em}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-content-type{font-size:0.9em;color:#303030;letter-spacing:0.65px;margin:0.25em 0}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-summary{color:#72757c;font-size:0.95em;white-space:nowrap;overflow:hidden;text-overflow:ellipsis}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-right{flex-grow:0;flex-shrink:0;position:relative;font-size:1.5em;width:1.5em;height:1em;text-align:center}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-right:before{color:#72757c;display:inline-block;opacity:0;transition:opacity 0.3s, left 0.3s;font-family:'h5p-hub';content:'\e906';position:relative;left:-0.2em}@media (max-width: 576px){.h5p-hub .h5p-hub-content-item.h5p-hub-tabular .h5p-hub-right{display:none}}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular:hover,.h5p-hub .h5p-hub-content-item.h5p-hub-tabular:focus{background-color:#fafafa}.h5p-hub .h5p-hub-content-item.h5p-hub-tabular:hover .h5p-hub-right:before,.h5p-hub .h5p-hub-content-item.h5p-hub-tabular:focus .h5p-hub-right:before{opacity:1;left:0em}

.h5p-hub .h5p-hub-content-item.h5p-hub-grid{display:inline-block;width:16.66%;margin-top:0.5em;outline:none}@media (max-width: 768px){.h5p-hub .h5p-hub-content-item.h5p-hub-grid{width:33.33%}.h5p-hub .h5p-hub-content-item.h5p-hub-grid .h5p-hub-content-icon{height:10.5em}}@media (max-width: 576px){.h5p-hub .h5p-hub-content-item.h5p-hub-grid{width:50%}.h5p-hub .h5p-hub-content-item.h5p-hub-grid .h5p-hub-content-icon{height:12em}}.h5p-hub .h5p-hub-content-item.h5p-hub-grid:focus .h5p-hub-grid-item,.h5p-hub .h5p-hub-content-item.h5p-hub-grid:hover .h5p-hub-grid-item{background-color:#fafafa}.h5p-hub .h5p-hub-content-item.h5p-hub-grid:focus .h5p-hub-grid-item{border-color:#5B9DD9}.h5p-hub .h5p-hub-grid-item{cursor:pointer;text-align:center;padding:0.5em;border:1px solid #ddd;margin:0 0.3em}.h5p-hub .h5p-hub-grid-item .h5p-hub-grid-item-title{font-weight:bold;letter-spacing:-0.5px;display:-webkit-box;-webkit-line-clamp:2;-webkit-box-orient:vertical;overflow:hidden;text-overflow:ellipsis;height:3em}.h5p-hub .h5p-hub-grid-item img{width:100%;height:6em;object-fit:contain;background-color:#d0d0d1}.h5p-hub .h5p-hub-grid-item button{margin:0.5em 0 0.75em}

.h5p-hub-list.h5p-hub-loading{pointer-events:none}.h5p-hub-list.h5p-hub-loading .h5p-hub-content-item{animation:opacitychange 1s infinite}.h5p-hub-list.h5p-hub-loading .h5p-hub-loading-block{background:#ccc;min-height:1.15em}.h5p-hub-list.h5p-hub-loading .h5p-hub-loading-block.h5p-hub-button{background:transparent;animation:none;border:2px solid #ccc}.h5p-hub-list.h5p-hub-loading .h5p-hub-middle{height:5.25em;display:flex;flex-direction:column;justify-content:space-between}.h5p-hub-list.h5p-hub-loading .h5p-hub-text-long{width:100%}.h5p-hub-list.h5p-hub-loading .h5p-hub-text-medium{width:60%}.h5p-hub-list.h5p-hub-loading .h5p-hub-text-short{width:30%}.h5p-hub-list.h5p-hub-loading .h5p-hub-grid-item{padding:1em}.h5p-hub-list.h5p-hub-loading .h5p-hub-grid-item .h5p-hub-image{height:8vw;min-height:5em}.h5p-hub-list.h5p-hub-loading .h5p-hub-grid-item .h5p-hub-loading-block{margin-bottom:0.5em}.h5p-hub-list.h5p-hub-loading .h5p-hub-grid-item .h5p-hub-loading-block.h5p-hub-button{margin-bottom:0;min-width:0;width:4em;max-width:40%}@keyframes opacitychange{0%{opacity:1}50%{opacity:0.6}100%{opacity:1}}

.h5p-hub .h5p-hub-content-list{transition:opacity 0.2s, visibility 0s linear 0s}.h5p-hub .h5p-hub-content-list[aria-hidden="true"]{display:inherit;opacity:0;visibility:hidden;transition:opacity 0.2s, visibility 0s linear 0.2s}.h5p-hub .h5p-hub-content-list .h5p-hub-list.h5p-hub-tabular:before{content:" ";width:100%;height:calc(100% - 5.996455834em);position:absolute;pointer-events:none}.h5p-hub .h5p-hub-content-list .h5p-hub-list.h5p-hub-grid ol{margin:0}.h5p-hub .h5p-hub-content-list .h5p-hub-fetching-failed{display:inline-block;font-size:0.925em;border-radius:5px;margin-left:1em;padding:0.5em 0.85em;color:#c64750;background:rgba(198,71,80,0.1)}

.h5p-hub .h5p-hub-navbar{background-color:transparent;list-style:none;color:#020202;border:0;padding:1.2em 1.2em 0.975422574em 2.292em;margin:0;font-size:1em;opacity:1;visibility:visible;transition:opacity 0.2s, visibility 0s linear 0s;overflow:auto;display:block}.h5p-hub .h5p-hub-navbar.h5p-hub-hidden{opacity:0;visibility:hidden;transition:opacity 0.2s, visibility 0s linear 0.2s}.h5p-hub .h5p-hub-result-header{font-size:1.042em;font-weight:600}.h5p-hub .h5p-hub-result-hits{font-size:0.8em;color:#697585;font-weight:400;padding-left:0.4em}.h5p-hub ul.h5p-hub-sort-by-list{margin-top:0.7844743em}.h5p-hub ul.h5p-hub-sort-by-list>li{float:left;margin-left:0.5em;user-select:none}.h5p-hub ul.h5p-hub-sort-by-list>li>a{display:block;color:#697585;padding:0.25em 0.75em;font-size:0.9em;text-decoration:none}.h5p-hub ul.h5p-hub-sort-by-list>li>a:hover{color:#3c434c}.h5p-hub ul.h5p-hub-sort-by-list>li>a:focus{box-shadow:0 0 1px 1px #1a93f4;outline:none}.h5p-hub ul.h5p-hub-sort-by-list>li>a.h5p-hub-highlight{font-weight:600;color:#000000;text-decoration:underline}.h5p-hub .h5p-hub-sort-by-header{float:left;padding:0.25em 0.75em 0.25em 0;font-size:0.9em;margin-top:0.871637586em}

.h5p-hub-content-selection-list{padding:2em 1em 0}.h5p-hub-content-selection-list .h5p-hub-header{position:relative;font-weight:bold;padding:0 1em;margin-bottom:0.5em}.h5p-hub-content-selection-list .h5p-hub-title{font-size:1.2em}.h5p-hub-content-selection-list .h5p-hub-action{position:absolute;right:1.5em;top:0}.h5p-hub-content-selection-list:last-child{padding-bottom:1em}

.h5p-hub .h5p-hub-filter-bar{background-color:#697585;font-size:1em;padding:0 0.833em 0.833em 1.125em;display:flex}.h5p-hub .h5p-hub-filter-bar .h5p-hub-filter-label{color:#ffffff;align-self:center}@media (max-width: 768px){.h5p-hub .h5p-hub-filter-bar{flex-direction:column}.h5p-hub .h5p-hub-filter-bar .h5p-hub-filter-label{align-self:start;margin-left:1em}}.h5p-hub .h5p-hub-filter-buttons{display:flex;flex-direction:row;flex-wrap:wrap}.h5p-hub .h5p-hub-filter-buttons li{outline:none;margin:0.5em 0 0.5em 0}.h5p-hub .h5p-hub-filter-buttons li:focus .h5p-hub-filter-button{box-shadow:0.06em 0 0.6em 0.1em #7bc1f9}.h5p-hub .h5p-hub-filter-buttons li:focus .h5p-hub-filter-button button{background-image:linear-gradient(0deg, #e3e3e3 0%, #fff 100%)}.h5p-hub .h5p-hub-clear-filters{flex-grow:1;align-self:center}@media (max-width: 768px){.h5p-hub .h5p-hub-clear-filters{align-self:start;margin-left:1em}}.h5p-hub .h5p-hub-clear-filters button{font-family:"Open Sans", sans-serif;align-items:center;float:right;display:flex;font-size:1em;background:none;color:#ffffff;font-size:0.88889em;cursor:pointer}.h5p-hub .h5p-hub-clear-filters button:focus{box-shadow:0.06em 0 0.6em 0.1em #7bc1f9;outline:none}.h5p-hub .h5p-hub-clear-filters button::before{font-family:'h5p-hub';content:"";font-style:normal;font-size:0.7em;margin-right:0.7em}

.h5p-hub .h5p-hub-filter-modal{background:rgba(71,79,90,0.6);top:14.7em;width:100%;height:100%;left:0;position:absolute}.h5p-hub .h5p-hub-filter-dialog-content{padding:1.5em}.h5p-hub .h5p-hub-filter-dialog{position:absolute;width:40em;max-width:100%;box-sizing:border-box;background-color:#ffffff;border-radius:0.25em;box-shadow:2px 5px 5px 1px rgba(0,0,0,0.1)}.h5p-hub .h5p-hub-filter-dialog .h5p-hub-apply-filters-button{background-color:#697585;text-align:center;width:100%;color:#ffffff;font-size:1.15em;font-family:'Open Sans', sans-serif;padding:1em;border-radius:0.25em;margin-top:1em;cursor:pointer}.h5p-hub .h5p-hub-filter-dialog .h5p-hub-apply-filters-button::before{font-family:'h5p-hub';content:"";font-style:normal;font-size:1em;margin:0 0.4em 0 0.4em;margin-right:1em}.h5p-hub .h5p-hub-filter-dialog .h5p-hub-header-text{color:#000000;font-size:1.25em;font-weight:bold;margin-bottom:1em}.h5p-hub .h5p-hub-filter-dialog .h5p-hub-loading:after{font-family:'h5p-hub';content:"";font-size:1em;line-height:1;width:1em;height:1em;padding:.833em;display:block;color:#7c8696;-webkit-animation:spin 2s linear infinite;animation:spin 2s linear infinite;width:1em;margin:0 auto}

.h5p-hub-message{position:relative;margin:.833em 0;padding:0.5em 1.250em;padding-right:2.250em}.h5p-hub-message .h5p-hub-title{font-size:1.042em;color:#1f2227}.h5p-hub-message .h5p-hub-message-content h2{margin-top:0.4165em;font-size:1.2em}.h5p-hub-message .h5p-hub-description{font-size:.95em;color:#697585}.h5p-hub-message .h5p-hub-message-header{margin:0;font-size:1.19em;line-height:1.9em}.h5p-hub-message .h5p-hub-message-body{font-size:0.925em;margin:0;padding-bottom:0.5em}.h5p-hub-message .h5p-hub-message-body ul.h5p-hub-message-item-list{list-style-type:disc;margin:1em}.h5p-hub-message .h5p-hub-message-content .h5p-hub-message-body{color:#697585}.h5p-hub-message .h5p-hub-get-help{margin-left:0.75em;color:#c62c15;text-decoration:none}.h5p-hub-message .h5p-hub-get-help:before{font-family:'h5p-hub';content:"";font-size:0.9em;margin-right:0.3em;display:inline-block}.h5p-hub-message .h5p-hub-message-close{cursor:pointer;display:block;position:absolute;background-color:transparent;color:#555;right:1.75em;line-height:2.75;font-size:0.75em;width:2.75em;text-align:center}.h5p-hub-message .h5p-hub-message-close:before{font-family:'h5p-hub';content:""}.h5p-hub-message .h5p-hub-message-close:hover{color:#333}.h5p-hub-message.h5p-hub-info{-webkit-box-shadow:0.167em 0 0 0 #00aa54 true;-moz-box-shadow:0.167em 0 0 0 #00aa54 true;box-shadow:0.167em 0 0 0 #00aa54 true;background:rgba(0,170,84,0.15);box-shadow:inset 0.167em 0 #00aa54}.h5p-hub-message.h5p-hub-info .h5p-hub-button{color:#00aa54;border:solid 1px;background-color:transparent;margin-top:1em}.h5p-hub-message.h5p-hub-warning{-webkit-box-shadow:0.167em 0 0 0 #fc3 true;-moz-box-shadow:0.167em 0 0 0 #fc3 true;box-shadow:0.167em 0 0 0 #fc3 true;background:rgba(255,204,51,0.15);box-shadow:inset 0.167em 0 #fc3}.h5p-hub-message.h5p-hub-warning .h5p-hub-button{color:#1f2227;border:solid 1px;background-color:transparent;margin-top:1em}.h5p-hub-message.h5p-hub-error{-webkit-box-shadow:0.167em 0 0 0 #c64750 true;-moz-box-shadow:0.167em 0 0 0 #c64750 true;box-shadow:0.167em 0 0 0 #c64750 true;background:rgba(198,71,80,0.15);box-shadow:inset 0.167em 0 #c64750}.h5p-hub-message.h5p-hub-error .h5p-hub-button{color:#c64750;border:solid 1px;background-color:transparent;margin-top:1em}.h5p-hub-message.h5p-hub-dismissible{overflow:auto;padding-right:3.5em}

.h5p-hub .h5p-hub-filter-button{margin-left:0.85em}.h5p-hub .h5p-hub-filter-button button{font-family:'Open Sans', sans-serif;font-weight:600;padding:0.25em 0.426em;-ms-user-select:none;user-select:none;display:flex;align-items:center;font-size:1em;box-shadow:2px 2px 3px 1px rgba(0,0,0,0.1);border-radius:2px;background-image:linear-gradient(0deg, #e3e3e3 0%, #fff 100%)}.h5p-hub .h5p-hub-filter-button button.h5p-hub-open{background-image:linear-gradient(0deg, #c2c6cf 0%, #dde2e6 100%)}.h5p-hub .h5p-hub-filter-button button.h5p-hub-filter-checked{background-image:linear-gradient(0deg, #f4ddcd 0%, #fbfffc 100%)}.h5p-hub .h5p-hub-filter-button button.h5p-hub-one-checked{background-image:linear-gradient(0deg, #f4ddcd 0%, #fbfffc 100%)}.h5p-hub .h5p-hub-filter-button button:hover{background-image:linear-gradient(0deg, #c2c6cf 0%, #dde2e6 100%);cursor:pointer}.h5p-hub .h5p-hub-filter-button button .h5p-hub-icon{margin-left:0.3em}@media (min-width: 576px){.h5p-hub .h5p-hub-filter-button button .h5p-hub-icon::after{content:"\e565";font-family:"h5p";font-style:normal;font-size:0.8em;margin-left:0.6em}}.h5p-hub .h5p-hub-filter-button button .h5p-hub-icon.h5p-hub-check::before{font-family:'h5p-hub';content:"";font-style:normal;font-size:0.7em;margin:0 0.4em 0 0.4em}

.h5p-hub .h5p-hub-checkbox-list{list-style:none}.h5p-hub .h5p-hub-checkbox-list.h5p-hub-animate-in-right{animation:move-from-right 0.35s}.h5p-hub .h5p-hub-checkbox-list.h5p-hub-animate-in-left{animation:move-from-left 0.35s}@keyframes move-from-right{0%{opacity:0;transform:translateX(30%)}30%{opacity:0}100%{opacity:1;transform:translateX(0)}}@keyframes move-from-left{0%{opacity:0;transform:translateX(-10%)}30%{opacity:0}100%{opacity:1;transform:translateX(0)}}

.h5p-hub .h5p-hub-checkbox{font-size:1.05em;padding:0.5em 0;font-weight:bold;display:flex;cursor:pointer;align-self:start;color:#4d5057;width:100%;position:relative}.h5p-hub .h5p-hub-checkbox:focus{box-shadow:0 0 0.4em 0.01em RGBA(26, 147, 244, 0.7);outline:none}.h5p-hub .h5p-hub-checkbox.h5p-hub-parent::after{font-family:'h5p-hub';content:"";font-family:"h5p";transform:rotate(90deg);font-style:normal;font-size:1.1em;display:block;align-self:center}.h5p-hub .h5p-hub-checkbox .h5p-hub-non-bold{font-weight:normal}.h5p-hub .h5p-hub-checkbox .h5p-hub-content{display:flex}.h5p-hub .h5p-hub-checkbox .h5p-hub-content .h5p-hub-icon::before{font-family:'h5p-hub';content:"";font-style:normal;font-weight:normal;font-size:1em;margin:0.4em 0.4em 0 0.4em;margin-right:1em;color:#697585}.h5p-hub .h5p-hub-checkbox.h5p-hub-checked .h5p-hub-content .h5p-hub-icon::before{font-family:'h5p-hub';content:"";font-style:normal;font-weight:normal;font-size:1em;margin:0.4em 0.4em 0 0.4em;margin-right:1em;color:#697585}

.h5p-hub .h5p-hub-bottom-line{border-bottom:2px solid #697585}.h5p-hub .h5p-hub-category-header{background-color:#f2f2f2;color:#313131;font-style:italic;padding:0 0 0 0.9em;font-size:0.81788em;text-transform:capitalize}

.h5p-hub .h5p-hub-search-button{width:100%}.h5p-hub .h5p-hub-search-button .h5p-hub-search-field{position:relative}.h5p-hub .h5p-hub-search-button .h5p-hub-search-field::before{position:absolute;font-family:'h5p-hub';content:"";font-style:normal;font-size:1em;margin-left:1em;color:#b7b8ba;top:50%;transform:translateY(-50%)}.h5p-hub .h5p-hub-search-button .h5p-hub-search-field .h5p-hub-icon-arrow{position:absolute;font-size:1.30353em;top:0.7em;right:1em;cursor:pointer;color:#000000}.h5p-hub .h5p-hub-search-button .h5p-hub-search-field .h5p-hub-icon-arrow::before{content:"\e565";font-family:"h5p";font-style:normal;font-size:1.25em}.h5p-hub .h5p-hub-filter-search-bar{border:0.04545em solid #e6e6e8;padding:0.7em;font-style:italic;font-size:1.25em;width:100%;box-sizing:border-box;padding-left:2.25em}.h5p-hub .h5p-hub-filter-search-bar::placeholder{color:#b7b8ba}.h5p-hub .h5p-hub-filter-search-bar:focus{border-color:#3ba0f2;background-color:#fff;outline:none}

.h5p-hub .h5p-hub-search-filter{outline:0.06815em solid #bebdcd;position:relative}.h5p-hub .h5p-hub-search-filter .h5p-hub-clear-button{background:none;position:absolute;top:0;padding:1.2em;right:3em;color:#757575}.h5p-hub .h5p-hub-search-filter .h5p-hub-clear-button::before{font-family:'h5p-hub';content:"";font-style:normal;font-size:1em}.h5p-hub .h5p-hub-search-filter .h5p-hub-navigate-parent{font-size:1.25em;padding:0.3534em;display:flex;font-weight:bold;font-family:"Open Sans", sans-serif;border:1px solid #e6e6e8;align-items:center}.h5p-hub .h5p-hub-search-filter .h5p-hub-navigate-parent button{background-color:#ffffff;position:relative;font-style:normal;font-size:0.8em;border-radius:50%;margin-right:0.4em;align-self:center;padding:0.5em;width:2.75em;height:2.75em}.h5p-hub .h5p-hub-search-filter .h5p-hub-navigate-parent button::before{font-family:'h5p-hub';content:"";transform:rotate(180deg);font-weight:normal;display:inline-block}.h5p-hub .h5p-hub-search-filter .h5p-hub-navigate-parent button:hover{background-color:#dadada;cursor:pointer}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list{overflow:hidden;overflow-y:scroll;max-height:25em}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list li{border:1px solid #e6e6e8}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list li .h5p-hub-content{padding:0.3em}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list li .h5p-hub-content .h5p-hub-label-text{margin-right:0.2em}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list li:focus{border-color:#3ba0f2;background-color:#fff;outline:none;box-shadow:none}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list li:hover{background-color:#f2f2f2}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list li.h5p-hub-highlighted{border-color:#3ba0f2;background-color:#fff;box-shadow:none}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list li:hover{background-color:rgba(230,230,232,0.25)}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list::-webkit-scrollbar{background-color:rgba(105,117,133,0.25);width:0.54525em;border-radius:5.45256em}.h5p-hub .h5p-hub-search-filter .h5p-hub-checkbox-list::-webkit-scrollbar-thumb{background-color:#697585;background-size:24px 100%;border-radius:5.45256em}

.h5p-hub .h5p-hub-downloading-modal-overlay{position:absolute;top:0;left:0;right:0;bottom:0;background-color:rgba(255,255,255,0.8)}.h5p-hub .h5p-hub-downloading-modal{position:absolute;width:30rem;max-width:90%;border-radius:0.25rem;background-color:#ffffff;box-shadow:0px 1px 82px 0px rgba(0,0,0,0.2);left:50%;top:50%;transform:translate(-50%, -50%);transition:background-color 0.3s ease-in;display:flex;flex-wrap:wrap;flex-direction:row;justify-content:center;align-items:center;outline:none;box-sizing:border-box;padding:1em}.h5p-hub .h5p-hub-downloading-modal .h5p-hub-spinner{margin:1em;animation:spin 0.5s linear infinite}.h5p-hub .h5p-hub-downloading-modal span.h5p-hub-downloading-message{font-family:'Open Sans', sans-serif;font-weight:bold;font-size:2rem}.h5p-hub .h5p-hub-downloading-modal span.h5p-hub-downloading-message:focus{outline:none}@keyframes spin{from{transform:rotate(0deg)}to{transform:rotate(360deg)}}

.h5p-hub-search-wrapper{padding:.833em 1.125em;background-color:#697585;position:relative}.h5p-hub-search-wrapper .h5p-hub-border-wrap{padding:3px;background:#fff;border-radius:1.25em;font-size:1.15em}.h5p-hub-search-wrapper .h5p-hub-icon-search{position:absolute;font-size:1em;top:1.4em;right:2.2em;cursor:pointer;color:#4f5662}.h5p-hub-search-bar{border:1px solid #bfc0c0;padding:0.5em 0.5em 0.5em 1.2em;font-style:italic;font-weight:400;font-size:1em;box-shadow:inset 0 0 10px rgba(0,0,0,0.4);color:#000000;border-radius:1.25em;width:100%;box-sizing:border-box}.h5p-hub-search-bar::placeholder{color:#000000}.h5p-hub-search-bar:focus{border-color:#3ba0f2;background-color:#fff;box-shadow:inset 0 0 10px rgba(0,0,0,0.2);outline:none}.h5p-hub-search-bar:focus::placeholder{color:#697585}.h5p-hub-search-bar:before{background:none;border:4px solid #fff;content:"";display:block;position:absolute;top:4px;left:4px;right:4px;bottom:4px;pointer-events:none}.h5p-hub-search-bar::-ms-clear{display:none;width:0;height:0}

.h5p-hub .h5p-hub-button.h5p-hub-button-orange{background:transparent;color:#d16b17;border:0.125em solid #d16b17}.h5p-hub .h5p-hub-button.h5p-hub-button-orange:hover{color:#fff;background-color:#d16b17;border-color:#d16b17}.h5p-hub .h5p-hub-button.h5p-hub-button-orange:active{color:#fff;background-color:#c26d13;border-color:#c26d13}

.h5p-hub-lightbox{position:absolute;top:0;left:0;right:0;bottom:0;background:rgba(0,0,0,0.8)}.h5p-hub-lightbox-inner{position:absolute;left:50%;top:50%;transform:translate(-50%, -50%);width:100%;box-sizing:border-box;padding:2em;text-align:center}.h5p-hub-lightbox-inner:focus{outline:0}.h5p-hub-lightbox-close{position:relative;top:-1.5em;right:0;float:right;width:1.2em;height:1.2em;border-radius:50%;color:#fff;font-size:2em;line-height:1;text-align:center;cursor:pointer;z-index:1;user-select:none;border:2px solid transparent;transition:background 0.2s}.h5p-hub-lightbox-close:focus{border-color:#7bc1f9;box-shadow:0.06em 0 0.6em 0.1em #7bc1f9;outline:0}.h5p-hub-lightbox-close:hover{background:rgba(132,143,158,0.8)}.h5p-hub-license .h5p-hub-lightbox-close:hover{background:rgba(165,175,189,0.43)}.h5p-hub-lightbox-close:before{font-family:'h5p-hub';content:"";font-size:0.6em}

.h5p-hub-carousel{width:100%;box-sizing:border-box;position:relative}.h5p-hub-carousel .h5p-hub-scroller{overflow:hidden;width:calc(100% - 5em);margin-left:2.5em}.h5p-hub-carousel ul{width:100%;padding:0;margin:0 auto;word-wrap:break-word;transition:margin 0.3s linear;display:flex;flex-direction:row;flex-wrap:nowrap;align-items:center}.h5p-hub-carousel li{padding:0 .3125em;width:20%;margin:0;list-style-type:none;float:left;box-sizing:border-box;overflow:hidden}.h5p-hub-carousel li img{max-width:100%;box-sizing:border-box;border:1px solid #ced6e3}.h5p-hub-carousel li img.h5p-hub-selectable{cursor:pointer}.h5p-hub-carousel li:focus{outline:0}.h5p-hub-carousel li:focus img{border-color:#0a78d1}.h5p-hub-carousel .h5p-hub-progress{color:#fff;position:relative;bottom:-2em;text-align:center}.h5p-hub-carousel .h5p-hub-navigation{width:2.1em;height:2.1em;line-height:2.1em;position:absolute;border-radius:50%;color:white;background-color:rgba(105,117,133,0.8);cursor:pointer;border:2px solid transparent;top:50%;transform:translateY(-50%)}.h5p-hub-carousel .h5p-hub-navigation[disabled]{background-color:rgba(189,195,203,0.8);color:rgba(255,255,255,0.6);pointer-events:none}.h5p-hub-carousel .h5p-hub-navigation:before{display:inline-block;line-height:1em;height:1em}.h5p-hub-carousel .h5p-hub-navigation:hover{background-color:rgba(132,143,158,0.8)}.h5p-hub-carousel .h5p-hub-navigation:focus{outline:0;box-shadow:0.06em 0 0.6em 0.1em #7bc1f9;border-color:#7bc1f9}.h5p-hub-carousel .h5p-hub-prev{left:0}.h5p-hub-carousel .h5p-hub-prev:before{font-family:'h5p-hub';content:""}.h5p-hub-carousel .h5p-hub-next{right:0}.h5p-hub-carousel .h5p-hub-next:before{font-family:'h5p-hub';content:"";transform:rotate(180deg)}.h5p-hub-lightbox-inner .h5p-hub-carousel ul{transition-duration:0.15s}.h5p-hub-lightbox-inner .h5p-hub-carousel li{padding:0}.h5p-hub-lightbox-inner .h5p-hub-carousel li img{border-color:transparent;background:#fff}.h5p-hub-lightbox-inner .h5p-hub-carousel .h5p-hub-navigation{font-size:1.2em}.h5p-hub-lightbox-inner .h5p-hub-carousel .h5p-hub-navigation:focus{background-color:transparent}.h5p-hub-lightbox-inner .h5p-hub-carousel .h5p-hub-next{right:-0.75em}.h5p-hub-lightbox-inner .h5p-hub-carousel .h5p-hub-prev{left:-0.75em}

.h5p-hub-text-details .h5p-read-more span{outline:none}.h5p-hub-text-details .h5p-read-more .h5p-hub-link{color:#1f2227;outline:none;margin-left:0.5em}.h5p-hub-text-details .h5p-read-more .h5p-hub-link:hover{color:#000}.h5p-hub-text-details .h5p-read-more .h5p-hub-link:focus{box-shadow:0 0 1px 1px #1a93f4}

.h5p-hub-accordion{border:1px solid #ced6e3;border-bottom:0}.h5p-hub-accordion .h5p-hub-icon-accordion-arrow{margin-top:-4px;font-size:0.7em;margin-right:.833em;margin-left:.833em;display:inline-block;vertical-align:middle;transition:transform 0.1s ease-out}.h5p-hub-accordion [aria-expanded="true"] .h5p-hub-icon-accordion-arrow{transform:rotate(90deg)}.h5p-hub-accordion-toggler{display:block;box-sizing:border-box;text-decoration:none;width:100%;padding:.833em;color:#1f2227}.h5p-hub-accordion-heading{border-bottom:1px solid #ced6e3;background-color:#f3f3f3;cursor:pointer}.h5p-hub-accordion-region{border-bottom:1px solid #ced6e3}.h5p-hub-accordion-region .h5p-hub-panel-body{padding:1em;line-height:1.7em;font-size:0.95em}

.h5p-hub .publisher-info{overflow:auto}.h5p-hub .publisher-info .h5p-hub-publisher-image{max-width:10em;max-height:10em;height:auto;width:auto;margin-right:1em;float:left}.h5p-hub .h5p-hub-publisher-image{max-width:150px}

.h5p-hub .h5p-hub-content-detail{background-color:white;padding:1.3em;display:inherit;position:absolute;top:0;left:0;visibility:hidden;outline:none;overflow:auto;width:100%;height:100%;box-sizing:border-box;transform:translateX(100%);transition:transform 0.2s ease-out, visibility 0s linear 0.2s}.h5p-hub .h5p-hub-content-detail::-webkit-scrollbar{background-color:#f3f3f3;width:0.750em}.h5p-hub .h5p-hub-content-detail::-webkit-scrollbar-thumb{background:#5e6978 url("data:image/svg+xml;base64,PHN2ZyBpZD0ic2Nyb2xsYmFyX2hhbmRsZSIgZGF0YS1uYW1lPSJzY3JvbGxiYXIgaGFuZGxlIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogIDxkZWZzPg0KICAgIDxzdHlsZT4NCiAgICAgIC5jbHMtMSB7DQogICAgICAgIGZpbGw6ICNjZWQ2ZTM7DQogICAgICB9DQoNCiAgICAgIC5jbHMtMiB7DQogICAgICAgIGZpbGw6IG5vbmU7DQogICAgICB9DQogICAgPC9zdHlsZT4NCiAgPC9kZWZzPg0KICA8dGl0bGU+c2Nyb2xsYmFyLWJhcnMtaWNvbjwvdGl0bGU+DQogIDxwYXRoIGNsYXNzPSJjbHMtMSIgZD0iTTE1LjI4LDguNWMwLC4yNC0uMS40My0uMjQuNDNIOS42MmMtLjE0LDAtLjI0LS4xOS0uMjQtLjQzVjcuNjVjMC0uMjQuMS0uNDMuMjQtLjQzSDE1Yy4xNCwwLC4yNC4xOS4yNC40M1ptMCwzLjQxYzAsLjI0LS4xLjQzLS4yNC40M0g5LjYyYy0uMTQsMC0uMjQtLjE5LS4yNC0uNDN2LS44NWMwLS4yNC4xLS40My4yNC0uNDNIMTVjLjE0LDAsLjI0LjE5LjI0LjQzWm0wLDMuNDNjMCwuMjQtLjEuNDMtLjI0LjQzSDkuNjJjLS4xNCwwLS4yNC0uMTktLjI0LS40M3YtLjg1YzAtLjI0LjEtLjQzLjI0LS40M0gxNWMuMTQsMCwuMjQuMTkuMjQuNDNaIi8+DQogIDxyZWN0IGNsYXNzPSJjbHMtMiIgd2lkdGg9IjI0IiBoZWlnaHQ9IjI0Ii8+DQo8L3N2Zz4NCg==") no-repeat center;background-size:24px 100%}.h5p-hub .h5p-hub-content-detail::-webkit-scrollbar-button:vertical{background-size:20px 20px;background-position-x:-4px;background-position-y:-3px;background-color:#5e6978;background-repeat:no-repeat}.h5p-hub .h5p-hub-content-detail::-webkit-scrollbar-button:vertical:decrement{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQoJPHN0eWxlIHR5cGU9InRleHQvY3NzIj4qIHtmaWxsOiAjY2VkNmUzO30gLnN0MHtmaWxsOm5vbmU7fTwvc3R5bGU+DQoJPHRpdGxlPmFycm93LXVwPC90aXRsZT4NCgk8cGF0aCBkPSJNOCAxMy45bDAuNCAwLjRjMC4zIDAuMyAwLjggMC4zIDEuMSAwbDIuNC0yLjQgMi40IDIuNGMwLjMgMC4zIDAuOCAwLjMgMS4xIDBsMC40LTAuNGMwLjMtMC4zIDAuMy0wLjggMC0xLjEgMCAwIDAgMCAwIDBsLTMuMy0zLjNjLTAuMy0wLjMtMC44LTAuMy0xLjEgMCAwIDAgMCAwIDAgMEw4IDEyLjhDNy43IDEzLjEgNy43IDEzLjUgOCAxMy45IDggMTMuOCA4IDEzLjggOCAxMy45eiIvPg0KCTxyZWN0IGNsYXNzPSJzdDAiIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIvPg0KPC9zdmc+")}.h5p-hub .h5p-hub-content-detail::-webkit-scrollbar-button:vertical:increment{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogICAgPHN0eWxlPioge2ZpbGw6ICNjZWQ2ZTM7fSAuY2xzLTEge2ZpbGw6IG5vbmU7fTwvc3R5bGU+DQogIDx0aXRsZT5hcnJvdy1kb3duPC90aXRsZT4NCiAgPHBhdGggZD0iTTE2LDEwLjE1bC0uMzUtLjM1YS43OS43OSwwLDAsMC0xLjA4LDBsLTIuNDMsMi40M0w5LjY5LDkuNzlhLjc4Ljc4LDAsMCwwLTEuMDgsMGwtLjM2LjM2YS43Ni43NiwwLDAsMCwwLDEuMDhsMy4zMiwzLjMyYS43Ni43NiwwLDAsMCwxLjA4LDBMMTYsMTEuMjNhLjc2Ljc2LDAsMCwwLDAtMS4wOFoiLz4NCiAgPHJlY3QgY2xhc3M9ImNscy0xIiB3aWR0aD0iMjQiIGhlaWdodD0iMjQiLz4NCjwvc3ZnPg0K")}.h5p-hub .h5p-hub-content-detail.h5p-hub-show{transform:translateX(0);visibility:visible;transition:transform 0.2s ease-out, visibility 0s linear 0s}.h5p-hub .h5p-hub-content-detail .h5p-hub-panel[aria-hidden="false"]{display:none}.h5p-hub .h5p-hub-content-detail .h5p-hub-container{max-width:100%;margin-left:auto;margin-right:auto;margin-top:.833em;margin-bottom:1.666em;padding-bottom:1em;display:flex;flex-wrap:wrap;flex-direction:column}.h5p-hub .h5p-hub-content-detail .h5p-hub-container:after{content:" ";display:block;clear:both}@media (min-width: 576px){.h5p-hub .h5p-hub-content-detail .h5p-hub-container{flex-direction:row}}.h5p-hub .h5p-hub-content-detail .h5p-hub-container .h5p-hub-image-wrapper{flex-shrink:0;order:1;display:flex;justify-content:center;align-items:center}@media (min-width: 768px){.h5p-hub .h5p-hub-content-detail .h5p-hub-container .h5p-hub-image-wrapper{justify-content:normal;align-items:normal;margin-top:2em}}.h5p-hub .h5p-hub-content-detail .h5p-hub-container .h5p-hub-image-wrapper img{width:16.5em;height:12.375em}@media (min-width: 768px){.h5p-hub .h5p-hub-content-detail .h5p-hub-container .h5p-hub-image-wrapper img{width:15.5em;height:11.625em}}.h5p-hub .h5p-hub-content-detail .h5p-hub-container>.h5p-hub-text-details{order:2;flex-grow:1;margin:2em}.h5p-hub .h5p-hub-content-detail .h5p-hub-container>.h5p-hub-text-details .h5p-hub-info-list{order:2}@media (min-width: 576px){.h5p-hub .h5p-hub-content-detail .h5p-hub-container>.h5p-hub-text-details{order:3;flex-basis:100%}.h5p-hub .h5p-hub-content-detail .h5p-hub-container>.h5p-hub-text-details .h5p-hub-info-list{display:none}}@media (min-width: 768px){.h5p-hub .h5p-hub-content-detail .h5p-hub-container>.h5p-hub-text-details{order:2;flex-basis:0;margin:2em 2em 0}}.h5p-hub .h5p-hub-content-detail .h5p-hub-container>.h5p-hub-text-details h2{margin-top:0;font-size:1.458em;line-height:1.458em}.h5p-hub .h5p-hub-content-detail .h5p-hub-container>.h5p-hub-info-list{order:3;flex-shrink:0;min-width:15em;margin:1em;display:none}@media (min-width: 576px){.h5p-hub .h5p-hub-content-detail .h5p-hub-container>.h5p-hub-info-list{display:block;order:2;flex-grow:1}}@media (min-width: 768px){.h5p-hub .h5p-hub-content-detail .h5p-hub-container>.h5p-hub-info-list{order:3;flex-grow:0}}.h5p-hub .h5p-hub-content-detail .h5p-hub-title{outline:0}.h5p-hub .h5p-hub-content-detail .h5p-hub-title.h5p-hub-reviewed:after{font-size:0.8em;font-family:"h5p-hub";font-weight:normal;content:"\e90b";color:#359d94;position:relative;top:-0.2em;margin-left:0.3em}.h5p-hub .h5p-hub-content-detail .h5p-hub-back-button{color:#474f5a;font-size:1.3em;display:inline-block;background:transparent;border:none;text-decoration:none;outline:none;transform:rotate(180deg)}.h5p-hub .h5p-hub-content-detail .h5p-hub-back-button:hover{color:#000}.h5p-hub .h5p-hub-content-detail .h5p-hub-back-button:focus{box-shadow:0 0 1px 1px #1a93f4}.h5p-hub .h5p-hub-content-detail .h5p-hub-back-button.h5p-hub-icon-arrow-thick:before{cursor:pointer}.h5p-hub .h5p-hub-content-detail .h5p-hub-button-bar{text-align:right;margin:1.666em 0}.h5p-hub .h5p-hub-content-detail .h5p-hub-button-download-content{display:inline-flex;align-items:center}.h5p-hub .h5p-hub-content-detail .h5p-hub-button-download-content::before{font-family:"h5p-hub";font-size:0.7em;content:"\e90c";display:inline-block;margin-right:1em;transform:rotate(-90deg)}.h5p-hub .h5p-hub-content-detail .h5p-hub-lightbox-inner.h5p-hub-license::-webkit-scrollbar{background-color:#f3f3f3;width:0.750em}.h5p-hub .h5p-hub-content-detail .h5p-hub-lightbox-inner.h5p-hub-license::-webkit-scrollbar-thumb{background:#5e6978 url("data:image/svg+xml;base64,PHN2ZyBpZD0ic2Nyb2xsYmFyX2hhbmRsZSIgZGF0YS1uYW1lPSJzY3JvbGxiYXIgaGFuZGxlIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogIDxkZWZzPg0KICAgIDxzdHlsZT4NCiAgICAgIC5jbHMtMSB7DQogICAgICAgIGZpbGw6ICNjZWQ2ZTM7DQogICAgICB9DQoNCiAgICAgIC5jbHMtMiB7DQogICAgICAgIGZpbGw6IG5vbmU7DQogICAgICB9DQogICAgPC9zdHlsZT4NCiAgPC9kZWZzPg0KICA8dGl0bGU+c2Nyb2xsYmFyLWJhcnMtaWNvbjwvdGl0bGU+DQogIDxwYXRoIGNsYXNzPSJjbHMtMSIgZD0iTTE1LjI4LDguNWMwLC4yNC0uMS40My0uMjQuNDNIOS42MmMtLjE0LDAtLjI0LS4xOS0uMjQtLjQzVjcuNjVjMC0uMjQuMS0uNDMuMjQtLjQzSDE1Yy4xNCwwLC4yNC4xOS4yNC40M1ptMCwzLjQxYzAsLjI0LS4xLjQzLS4yNC40M0g5LjYyYy0uMTQsMC0uMjQtLjE5LS4yNC0uNDN2LS44NWMwLS4yNC4xLS40My4yNC0uNDNIMTVjLjE0LDAsLjI0LjE5LjI0LjQzWm0wLDMuNDNjMCwuMjQtLjEuNDMtLjI0LjQzSDkuNjJjLS4xNCwwLS4yNC0uMTktLjI0LS40M3YtLjg1YzAtLjI0LjEtLjQzLjI0LS40M0gxNWMuMTQsMCwuMjQuMTkuMjQuNDNaIi8+DQogIDxyZWN0IGNsYXNzPSJjbHMtMiIgd2lkdGg9IjI0IiBoZWlnaHQ9IjI0Ii8+DQo8L3N2Zz4NCg==") no-repeat center;background-size:24px 100%}.h5p-hub .h5p-hub-content-detail .h5p-hub-lightbox-inner.h5p-hub-license::-webkit-scrollbar-button:vertical{background-size:20px 20px;background-position-x:-4px;background-position-y:-3px;background-color:#5e6978;background-repeat:no-repeat}.h5p-hub .h5p-hub-content-detail .h5p-hub-lightbox-inner.h5p-hub-license::-webkit-scrollbar-button:vertical:decrement{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQoJPHN0eWxlIHR5cGU9InRleHQvY3NzIj4qIHtmaWxsOiAjY2VkNmUzO30gLnN0MHtmaWxsOm5vbmU7fTwvc3R5bGU+DQoJPHRpdGxlPmFycm93LXVwPC90aXRsZT4NCgk8cGF0aCBkPSJNOCAxMy45bDAuNCAwLjRjMC4zIDAuMyAwLjggMC4zIDEuMSAwbDIuNC0yLjQgMi40IDIuNGMwLjMgMC4zIDAuOCAwLjMgMS4xIDBsMC40LTAuNGMwLjMtMC4zIDAuMy0wLjggMC0xLjEgMCAwIDAgMCAwIDBsLTMuMy0zLjNjLTAuMy0wLjMtMC44LTAuMy0xLjEgMCAwIDAgMCAwIDAgMEw4IDEyLjhDNy43IDEzLjEgNy43IDEzLjUgOCAxMy45IDggMTMuOCA4IDEzLjggOCAxMy45eiIvPg0KCTxyZWN0IGNsYXNzPSJzdDAiIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIvPg0KPC9zdmc+")}.h5p-hub .h5p-hub-content-detail .h5p-hub-lightbox-inner.h5p-hub-license::-webkit-scrollbar-button:vertical:increment{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogICAgPHN0eWxlPioge2ZpbGw6ICNjZWQ2ZTM7fSAuY2xzLTEge2ZpbGw6IG5vbmU7fTwvc3R5bGU+DQogIDx0aXRsZT5hcnJvdy1kb3duPC90aXRsZT4NCiAgPHBhdGggZD0iTTE2LDEwLjE1bC0uMzUtLjM1YS43OS43OSwwLDAsMC0xLjA4LDBsLTIuNDMsMi40M0w5LjY5LDkuNzlhLjc4Ljc4LDAsMCwwLTEuMDgsMGwtLjM2LjM2YS43Ni43NiwwLDAsMCwwLDEuMDhsMy4zMiwzLjMyYS43Ni43NiwwLDAsMCwxLjA4LDBMMTYsMTEuMjNhLjc2Ljc2LDAsMCwwLDAtMS4wOFoiLz4NCiAgPHJlY3QgY2xhc3M9ImNscy0xIiB3aWR0aD0iMjQiIGhlaWdodD0iMjQiLz4NCjwvc3ZnPg0K")}

.h5p-hub .h5p-hub-info-list{max-width:20em}.h5p-hub .h5p-hub-info-list li{font-size:0.8em;border-bottom:1px solid #dadada;padding:0.85em 0.5em}.h5p-hub .h5p-hub-info-list li .h5p-hub-label{color:#72757c;margin-right:0.3em}.h5p-hub .h5p-hub-info-list li .h5p-hub-content{color:#242424;font-weight:bold}.h5p-hub .h5p-hub-info-list li .h5p-hub-capitalize{text-transform:capitalize}.h5p-hub .h5p-hub-info-list li:last-child{border-bottom:none}

.h5p-hub .h5p-hub-short-license-info h3{display:inline-block}.h5p-hub .h5p-hub-short-license-info .h5p-hub-short-license-read-more{border:0;background-color:transparent;cursor:pointer}.h5p-hub .h5p-hub-short-license-info a.h5p-hub-short-license-read-more{text-decoration:none}.h5p-hub .h5p-hub-short-license-info .h5p-hub-short-license-read-more:before{font-family:'h5p-hub';content:"";margin-left:0.5em}

.h5p-hub .h5p-hub-reuse-content-result{position:relative;overflow-y:auto;overflow-x:hidden;max-height:56em;box-shadow:0 0.4em 0.4em -0.2em rgba(79,87,99,0.2) inset}.h5p-hub .h5p-hub-reuse-content-result::-webkit-scrollbar{background-color:#F3F3F3;width:0.750em}.h5p-hub .h5p-hub-reuse-content-result::-webkit-scrollbar-thumb{background:#5e6978 url("data:image/svg+xml;base64,PHN2ZyBpZD0ic2Nyb2xsYmFyX2hhbmRsZSIgZGF0YS1uYW1lPSJzY3JvbGxiYXIgaGFuZGxlIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogIDxkZWZzPg0KICAgIDxzdHlsZT4NCiAgICAgIC5jbHMtMSB7DQogICAgICAgIGZpbGw6ICNjZWQ2ZTM7DQogICAgICB9DQoNCiAgICAgIC5jbHMtMiB7DQogICAgICAgIGZpbGw6IG5vbmU7DQogICAgICB9DQogICAgPC9zdHlsZT4NCiAgPC9kZWZzPg0KICA8dGl0bGU+c2Nyb2xsYmFyLWJhcnMtaWNvbjwvdGl0bGU+DQogIDxwYXRoIGNsYXNzPSJjbHMtMSIgZD0iTTE1LjI4LDguNWMwLC4yNC0uMS40My0uMjQuNDNIOS42MmMtLjE0LDAtLjI0LS4xOS0uMjQtLjQzVjcuNjVjMC0uMjQuMS0uNDMuMjQtLjQzSDE1Yy4xNCwwLC4yNC4xOS4yNC40M1ptMCwzLjQxYzAsLjI0LS4xLjQzLS4yNC40M0g5LjYyYy0uMTQsMC0uMjQtLjE5LS4yNC0uNDN2LS44NWMwLS4yNC4xLS40My4yNC0uNDNIMTVjLjE0LDAsLjI0LjE5LjI0LjQzWm0wLDMuNDNjMCwuMjQtLjEuNDMtLjI0LjQzSDkuNjJjLS4xNCwwLS4yNC0uMTktLjI0LS40M3YtLjg1YzAtLjI0LjEtLjQzLjI0LS40M0gxNWMuMTQsMCwuMjQuMTkuMjQuNDNaIi8+DQogIDxyZWN0IGNsYXNzPSJjbHMtMiIgd2lkdGg9IjI0IiBoZWlnaHQ9IjI0Ii8+DQo8L3N2Zz4NCg==") no-repeat center;background-size:24px 100%}.h5p-hub .h5p-hub-reuse-content-result::-webkit-scrollbar-button:vertical{background-size:20px 20px;background-position-x:-4px;background-position-y:-3px;background-color:#5e6978;background-repeat:no-repeat}.h5p-hub .h5p-hub-reuse-content-result::-webkit-scrollbar-button:vertical:decrement{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQoJPHN0eWxlIHR5cGU9InRleHQvY3NzIj4qIHtmaWxsOiAjY2VkNmUzO30gLnN0MHtmaWxsOm5vbmU7fTwvc3R5bGU+DQoJPHRpdGxlPmFycm93LXVwPC90aXRsZT4NCgk8cGF0aCBkPSJNOCAxMy45bDAuNCAwLjRjMC4zIDAuMyAwLjggMC4zIDEuMSAwbDIuNC0yLjQgMi40IDIuNGMwLjMgMC4zIDAuOCAwLjMgMS4xIDBsMC40LTAuNGMwLjMtMC4zIDAuMy0wLjggMC0xLjEgMCAwIDAgMCAwIDBsLTMuMy0zLjNjLTAuMy0wLjMtMC44LTAuMy0xLjEgMCAwIDAgMCAwIDAgMEw4IDEyLjhDNy43IDEzLjEgNy43IDEzLjUgOCAxMy45IDggMTMuOCA4IDEzLjggOCAxMy45eiIvPg0KCTxyZWN0IGNsYXNzPSJzdDAiIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIvPg0KPC9zdmc+")}.h5p-hub .h5p-hub-reuse-content-result::-webkit-scrollbar-button:vertical:increment{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogICAgPHN0eWxlPioge2ZpbGw6ICNjZWQ2ZTM7fSAuY2xzLTEge2ZpbGw6IG5vbmU7fTwvc3R5bGU+DQogIDx0aXRsZT5hcnJvdy1kb3duPC90aXRsZT4NCiAgPHBhdGggZD0iTTE2LDEwLjE1bC0uMzUtLjM1YS43OS43OSwwLDAsMC0xLjA4LDBsLTIuNDMsMi40M0w5LjY5LDkuNzlhLjc4Ljc4LDAsMCwwLTEuMDgsMGwtLjM2LjM2YS43Ni43NiwwLDAsMCwwLDEuMDhsMy4zMiwzLjMyYS43Ni43NiwwLDAsMCwxLjA4LDBMMTYsMTEuMjNhLjc2Ljc2LDAsMCwwLDAtMS4wOFoiLz4NCiAgPHJlY3QgY2xhc3M9ImNscy0xIiB3aWR0aD0iMjQiIGhlaWdodD0iMjQiLz4NCjwvc3ZnPg0K")}.h5p-hub .h5p-hub-reuse-content-result>.h5p-hub-content-list .h5p-hub-fetching-failed{margin-top:2em;margin-left:2em}.h5p-hub .h5p-hub-reuse-content-container{position:relative}.h5p-hub .h5p-hub-content-list-hidden{visibility:hidden}

.h5p-hub .h5p-hub-content-type-list .h5p-hub-media .h5p-hub-media-body{overflow:visible;zoom:1;width:10000px}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media-left{display:table-cell;vertical-align:top}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media-body{display:table-cell;vertical-align:middle}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media-left{padding-right:.833em}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media-object{width:50px}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media-heading{margin-top:0;float:left;margin-right:0.5em}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media-text-icon{float:left;height:15px}.h5p-hub .h5p-hub-content-type-list .h5p-hub-button{min-width:4em;padding:0.65em 1.5em;float:right}.h5p-hub .h5p-hub-content-type-list .h5p-hub-description{color:#697585;font-size:.95em;float:left;clear:left}.h5p-hub .h5p-hub-content-type-list .h5p-hub-content-type-update-info{margin-right:0;color:#697585;font-size:0.8em;font-style:italic;float:left}.h5p-hub .h5p-hub-content-type-list .h5p-hub-content-type-update-info:before{font-family:'h5p-hub';content:"";font-style:normal;font-size:0.8em;margin-right:0.4em}.h5p-hub .h5p-hub-content-type-list .h5p-hub-button-install:before{font-family:'h5p-hub';content:"";display:inline-block;transform:rotate(-90deg);margin-right:0.7em;font-size:0.9em}@media (min-width: 768px){.h5p-hub .h5p-hub-content-type-list .h5p-hub-button{margin-top:0.25em}.h5p-hub .h5p-hub-content-type-list .h5p-hub-content-type-update-info{margin-right:2em;margin-top:1em;float:right}}@media (min-width: 576px){.h5p-hub .h5p-hub-content-type-list .h5p-hub-media-object{width:100px}}

.h5p-hub .h5p-hub-content-type-list{overflow:auto;height:37.422137em;opacity:1;transition:opacity 0.2s, visibility 0s linear 0s}.h5p-hub .h5p-hub-content-type-list[aria-hidden="true"]{display:inherit;opacity:0;visibility:hidden;transition:opacity 0.2s, visibility 0s linear 0.2s}.h5p-hub .h5p-hub-content-type-list::-webkit-scrollbar{background-color:#F3F3F3;width:0.750em}.h5p-hub .h5p-hub-content-type-list::-webkit-scrollbar-thumb{background:#5e6978 url("data:image/svg+xml;base64,PHN2ZyBpZD0ic2Nyb2xsYmFyX2hhbmRsZSIgZGF0YS1uYW1lPSJzY3JvbGxiYXIgaGFuZGxlIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogIDxkZWZzPg0KICAgIDxzdHlsZT4NCiAgICAgIC5jbHMtMSB7DQogICAgICAgIGZpbGw6ICNjZWQ2ZTM7DQogICAgICB9DQoNCiAgICAgIC5jbHMtMiB7DQogICAgICAgIGZpbGw6IG5vbmU7DQogICAgICB9DQogICAgPC9zdHlsZT4NCiAgPC9kZWZzPg0KICA8dGl0bGU+c2Nyb2xsYmFyLWJhcnMtaWNvbjwvdGl0bGU+DQogIDxwYXRoIGNsYXNzPSJjbHMtMSIgZD0iTTE1LjI4LDguNWMwLC4yNC0uMS40My0uMjQuNDNIOS42MmMtLjE0LDAtLjI0LS4xOS0uMjQtLjQzVjcuNjVjMC0uMjQuMS0uNDMuMjQtLjQzSDE1Yy4xNCwwLC4yNC4xOS4yNC40M1ptMCwzLjQxYzAsLjI0LS4xLjQzLS4yNC40M0g5LjYyYy0uMTQsMC0uMjQtLjE5LS4yNC0uNDN2LS44NWMwLS4yNC4xLS40My4yNC0uNDNIMTVjLjE0LDAsLjI0LjE5LjI0LjQzWm0wLDMuNDNjMCwuMjQtLjEuNDMtLjI0LjQzSDkuNjJjLS4xNCwwLS4yNC0uMTktLjI0LS40M3YtLjg1YzAtLjI0LjEtLjQzLjI0LS40M0gxNWMuMTQsMCwuMjQuMTkuMjQuNDNaIi8+DQogIDxyZWN0IGNsYXNzPSJjbHMtMiIgd2lkdGg9IjI0IiBoZWlnaHQ9IjI0Ii8+DQo8L3N2Zz4NCg==") no-repeat center;background-size:24px 100%}.h5p-hub .h5p-hub-content-type-list::-webkit-scrollbar-button:vertical{background-size:20px 20px;background-position-x:-4px;background-position-y:-3px;background-color:#5e6978;background-repeat:no-repeat}.h5p-hub .h5p-hub-content-type-list::-webkit-scrollbar-button:vertical:decrement{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQoJPHN0eWxlIHR5cGU9InRleHQvY3NzIj4qIHtmaWxsOiAjY2VkNmUzO30gLnN0MHtmaWxsOm5vbmU7fTwvc3R5bGU+DQoJPHRpdGxlPmFycm93LXVwPC90aXRsZT4NCgk8cGF0aCBkPSJNOCAxMy45bDAuNCAwLjRjMC4zIDAuMyAwLjggMC4zIDEuMSAwbDIuNC0yLjQgMi40IDIuNGMwLjMgMC4zIDAuOCAwLjMgMS4xIDBsMC40LTAuNGMwLjMtMC4zIDAuMy0wLjggMC0xLjEgMCAwIDAgMCAwIDBsLTMuMy0zLjNjLTAuMy0wLjMtMC44LTAuMy0xLjEgMCAwIDAgMCAwIDAgMEw4IDEyLjhDNy43IDEzLjEgNy43IDEzLjUgOCAxMy45IDggMTMuOCA4IDEzLjggOCAxMy45eiIvPg0KCTxyZWN0IGNsYXNzPSJzdDAiIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIvPg0KPC9zdmc+")}.h5p-hub .h5p-hub-content-type-list::-webkit-scrollbar-button:vertical:increment{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogICAgPHN0eWxlPioge2ZpbGw6ICNjZWQ2ZTM7fSAuY2xzLTEge2ZpbGw6IG5vbmU7fTwvc3R5bGU+DQogIDx0aXRsZT5hcnJvdy1kb3duPC90aXRsZT4NCiAgPHBhdGggZD0iTTE2LDEwLjE1bC0uMzUtLjM1YS43OS43OSwwLDAsMC0xLjA4LDBsLTIuNDMsMi40M0w5LjY5LDkuNzlhLjc4Ljc4LDAsMCwwLTEuMDgsMGwtLjM2LjM2YS43Ni43NiwwLDAsMCwwLDEuMDhsMy4zMiwzLjMyYS43Ni43NiwwLDAsMCwxLjA4LDBMMTYsMTEuMjNhLjc2Ljc2LDAsMCwwLDAtMS4wOFoiLz4NCiAgPHJlY3QgY2xhc3M9ImNscy0xIiB3aWR0aD0iMjQiIGhlaWdodD0iMjQiLz4NCjwvc3ZnPg0K")}.h5p-hub .h5p-hub-content-type-list:before{content:" ";width:100%;box-shadow:0 0.4em 0.4em -0.2em rgba(79,87,99,0.2) inset,0 -0.4em 0.4em -0.2em rgba(79,87,99,0.2) inset;height:calc(100% - 5.996455834em);position:absolute;pointer-events:none}.h5p-hub .h5p-hub-content-type-list ol{margin:1.25em 0;padding:0;list-style:none}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media{width:100%;padding:0.5em 1.5em 0.5em 1em;box-sizing:border-box;cursor:pointer;overflow:visible;outline:none;margin-top:0}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media:first-child{margin-top:0}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media:hover{background-color:#f5faff}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media:focus{box-shadow:0 0 1px 1px #1a93f4}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media:not(:last-child){border-bottom:1px solid #ececec}.h5p-hub .h5p-hub-content-type-list .h5p-hub-media[tabindex="0"]{background-color:#f5faff}.h5p-hub .h5p-hub-filtering .h5p-hub-content-type-list:before{height:calc(100% - 3.536668484em)}.h5p-hub-no-results{background:no-repeat 4em 0 url(data:image/svg+xml;base64, );background-size:16% 100%;padding:0 1em 0.5em 17em;margin-top:2em}.h5p-hub-no-results-title{margin-bottom:0.4em;font-size:1.2em;font-weight:bold}.h5p-hub-no-results-desc{color:#4d4d4d}

.h5p-hub .h5p-hub-panel.h5p-section-content-types{max-height:3.0em;overflow:hidden}

.h5p-hub-content-type-detail-button-bar{margin:1.666em 0;text-align:right}.h5p-hub-content-type-detail-button-bar .h5p-hub-button-update,.h5p-hub-content-type-detail-button-bar .h5p-hub-button-updating{margin-right:0.6em}.h5p-hub-content-type-detail-button-bar .h5p-hub-button-install:before,.h5p-hub-content-type-detail-button-bar .h5p-hub-button-installing:before,.h5p-hub-content-type-detail-button-bar .h5p-hub-button-update:before,.h5p-hub-content-type-detail-button-bar .h5p-hub-button-updating:before{display:inline-block;margin-right:0.7em;font-weight:normal;font-size:0.9em}.h5p-hub-content-type-detail-button-bar .h5p-hub-button-install:before{font-family:'h5p-hub';content:"";transform:rotate(-90deg)}.h5p-hub-content-type-detail-button-bar .h5p-hub-button-update:before{font-family:'h5p-hub';content:""}.h5p-hub-content-type-detail-button-bar .h5p-hub-button-installing:before,.h5p-hub-content-type-detail-button-bar .h5p-hub-button-updating:before{font-family:'h5p-hub';content:"";-webkit-animation:spin 2s linear infinite;animation:spin 2s linear infinite}

.h5p-hub .h5p-hub-content-type-detail{background-color:white;padding:1.3em;display:inherit;position:absolute;top:0;left:0;visibility:hidden;outline:none;overflow:auto;width:100%;height:100%;box-sizing:border-box;transform:translateX(100%);transition:transform 0.2s ease-out, visibility 0s linear 0.2s;box-shadow:0 0.4em 0.4em -0.2em rgba(79,87,99,0.2) inset,0 -0.4em 0.4em -0.2em rgba(79,87,99,0.2) inset}.h5p-hub .h5p-hub-content-type-detail::-webkit-scrollbar{background-color:#F3F3F3;width:0.750em}.h5p-hub .h5p-hub-content-type-detail::-webkit-scrollbar-thumb{background:#5e6978 url("data:image/svg+xml;base64,PHN2ZyBpZD0ic2Nyb2xsYmFyX2hhbmRsZSIgZGF0YS1uYW1lPSJzY3JvbGxiYXIgaGFuZGxlIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogIDxkZWZzPg0KICAgIDxzdHlsZT4NCiAgICAgIC5jbHMtMSB7DQogICAgICAgIGZpbGw6ICNjZWQ2ZTM7DQogICAgICB9DQoNCiAgICAgIC5jbHMtMiB7DQogICAgICAgIGZpbGw6IG5vbmU7DQogICAgICB9DQogICAgPC9zdHlsZT4NCiAgPC9kZWZzPg0KICA8dGl0bGU+c2Nyb2xsYmFyLWJhcnMtaWNvbjwvdGl0bGU+DQogIDxwYXRoIGNsYXNzPSJjbHMtMSIgZD0iTTE1LjI4LDguNWMwLC4yNC0uMS40My0uMjQuNDNIOS42MmMtLjE0LDAtLjI0LS4xOS0uMjQtLjQzVjcuNjVjMC0uMjQuMS0uNDMuMjQtLjQzSDE1Yy4xNCwwLC4yNC4xOS4yNC40M1ptMCwzLjQxYzAsLjI0LS4xLjQzLS4yNC40M0g5LjYyYy0uMTQsMC0uMjQtLjE5LS4yNC0uNDN2LS44NWMwLS4yNC4xLS40My4yNC0uNDNIMTVjLjE0LDAsLjI0LjE5LjI0LjQzWm0wLDMuNDNjMCwuMjQtLjEuNDMtLjI0LjQzSDkuNjJjLS4xNCwwLS4yNC0uMTktLjI0LS40M3YtLjg1YzAtLjI0LjEtLjQzLjI0LS40M0gxNWMuMTQsMCwuMjQuMTkuMjQuNDNaIi8+DQogIDxyZWN0IGNsYXNzPSJjbHMtMiIgd2lkdGg9IjI0IiBoZWlnaHQ9IjI0Ii8+DQo8L3N2Zz4NCg==") no-repeat center;background-size:24px 100%}.h5p-hub .h5p-hub-content-type-detail::-webkit-scrollbar-button:vertical{background-size:20px 20px;background-position-x:-4px;background-position-y:-3px;background-color:#5e6978;background-repeat:no-repeat}.h5p-hub .h5p-hub-content-type-detail::-webkit-scrollbar-button:vertical:decrement{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQoJPHN0eWxlIHR5cGU9InRleHQvY3NzIj4qIHtmaWxsOiAjY2VkNmUzO30gLnN0MHtmaWxsOm5vbmU7fTwvc3R5bGU+DQoJPHRpdGxlPmFycm93LXVwPC90aXRsZT4NCgk8cGF0aCBkPSJNOCAxMy45bDAuNCAwLjRjMC4zIDAuMyAwLjggMC4zIDEuMSAwbDIuNC0yLjQgMi40IDIuNGMwLjMgMC4zIDAuOCAwLjMgMS4xIDBsMC40LTAuNGMwLjMtMC4zIDAuMy0wLjggMC0xLjEgMCAwIDAgMCAwIDBsLTMuMy0zLjNjLTAuMy0wLjMtMC44LTAuMy0xLjEgMCAwIDAgMCAwIDAgMEw4IDEyLjhDNy43IDEzLjEgNy43IDEzLjUgOCAxMy45IDggMTMuOCA4IDEzLjggOCAxMy45eiIvPg0KCTxyZWN0IGNsYXNzPSJzdDAiIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIvPg0KPC9zdmc+")}.h5p-hub .h5p-hub-content-type-detail::-webkit-scrollbar-button:vertical:increment{background-image:url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCI+DQogICAgPHN0eWxlPioge2ZpbGw6ICNjZWQ2ZTM7fSAuY2xzLTEge2ZpbGw6IG5vbmU7fTwvc3R5bGU+DQogIDx0aXRsZT5hcnJvdy1kb3duPC90aXRsZT4NCiAgPHBhdGggZD0iTTE2LDEwLjE1bC0uMzUtLjM1YS43OS43OSwwLDAsMC0xLjA4LDBsLTIuNDMsMi40M0w5LjY5LDkuNzlhLjc4Ljc4LDAsMCwwLTEuMDgsMGwtLjM2LjM2YS43Ni43NiwwLDAsMCwwLDEuMDhsMy4zMiwzLjMyYS43Ni43NiwwLDAsMCwxLjA4LDBMMTYsMTEuMjNhLjc2Ljc2LDAsMCwwLDAtMS4wOFoiLz4NCiAgPHJlY3QgY2xhc3M9ImNscy0xIiB3aWR0aD0iMjQiIGhlaWdodD0iMjQiLz4NCjwvc3ZnPg0K")}.h5p-hub .h5p-hub-content-type-detail.h5p-hub-show{transform:translateX(0);visibility:visible;transition:transform 0.2s ease-out, visibility 0s linear 0s}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-panel[aria-hidden="false"]{display:none}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-container{max-width:100%;margin-left:auto;margin-right:auto;margin-bottom:1.666em;padding-bottom:1em}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-container:after{content:" ";display:block;clear:both}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-image-wrapper{width:100%;float:left;margin-left:0;margin-right:0}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-title{outline:0}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-text-details{width:100%;float:left;margin-left:0;margin-right:0}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-text-details h2{margin-top:0}@media (min-width: 576px){.h5p-hub .h5p-hub-content-type-detail .h5p-hub-image-wrapper{width:32.20339%;float:left;margin-right:1.69492%}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-text-details{width:66.10169%;float:right;margin-right:0}}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-button-bar{margin:1.666em 0;text-align:right}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-button-bar .h5p-hub-button-update,.h5p-hub .h5p-hub-content-type-detail .h5p-hub-button-bar .h5p-hub-button-updating{margin-right:0.6em}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-button-bar .h5p-hub-button-update:before{font-family:'h5p-hub';content:"";margin-right:0.5em;font-weight:normal;font-size:0.9em}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-back-button{color:#474f5a;font-size:1.3em;display:inline-block;background:transparent;border:none;text-decoration:none;outline:none;transform:rotate(180deg)}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-back-button:hover{color:#000}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-back-button:focus{box-shadow:0 0 1px 1px #1a93f4}.h5p-hub .h5p-hub-content-type-detail .h5p-hub-back-button.h5p-hub-icon-arrow-thick:before{cursor:pointer}.h5p-hub-lightbox-inner.h5p-hub-license{background:#fff;text-align:left;max-height:calc(100% - 5em);width:calc(100% - 5em)}.h5p-hub-lightbox-inner.h5p-hub-license :not(.h5p-hub-loading) .h5p-hub-modal-content{max-height:50em;overflow-y:auto}.h5p-hub-lightbox-inner.h5p-hub-license .h5p-hub-loading:after{font-family:'h5p-hub';content:"";font-size:3em;line-height:1;width:1em;height:1em;padding:.833em;display:block;color:#7c8696;-webkit-animation:spin 2s linear infinite;animation:spin 2s linear infinite;width:1em;margin:0 auto}.h5p-hub-lightbox-inner.h5p-hub-license .h5p-hub-lightbox-close{top:-0.8em;right:-0.8em;color:#333}.h5p-hub-lightbox-inner.h5p-hub-license .h5p-hub-modal-header{font-weight:400;font-size:1.6em;margin-bottom:1em}.h5p-hub-lightbox-inner.h5p-hub-license .h5p-hub-modal-title{font-weight:700}

.h5p-hub .h5p-hub-upload-wrapper{padding:1em 2em 2em 2em}.h5p-hub .h5p-hub-upload-wrapper .h5p-hub-message{margin-top:0}.h5p-hub .h5p-hub-upload-wrapper .h5p-hub-upload-throbber{outline:0;height:10em}.h5p-hub .h5p-hub-upload-wrapper .h5p-hub-upload-throbber.h5p-hub-hidden{display:none}.h5p-hub .h5p-hub-upload-wrapper .h5p-hub-upload-throbber:before{font-family:'h5p-hub';content:"";font-size:4em;line-height:1;width:1em;height:1em;padding:.833em;display:block;color:#7c8696;-webkit-animation:spin 2s linear infinite;animation:spin 2s linear infinite;margin:auto}.h5p-hub .h5p-hub-upload-wrapper a:focus{outline:auto}.h5p-hub h1.h5p-hub-upload-instruction-header{font-size:1.25em;margin-top:.833em;margin-bottom:.833em}.h5p-hub p.h5p-hub-upload-instruction-description{font-size:1.042em}

.h5p-hub .h5p-hub-upload-form .h5p-hub-input-wrapper{display:inline-block}.h5p-hub .h5p-hub-upload-form .h5p-hub-input-wrapper input[type="file"]{display:none}.h5p-hub .h5p-hub-upload-form [aria-hidden="true"]{display:none}.h5p-hub .h5p-hub-upload-form .h5p-hub-upload-button{background:transparent;color:#697585;border:0.125em solid #697585;margin-left:0.5em}.h5p-hub .h5p-hub-upload-form .h5p-hub-use-button{background:#697585;color:white;margin-left:0.5em}.h5p-hub .h5p-hub-upload-form button[disabled]{opacity:0.5}.h5p-hub .h5p-hub-upload-form .h5p-hub-upload-path{display:inline-block;min-width:14em;padding:10px;padding-left:1.5em;margin-bottom:1em;border:0.125em solid #ced6e3;border-radius:1.375em;outline:0;color:#697585;cursor:pointer}.h5p-hub .h5p-hub-upload-form .h5p-hub-upload-path[disabled]{cursor:default}@media (min-width: 576px){.h5p-hub .h5p-hub-upload-form .h5p-hub-upload-path{width:60%}}

.h5p-hub{font-family:'Open Sans', sans-serif;padding-top:1px;padding-bottom:.833em;position:relative;overflow:hidden}.h5p-hub .h5p-hub-panel{border:1px solid #697585;background:#fff}.h5p-hub .h5p-hub-panel.h5p-hub-section-content-types{border-color:#0a78d1}.h5p-hub .h5p-hub-panel.h5p-hub-section-reuse{border-color:#db6f28}.h5p-hub .h5p-hub-panel.h5p-hub-section-upload{border-color:#ced6e3}.h5p-hub .h5p-hub-ssection-content-types.open{max-height:58.65em}.h5p-hub .h5p-hub-button-install [class^="icon-"]{margin-right:.4165em;display:inline-block}.h5p-hub .h5p-hub-button-install .h5p-hub-icon-arrow-thick{transform:rotate(-90deg)}.h5p-hub .h5p-hub-content-type-section{position:relative;overflow:hidden}.h5p-hub .h5p-hub-content-type-section.h5p-hub-height-limit{height:34em}.h5p-hub .h5p-hub-content-type-section-view>.h5p-hub-message{margin:0;padding:0.5em 3.5em 0.7em 2.292em}.h5p-hub .h5p-hub-error .h5p-hub-button.h5p-hub-retry-button{margin:0.5em 0 0.75em;border-width:2px}.h5p-hub .h5p-hub-error .h5p-hub-button.h5p-hub-retry-button:hover,.h5p-hub .h5p-hub-error .h5p-hub-button.h5p-hub-retry-button:active{background-color:#c64750;border-color:#c64750;color:#fff}.h5p-hub .h5p-hub-error .h5p-hub-button.h5p-hub-retry-button:focus{box-shadow:0.06em 0 0.6em 0.1em #c64750}.h5p-hub .h5p-hub-error .h5p-hub-button.h5p-hub-retry-button[disabled]{box-shadow:none;background-color:#c64750;border-color:#c64750;color:#fff;opacity:0.5;padding-left:3em}.h5p-hub .h5p-hub-error .h5p-hub-button.h5p-hub-retry-button[disabled]:before{font-family:'h5p-hub';content:"";font-size:1em;line-height:1;width:1em;height:1em;padding:.833em;display:block;color:#7c8696;-webkit-animation:spin 2s linear infinite;animation:spin 2s linear infinite;position:absolute;margin-left:-2.35em;margin-top:-0.8em;color:#fff}.h5p-hub.h5p-hub-hidden{display:none}.h5p-hub-client-container-using-mouse [role=button]{outline:none}

