<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'workshopallocation_manual', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package    workshopallocation
 * @subpackage manual
 * @copyright  2009 David Mudrak <david@moodle.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['addreviewee'] = 'Add reviewee';
$string['addreviewer'] = 'Add reviewer';
$string['allocationadded'] = 'The submission has been successfully allocated';
$string['allocationexists'] = 'The allocation already exists';
$string['areyousuretodeallocate'] = 'Are you sure you want to deallocate the selected assessment?';
$string['areyousuretodeallocategraded'] = 'You are going to remove the assessment that has already been graded. Are you really sure you want to do it?';
$string['pluginname'] = 'Manual allocation';
$string['privacy:metadata:preference:perpage'] = 'Number of allocated assessments the user prefers to see on one page.';
$string['showallparticipants'] = 'Show all participants';
