{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template qbank_columnsortorder/reset_columns

    Button to reset question bank column settings to defaults.

    Context variables required for this template:

    Example context (json):
    {
        "reseturl": "https://example.com/question/bank/columnsortorder/actions.php?action=reset"
    }
}}
<span>
    <a href="{{reseturl}}" class="btn btn-outline-dark action-link ms-1" data-action="reset">
        {{#str}}resetcolumns, qbank_columnsortorder{{/str}}
    </a>
</span>
