{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_course/activity_icon

    Displays an activity plugin icon.

    Example context (json):
    {
        "icon": "../../../pix/help.svg",
        "iconclass": "",
        "purpose": "content",
        "branded": 0,
        "pluginname": "File",
        "title": "File",
        "extraclasses": "mycustomclass"
    }
}}
<div
    class="activity-icon activityiconcontainer {{!
        }}{{purpose}} {{#branded}}isbranded{{/branded}} {{extraclasses}}"
>
    <img
        src="{{{icon}}}"
        class="activityicon icon {{iconclass}}"
        data-region="activity-icon"
        alt="{{#title}}{{#cleanstr}} activityicon, moodle, {{{pluginname}}} {{/cleanstr}}{{/title}}"
        {{#title}}title="{{{title}}}"{{/title}}
    >
</div>
