<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component qbank_columnsortorder, language 'en'.
 *
 * @package    qbank_columnsortorder
 * @copyright  2021 Catalyst IT Australia Pty Ltd
 * @author     Ghaly Marc-Alexandre <marc-alexandreghaly@catalyst-ca.net>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['addcolumn'] = 'Add column \'{$a}\'';
$string['addcolumns'] = 'Add columns';
$string['auto'] = 'Auto';
$string['columnwidth'] = 'Column width (pixels)';
$string['invalidwidth'] = 'Width must be at least {$a}.';
$string['movecolumn'] = 'Move column \'{$a}\'';
$string['pluginname'] = 'Column sort order';
$string['privacy:metadata:preference:enabledcol'] = 'The Column sort order question bank plugin saves user preference of column orders.';
$string['privacy:metadata:preference:hiddencols'] = 'The Column sort order question bank plugin saves user preference of hidden columns.';
$string['privacy:metadata:preference:colsize'] = 'The Column sort order question bank plugin saves user preference of column sizes.';
$string['qbankcolumnsortorder'] = 'Column sort order';
$string['qbankgotocolumnsort'] = 'You can change the order of the columns in the question bank on the page {$a}.';
$string['qbankcolumnsdisabled'] = 'Currently disabled question bank plugins:';
$string['qbankgotomanageqbanks'] = 'You can remove a column by disabling the plugin in <a href=\'{$a}\'>Manage question bank plugins</a>.';
$string['qbankcolumnname'] = '{$a}';
$string['qbanksortdescription'] = 'The order in which plugins are listed below determines the order of the columns in the question bank.';
$string['resetcolumns'] = 'Reset columns';
$string['resize'] = 'Resize';
$string['resizecolumn'] = 'Resize column {$a}';
$string['removecolumn'] = 'Remove column \'{$a}\'';
$string['showhidecolumn'] = 'Show/hide column';
$string['width'] = 'Width of \'{$a}\' in pixels';
