{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template assignsubmission_onlinetext/email_summary_html

    This template is used render the HTML version of the summary of online text submissions to go into emails.

    Classes required for JS:
    * none

    Context variables required for this template:
    * wordcount - Word count of the student's submission.

    Example context (json):
    {
        "wordcount": 123
    }
}}
<h3>{{#str}}onlinetext, assignsubmission_onlinetext{{/str}}</h3>
<p>{{#str}}numwords, assignsubmission_onlinetext, {{wordcount}}{{/str}}</p>
