<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Core admin presets component to load some settings/plugins.
 *
 * @package          core_adminpresets
 * @copyright        2021 Sara Arjona (sara@moodle.com)
 * @license          http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['disabled'] = 'Disabled';
$string['disabledwithvalue'] = 'Disabled ({$a})';
$string['enabled'] = 'Enabled';
$string['errordeleting'] = 'Error deleting from database.';
$string['errorinserting'] = 'Error inserting into database.';
$string['errornopreset'] = 'A preset with this name doesn\'t exist.';
$string['fullpreset'] = 'Full';
$string['fullpresetdescription'] = 'All the Starter features plus External (LTI) tool, SCORM, Workshop, Analytics, Badges, Competencies, Learning plans and lots more.';
$string['markedasadvanced'] = 'marked as advanced';
$string['markedasforced'] = 'marked as forced';
$string['markedaslocked'] = 'marked as locked';
$string['markedasnonadvanced'] = 'not marked as advanced';
$string['markedasnonforced'] = 'not marked as forced';
$string['markedasnonlocked'] = 'not marked as locked';
$string['privacy:metadata:adminpresets'] = 'The list of configuration presets.';
$string['privacy:metadata:adminpresets:comments'] = 'The description of the preset.';
$string['privacy:metadata:adminpresets:moodlerelease'] = 'The Moodle release version that the preset is based on.';
$string['privacy:metadata:adminpresets:name'] = 'The name of the preset.';
$string['privacy:metadata:adminpresets:site'] = 'The Moodle site where this preset was created.';
$string['privacy:metadata:adminpresets:timecreated'] = 'The time that the change was made.';
$string['privacy:metadata:adminpresets:userid'] = 'The user who create the preset.';
$string['privacy:metadata:adminpresets_app'] = 'The configuration presets that have been applied.';
$string['privacy:metadata:adminpresets_app:adminpresetid'] = 'The ID of the preset applied.';
$string['privacy:metadata:adminpresets_app:time'] = 'The time that the preset was applied.';
$string['privacy:metadata:adminpresets_app:userid'] = 'The user who applied the preset.';
$string['sensiblesettings'] = 'Settings with passwords';
$string['sensiblesettingstext'] = 'Settings with passwords or other sensitive information can be excluded when creating a site admin preset. Enter additional settings with format SETTINGNAME@@PLUGINNAME separated by commas.';
$string['siteadminpresetspluginname'] = 'Site admin presets';
$string['starterpreset'] = 'Starter';
$string['starterpresetdescription'] = 'Moodle with all of the most popular features, including Assignment, Feedback, Forum, H5P, Quiz and Completion tracking.';
$string['wrongid'] = 'Wrong ID';
