{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_quiz/overrides_actions
    Actions bar at the top of the overrides page UI.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * overridesnav - must alwasy be present.
    * addoverridebutton - should only be present if the user has permission to add a new override.

    Example context (json):
    {
        "overridesnav": {
            "id": "url_select_test",
            "action": "https://example.com/post",
            "formid": "url_select_form",
            "sesskey": "sesskey",
            "label": "core/url_select",
            "options": [{
                    "name": "User overrides",
                    "value": "/mod/quiz/overrides.php?cmid=71&mode=user"
                },
                {
                    "name": "Group overrides",
                    "value": "/mod/quiz/overrides.php?cmid=71&mode=group"
                }
            ],
            "disabled": false,
            "title": "Some cool title"
        },
        "addoverridebutton": {
            "id": "single_button1234",
            "type": "primary",
            "method": "get",
            "url": "http://localhost/moodle/mod/quiz/overrideedit.php",
            "label": "Add user override",
            "classes": "singlebutton",
            "disabled": false,
            "params": [
                {
                    "name": "cmid",
                    "value": "73"
                },
                {
                    "name": "action",
                    "value": "adduser"
                }
            ]
        }
    }
}}
<div class="tertiary-navigation">
    <div class="d-flex">
        {{#overridesnav}}
            <div class="navitem">
                {{>core/url_select}}
            </div>
        {{/overridesnav}}
        {{#addoverridebutton}}
            <div class="navitem">
                {{>core/single_button}}
            </div>
        {{/addoverridebutton}}
    </div>
</div>
