{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_webservice/missing_capabilities

    Display missing capabilities in Manage tokens system report

    Example context (json):
    {
        "missingcapabilities": [
            {
                "capabilityname": "capabilityname",
                "capabilitylink": "<a href='capabilitylink'>capabilityname</a>"
            }
        ],
        "helpicon": "<a class='btn'><i class='icon fa fa-question-circle'></i></a>"
    }
}}
<div class="d-flex missingcaps">
    <ul class="list-unstyled">
        {{#missingcapabilities}}
        <li class="mb-1">
            {{{link}}}
            <div class="text-muted">{{name}}</div>
        </li>
        {{/missingcapabilities}}
    </ul>
    <div>{{{helpicon}}}</div>
</div>
