{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template gradingform_guide/comment_chooser

    Moodle comment chooser template for marking guide.

    The purpose of this template is to render a list of frequently used comments that can be used by the comment chooser dialog.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * criterionId The criterion ID this chooser template is being generated for.
    * comments Array of id / description pairs.

    Example context (json):
    {
        "criterionId": "1",
        "comments": [
            {
                "id": "1",
                "description": "Test comment description 1"
            },
            {
                "id": "2",
                "description": "Test comment description 2"
            }
        ]
    }
}}
<div class="gradingform_guide_comment_chooser" id="comment_chooser">
    <div class="list-group">
        {{#comments}}
            <button class="list-group-item list-group-item-action" id="comment-option-{{criterionId}}-{{id}}" tabindex="0">
                {{description}}
            </button>
        {{/comments}}
    </div>
</div>
