{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_bigbluebuttonbn/participant_form_add

    This template renders a row in the participant add form

    Example context (json):
    {
        "selectiontypevalue": "example",
        "selectionvalue": "example",
        "participanttype": "student",
        "participantvalue": "",
        "roles": [],
        "canRemove": true
    }
}}

<tr id="participant_list_tr_{{selectiontypevalue}}-{{selectionvalue}}">
    <td class="participant_list_c0">
        <b><i>{{participanttype}}</i></b>
    </td>
    <td class="participant_list_c1">
        {{participantvalue}}
    </td>
    <td class="participant_list_c2">
        <i>{{#str}}mod_form_field_participant_list_text_as, mod_bigbluebuttonbn{{/str}}</i>
    </td>
    <td class="participant_list_c3">
        <select class="participant-select form-select" name="select-for-{{selectiontypevalue}}-{{selectionvalue}}">
            {{#roles}}
                <option value="{{id}}" {{#isselected}}selected{{/isselected}}>{{label}}</option>
            {{/roles}}
        </select>
    </td>
    <td class="participant_list_c4">
        {{#canRemove}}
            <a class='btn btn-secondary btn-sm remove-button'>
                {{#pix}}t/delete, core, {{#str}}mod_form_field_participant_list_action_remove, mod_bigbluebuttonbn{{/str}}{{/pix}}
            </a>
        {{/canRemove}}
    </td>
</tr>
