{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_templatelibrary/display_template

    Moodle template to display another template.

    The purpose of this template is to put scafolding in the page, so the javascript can fetch templates and
    insert them into this part of the page.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * data-region

    Context variables required for this template:
    * none

    Example context (json):
    { }

}}
<div data-region="displaytemplate">
        <h3 data-region="displaytemplateheader">{{#str}}notemplateselected, tool_templatelibrary{{/str}}</h3>
        <div>
            <h4>{{#str}}example, tool_templatelibrary{{/str}}</h4>
            <div data-region="displaytemplateexample">
                -
            </div>
        </div>
        <div>
            <h4>{{#str}}documentation, tool_templatelibrary{{/str}}</h4>
            <pre data-region="displaytemplatesource"> - </pre>
        </div>
</div>
