{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_drawer_view_group_info_body_content

    This template will render the content for the body section of the group
    info page in the message drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {}

}}

<a class="px-2 align-self-start" href="#" data-route-back role="button">
    {{> core_message/message_drawer_icon_back }}
</a>
<div class="px-2">
    <div class="text-center">
        {{#imageurl}}
        <img
            class="rounded-circle"
            src="{{.}}"
            aria-hidden="true"
            alt="{{#str}} pictureof, moodle, {{name}} {{/str}}"
            title="{{#str}} pictureof, moodle, {{name}} {{/str}}"
            style="height: 100px; width: 100px"
        >
        {{/imageurl}}
    </div>
    <h2 class="mt-2 mb-0 text-center text-truncate h4">{{name}}</h2>
    {{#subname}}<h3 class="mt-2 mb-0 text-center text-truncate h5">{{.}}</h3>{{/subname}}
</div>
<h3 class="border-bottom h6 mt-3 px-3 py-2 mb-0 fw-bold">{{#str}} otherparticipants, core_message {{/str}}</h3>
<div class="pt-1 bg-white overflow-y" data-region="members-list-container">
    {{< core_message/message_drawer_lazy_load_list }}
        {{$rootattributes}}
            data-region="members-list"
        {{/rootattributes}}
        {{$emptymessage}}{{#str}} noparticipants, core_message {{/str}}{{/emptymessage}}
        {{$placeholder}}
            {{#placeholders}}
                {{> core_message/message_drawer_view_group_info_participants_list_item_placeholder }}
            {{/placeholders}}
        {{/placeholder}}
    {{/ core_message/message_drawer_lazy_load_list }}
</div>
