{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_grades/initials_dropdown_form

    @deprecated since Moodle 4.5 - please use core_course/initials_dropdown_form instead.
    @todo Final deprecation on Moodle 6.0. See MDL-82421.

    The form component of our dropdown within the tertiary navigation.

    Context variables required for this template:
    * courseid - The course ID.
    * initialsbars - The rendered mini renderer from the user renderer.

    Example context (json):
    {
        "courseid": "2",
        "initialsbars": "Output from renderer method"
    }
}}
<form class="initialsdropdownform" action="#" data-courseid="{{courseid}}">
    {{{initialsbars}}}
    <div class="d-flex flex-row justify-content-end">
        <input class="btn btn-outline-secondary pull-right mx-2" data-action="cancel" type="submit" value="{{#str}}closebuttontitle{{/str}}">
        <input class="btn btn-primary pull-right" data-action="save" type="submit" value="{{#str}}apply{{/str}}">
    </div>
</form>
