<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'webservice', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package   core_webservice
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['privacy:metadata:serviceusers'] = 'A list of users who can use a certain external service';
$string['privacy:metadata:serviceusers:iprestriction'] = 'IP restricted to use the service';
$string['privacy:metadata:serviceusers:timecreated'] = 'The date when the record was created';
$string['privacy:metadata:serviceusers:userid'] = 'The ID of the user';
$string['privacy:metadata:serviceusers:validuntil'] = 'The date that the authorisation is valid until';
$string['privacy:metadata:tokens'] = 'A record of tokens for interacting with Moodle through web services or Mobile applications.';
$string['privacy:metadata:tokens:creatorid'] = 'The ID of the user who created the token';
$string['privacy:metadata:tokens:iprestriction'] = 'IP restricted to use this token';
$string['privacy:metadata:tokens:lastaccess'] = 'The date when the token was last used';
$string['privacy:metadata:tokens:name'] = 'The token name';
$string['privacy:metadata:tokens:privatetoken'] = 'A more private token occasionally used to validate certain operations, such as SSO';
$string['privacy:metadata:tokens:timecreated'] = 'The date when the token was created';
$string['privacy:metadata:tokens:token'] = 'The user\'s token';
$string['privacy:metadata:tokens:tokentype'] = 'The type of token';
$string['privacy:metadata:tokens:userid'] = 'The ID of the user whose token it is';
$string['privacy:metadata:tokens:validuntil'] = 'The date that the token is valid until';
$string['privacy:request:notexportedsecurity'] = 'Not exported for security reasons';
$string['services'] = 'External services';
