{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_course/activity_info

    Container to display activity information such as:
      - Activity dates
      - Activity completion requirements (automatic completion)
      - Manual completion button

    Example context (json):
    {
        "activityname": "Course announcements",
        "hascompletion": true,
        "uservisible": true,
        "hasdates": true,
        "isautomatic": true,
        "istrackeduser": true,
        "showmanualcompletion": true,
        "activitydates": [
            {
                "label": "Opens:",
                "timestamp": 1293876000
            }
        ],
        "completiondetails": [
             {
                "statuscomplete": 1,
                "description": "Viewed"
            },
            {
                "statusincomplete": 1,
                "description": "Receive a grade"
            }
        ]
    }
}}
<div data-region="activity-information" data-activityname="{{activityname}}" class="activity-information">

    {{#hascompletion}}
        {{#uservisible}}
            <div class="completion-info" data-region="completion-info">
                {{#isautomatic}}
                    <div class="automatic-completion-conditions" data-region ="completionrequirements" role="list" aria-label="{{#str}}completionrequirements, core_course, {{activityname}}{{/str}}">
                        {{#completiondetails}}
                            {{> core_course/completion_automatic }}
                        {{/completiondetails}}
                    </div>
                {{/isautomatic}}
                {{^isautomatic}}
                    {{#showmanualcompletion}}
                        {{> core_course/completion_manual }}
                    {{/showmanualcompletion}}
                {{/isautomatic}}
            </div>
        {{/uservisible}}
    {{/hascompletion}}

    {{#hasdates}}
    <div data-region="activity-dates" class="activity-dates">
        {{#activitydates}}
            {{>core_course/activity_date}}
        {{/activitydates}}
    </div>
    {{/hasdates}}
</div>
