// stylelint-disable
//
// Automatically style Markdown-based tables like a Bootstrap `.table`.
//

.bd-content {

    // Hack the sticky header
    >h2[id],
    >h3[id],
    >h4[id] {
        pointer-events: none;

        &::before {
            display: block;
            height: 6rem;
            margin-top: -6rem;
            content: "";
        }
    }

    // Override Bootstrap defaults
    >.table {
        max-width: 100%;
        margin-bottom: 1.5rem;

        @include media-breakpoint-down(md) {
            display: block;
            overflow-x: auto;

            &.table-bordered {
                border: 0;
            }
        }

        th,
        td {
            &:first-child {
                padding-left: 0;
            }

            &:not(:last-child) {
                padding-right: 1.5rem;
            }
        }

        // Prevent breaking of code (e.g., Grunt tasks list)
        td:first-child>code {
            white-space: nowrap;
        }
    }
}

.bd-content-title {
    display: block;
    pointer-events: auto;
}

//
// Docs sections
//

.bd-content {
    >h2:not(:first-child) {
        margin-top: 3rem;
    }

    >h3 {
        margin-top: 1.5rem;
    }

    >ul li,
    >ol li {
        margin-bottom: .25rem;
    }
}

.bd-title {
    margin-bottom: .5rem;
    @include font-size(3rem);
}

.bd-lead {
    @include font-size(1.5rem);
    font-weight: 300;
}

.bd-text-purple {
    color: $bd-purple;
}

.bd-text-purple-bright {
    color: $bd-purple-bright;
}
