videojs.addLanguage('eu', {
  "Audio Player": "Audio jogailua",
  "Video Player": "Bideo jogailua",
  "Play": "Hasi",
  "Pause": "Gelditu",
  "Replay": "Berriz hasi",
  "Current Time": "Uneko denbora",
  "Duration": "Iraupena",
  "Remaining Time": "Gelditzen den denbora",
  "Stream Type": "Jario mota",
  "LIVE": "ZUZENEAN",
  "Seek to live, currently behind live": "Zuzenekora joan, orain zuzenekoaren atzetik",
  "Seek to live, currently playing live": "Zuzenekora joan, orain zuzenean ari da",
  "Loaded": "Kargatuta",
  "Progress": "Aurrerapena",
  "Progress Bar": "Aurrerapen barra",
  "progress bar timing: currentTime={1} duration={2}": "{1} / {2}",
  "Fullscreen": "Pantaila osoa",
  "Exit Fullscreen": "Irten pantaila osotik",
  "Mute": "Ixildu",
  "Unmute": "Soinua jarri",
  "Playback Rate": "Abiadura",
  "Subtitles": "Azpitituluak",
  "subtitles off": "azpitituluak kenduta",
  "Captions": "Oharrak",
  "captions off": "oharrak kenduta",
  "Chapters": "Kapituluak",
  "Descriptions": "Deskribapenak",
  "descriptions off": "deskribapenak kenduta",
  "Audio Track": "Audio pista",
  "Volume Level": "Bolumen maila",
  "You aborted the media playback": "Bertan behera utzi duzu",
  "A network error caused the media download to fail part-way.": "Sare errore batek deskargak huts egitea eragin du.",
  "The media could not be loaded, either because the server or network failed or because the format is not supported.": "Media ezin izan da kargatu, zerbitzariak edo sareak huts egin duelako edo formatu horretako media erabili ezin delako.",
  "The media playback was aborted due to a corruption problem or because the media used features your browser did not support.": "Bertan behera gelditu da fitxategia ondo ez dagoelako edo zure nabigatzailean erabili ezin diren ezaugarriak dituelako.",
  "No compatible source was found for this media.": "Ez dago media honentzako iturburu bateragarririk.",
  "The media is encrypted and we do not have the keys to decrypt it.": "Media zifratuta dago eta ez ditugu beharrezko gakoak.",
  "Play Video": "Bideoa hasi",
  "Close": "Itxi",
  "Close Modal Dialog": "Leihoa itxi",
  "Modal Window": "Leihoa",
  "This is a modal window": "Hau leiho modal bat da",
  "This modal can be closed by pressing the Escape key or activating the close button.": "Leiho modal hau zure teklatuko Escape tekla sakatuz edo ixteko botoia sakatuz itxi daiteke.",
  ", opens captions settings dialog": ", oharren ezarpenen leihoa irekitzen du",
  ", opens subtitles settings dialog": ", azpitituluen ezarpenen leihoa irekitzen du",
  ", opens descriptions settings dialog": ", deskribapenen ezarpenen leihoa irekitzen du",
  ", selected": ", aukeratuta",
  "captions settings": "oharren ezarpenak",
  "subtitles settings": "azpitituluen ezarpenak",
  "descriptions settings": "deskribapenen ezarpenak",
  "Text": "Testua",
  "White": "Zuria",
  "Black": "Beltza",
  "Red": "Gorria",
  "Green": "Berdea",
  "Blue": "Urdina",
  "Yellow": "Horia",
  "Magenta": "Magenta",
  "Cyan": "Cyan (urdina)",
  "Background": "Atzeko planoa",
  "Window": "Leihoa",
  "Transparent": "Gardena",
  "Semi-Transparent": "Erdi-gardena",
  "Opaque": "Opaku",
  "Font Size": "Letra-tamaina",
  "Text Edge Style": "Tesuaren etzen estiloa",
  "None": "Bat ere ez",
  "Raised": "Jasoa",
  "Depressed": "Hondoratua",
  "Uniform": "Uniformea",
  "Drop shadow": "Itzalduna",
  "Font Family": "Letra-tipoa",
  "Proportional Sans-Serif": "Sans-Serif proportzionala",
  "Monospace Sans-Serif": "Tarte berdineko Sans-Serif",
  "Proportional Serif": "Serif proporzionala",
  "Monospace Serif": "Tarte berdineko Serif",
  "Casual": "Informala",
  "Script": "Eskuz idatzitakoa",
  "Small Caps": "letra xeheak",
  "Reset": "Berrezarri",
  "restore all settings to the default values": "berrezarri ezarpen guztiak defektuzko balioetara",
  "Done": "Eginda",
  "Caption Settings Dialog": "Oharren ezarpenen leihoa",
  "Beginning of dialog window. Escape will cancel and close the window.": "Leihoaren hasiera. Escapeta teklak leihoa itxi egingo du.",
  "End of dialog window.": "Leihoaren amaiera.",
  "{1} is loading.": "{1} kargatzen ari da.",
  "Exit Picture-in-Picture": "Irten irudiz-irudiztik",
  "Picture-in-Picture": "Irudiz-irudi"
});