{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_scorm/player_exitbar
    Actions bar for the user reports page UI.
    Classes required for JS:
    * none
    Data attributes required for JS:
    * none
    Context variables required for this template:
    * action
    Example context (json):
    {
        "action": "http://localhost/moodle/mod/scorm/report.php?id=70&mode=interactions"
    }
}}
<div class="d-flex flex-row-reverse mb-2">
    <a class="btn btn-secondary" href="{{{action}}}" title="{{#str}}exitactivity, mod_scorm{{/str}}">
        {{#str}}exitactivity, mod_scorm{{/str}}
    </a>
</div>
