/* History */

.path-grade-report-history div.gradeparent {
    overflow-x: scroll;
}

.path-grade-report-history .singlebutton div,
.path-grade-report-history .singlebutton div input[type="button"] {
    margin: 0;
}

/* User Selector */
.yui3-gradereport_history_usp-hidden {
    display: none;
}

.gradereport_history_usp .usp-content {
    position: relative;
}

.gradereport_history_usp .usp-ajax-content {
    overflow: auto;
    border-top: 1px solid #ccc;
    border-bottom: 1px solid #ccc;
}

.gradereport_history_usp .usp-ajax-content,
.gradereport_history_usp .usp-loading-lightbox {
    height: 375px;
}

.gradereport_history_usp .usp-loading-lightbox {
    background-color: #fff;
    opacity: .5;
    position: absolute;
    text-align: center;
    width: 100%;
    top: 0;
    left: 0;
}

.gradereport_history_usp .usp-loading-lightbox img {
    margin-top: 100px;
    opacity: 1;
}

.gradereport_history_usp .usp-search {
    text-align: center;
}

.gradereport_history_usp .usp-user {
    width: 100%;
    text-align: left;
    border-top: 1px solid #eee;
}

.gradereport_history_usp .usp-user:nth-child(odd) {
    background-color: #f9f9f9;
}

.gradereport_history_usp .usp-first-added {
    border-top: 1px solid #bbb;
}

.gradereport_history_usp .usp-checkbox {
    text-align: center;
    float: left;
    padding: 11px 6px 0 6px;
}

.gradereport_history_usp .usp-checkbox input[type=checkbox] {
    margin: 0;
}

.gradereport_history_usp .usp-picture {
    margin: 6px 3px 0 3px;
    float: left;
}

.gradereport_history_usp .usp-userpicture {
    cursor: pointer;
}

.gradereport_history_usp .usp-user .details {
    margin-left: 67px;
    padding: 3px 6px 0 6px;
    word-wrap: break-word;
}

.gradereport_history_usp .usp-user .details label {
    margin: 0;
}

.gradereport_history_usp .usp-more-results {
    padding: 5px;
    border-top: 1px solid #bbb;
}

.gradereport_history_usp .usp-finish {
    padding-top: 1em;
    text-align: center;
}

.gradereport_history_usp .usp-finish input {
    margin: 0;
}
