{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template theme_boost/admin_setting_tabs


    Example context (json):
    {
        "tabs": [
         {
            "name": "tab1",
            "active": 0,
            "displayname": "Inactive tab1",
            "html": "<p>Tab 1 content</p>"
         },
         {
            "name": "tab2",
            "active": 1,
            "displayname": "Active tab2",
            "html": "<p>Tab 2 content</p>"
         }
        ]
    }
}}
<ul class="nav nav-tabs" role="tablist">
    {{#tabs}}
        <li class="nav-item">
            <a href="#{{name}}" class="nav-link {{#active}}active{{/active}}" data-bs-toggle="tab" role="tab"
                {{#active}}aria-selected="true"{{/active}}
                {{^active}}aria-selected="false" tabindex="-1"{{/active}}>{{displayname}}</a>
        </li>
    {{/tabs}}
</ul>
<div class="tab-content mt-3">
    {{#tabs}}
        <div class="tab-pane {{#active}}active{{/active}}" id="{{name}}" role="tabpanel">
            {{{html}}}
        </div>
    {{/tabs}}
</div>

