button.h5p-core-button:visited,
button.h5p-core-button:link,
button.h5p-core-button {
  font-family: "Open Sans", sans-serif;
  font-weight: 600;
  font-size: 1em;
  line-height: 1.2;
  padding: 0.5em 1.25em;
  border-radius: 2em;

  background: #2579c6;
  color: #fff;

  cursor: pointer;
  border: none;
  box-shadow: none;
  outline: none;

  display: inline-block;
  text-align: center;
  text-shadow: none;
  vertical-align: baseline;
  text-decoration: none;

  -webkit-transition: initial;
  transition: initial;
}
button.h5p-core-button:focus {
  background: #1f67a8;
}
button.h5p-core-button:hover {
  background: rgba(31, 103, 168, 0.83);
}
button.h5p-core-button:active {
  background: #104888;
}
button.h5p-core-button:before {
  font-family: 'H5P';
  padding-right: 0.15em;
  font-size: 1.5em;
  vertical-align: middle;
  line-height: 0.7;
}
button.h5p-core-cancel-button:visited,
button.h5p-core-cancel-button:link,
button.h5p-core-cancel-button {
  border: none;
  background: none;
  color: #a00;
  margin-right: 1em;
  font-size: 1em;
  text-decoration: none;
  cursor: pointer;
}
button.h5p-core-cancel-button:hover,
button.h5p-core-cancel-button:focus {
  background: none;
  border: none;
  color: #e40000;
}
