{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/cm/title

    Displays a course module instance inside a course section.

    Example context (json):
    {
        "url": "#",
        "instancename": "Activity name",
        "uservisible": true,
        "onclick": "alert('ok')",
        "altname": "PDF file",
        "linkclasses": "",
        "modname": "resource"
    }
}}
{{#uservisible}}
    <a href="{{url}}" class="{{linkclasses}} aalink stretched-link" onclick="{{{onclick}}}">
        <span class="instancename">{{{instancename}}} {{{altname}}}</span>
    </a>
{{/uservisible}}
{{^uservisible}}
    <span class="instancename">
        {{{instancename}}} {{{altname}}}
    </span>
{{/uservisible}}
