videojs.addLanguage('tr', {
  "Audio Player": "Ses Oynatıcısı",
  "Video Player": "Video Oynatıcısı",
  "Play": "Oynat",
  "Pause": "Duraklat",
  "Replay": "Yeniden Oynat",
  "Current Time": "Süre",
  "Duration": "Toplam Süre",
  "Remaining Time": "Kalan Süre",
  "Stream Type": "Yayın Tipi",
  "LIVE": "CANLI",
  "Seek to live, currently behind live": "Canlıya git, şu anda canlının gerisinde",
  "Seek to live, currently playing live": "Canlıya git, şu anda canlı oynuyor",
  "Loaded": "Yüklendi",
  "Progress": "Yükleniyor",
  "Progress Bar": "İlerleme Çubuğu",
  "progress bar timing: currentTime={1} duration={2}": "{1}/{2}",
  "Fullscreen": "Tam Ekran",
  "Exit Fullscreen": "Tam Ekrandan Çık",
  "Mute": "Sessiz",
  "Unmute": "Sesi Aç",
  "Playback Rate": "Oynatma Hızı",
  "Subtitles": "Altyazılar",
  "subtitles off": "Altyazı Kapalı",
  "Captions": "Altyazılar",
  "captions off": "Altyazı Kapalı",
  "Chapters": "Bölümler",
  "Close Modal Dialog": "Etkileşim Penceresini Kapat",
  "Descriptions": "Açıklamalar",
  "descriptions off": "açıklamalar kapalı",
  "Audio Track": "Ses Parçası",
  "Volume Level": "Ses Düzeyi",
  "You aborted the media playback": "Medyayı oynatmayı iptal ettiniz",
  "A network error caused the media download to fail part-way.": "Medya indirme işleminin kısmen başarısız olmasına neden olan bir ağ sorunu oluştu.",
  "The media could not be loaded, either because the server or network failed or because the format is not supported.": "Sunucu veya ağ hatasından ya da biçim desteklenmediğinden medya yüklenemedi.",
  "The media playback was aborted due to a corruption problem or because the media used features your browser did not support.": "Medya oynatma, bir bozulma sorunu nedeniyle veya medya, tarayıcınızın desteklemediği özellikleri kullandığı için durduruldu.",
  "No compatible source was found for this media.": "Bu medya için uyumlu bir kaynak bulunamadı.",
  "The media is encrypted and we do not have the keys to decrypt it.": "Medya, şifrelenmiş bir kaynaktan geliyor ve oynatmak için gerekli anahtar bulunamadı.",
  "Play Video": "Videoyu Oynat",
  "Close": "Kapat",
  "Modal Window": "Etkileşim Penceresi",
  "This is a modal window": "Bu bir etkileşim penceresidir",
  "This modal can be closed by pressing the Escape key or activating the close button.": "Bu etkileşim penceresi ESC tuşuna basarak ya da kapat butonuna tıklanarak kapatılabilir.",
  ", opens captions settings dialog": ", altyazı ayarları menüsünü açar",
  ", opens subtitles settings dialog": ", altyazı ayarları menüsünü açar",
  ", opens descriptions settings dialog": ", açıklama ayarları menüsünü açar",
  ", selected": ", seçildi",
  "captions settings": "altyazı ayarları",
  "subtitles settings": "altyazı ayarları",
  "descriptions settings": "açıklama ayarları",
  "Text": "Metin",
  "White": "Beyaz",
  "Black": "Siyah",
  "Red": "Kırmızı",
  "Green": "Yeşil",
  "Blue": "Mavi",
  "Yellow": "Sarı",
  "Magenta": "Macenta",
  "Cyan": "Açık Mavi (Camgöbeği)",
  "Background": "Arka plan",
  "Window": "Pencere",
  "Transparent": "Saydam",
  "Semi-Transparent": "Yarı-Saydam",
  "Opaque": "Mat",
  "Font Size": "Metin Boyutu",
  "Text Edge Style": "Metin Kenar Stili",
  "None": "Hiçbiri",
  "Raised": "Kabarık",
  "Depressed": "Yassı",
  "Uniform": "Düz",
  "Drop shadow": "Gölgeli",
  "Font Family": "Yazı Tipi",
  "Proportional Sans-Serif": "Orantılı Sans-Serif",
  "Monospace Sans-Serif": "Eşaralıklı Sans-Serif",
  "Proportional Serif": "Orantılı Serif",
  "Monospace Serif": "Eşaralıklı Serif",
  "Casual": "Gündelik",
  "Script": "El Yazısı",
  "Small Caps": "Küçük Boyutlu Büyük Harfli",
  "Reset": "Sıfırla",
  "restore all settings to the default values": "tüm ayarları varsayılan değerlere geri yükler",
  "Done": "Tamam",
  "Caption Settings Dialog": "Altyazı Ayarları Menüsü",
  "Beginning of dialog window. Escape will cancel and close the window.": "Etkileşim penceresinin başlangıcı. ESC tuşu işlemi iptal edip pencereyi kapatacaktır.",
  "End of dialog window.": "Etkileşim penceresinin sonu.",
  "{1} is loading.": "{1} yükleniyor.",
  "Exit Picture-in-Picture": "Mini oynatıcıdan çık",
  "Picture-in-Picture": "Mini oynatıcı",
  "No content": "İçerik yok"
});