{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/section/availability

    Displays a section availability.

    The 3.x version course edition requires an existing section_availability div.

    Example context (json):
    {
        "info": [
            {
                "text": "Not available unless: <ul><li>It is on or after <strong>8 June 2012</strong></li></ul>",
                "isrestricted": 1,
                "isfullinfo": 1
            }
        ],
        "hasavailability": true
    }
}}
<div class="section_availability">
{{#hasavailability}}
    {{#info}}
        <div class="availabilityinfo {{classes}}" data-region="availabilityinfo">
        {{^isrestricted}}
            <span class="badge rounded-pill bg-warning text-dark">{{{text}}}</span>
        {{/isrestricted}}
        {{#isrestricted}}
            {{> core_courseformat/local/content/availability }}
        {{/isrestricted}}
        </div>
    {{/info}}
{{/hasavailability}}
</div>
