<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'notes', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package    core_notes
 * @subpackage notes
 * @copyright  1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['addnewnote'] = 'Add a new note';
$string['addbulknote'] = 'Add a new note to {$a} people';
$string['addbulknotesingle'] = 'Add a new note to 1 person';
$string['addbulknotedone'] = 'Note added to {$a} people';
$string['addbulknotedonesingle'] = 'Note added to 1 person';
$string['addnewnoteselect'] = 'Select users to write notes about';
$string['author'] = 'Author';
$string['bynameondate'] = 'by {$a->name} - {$a->date}';
$string['configenablenotes'] = 'Enable storing of notes about individual users.';
$string['content'] = 'Content';
$string['course'] = 'course';
$string['coursenotes'] = 'Course notes';
$string['created'] = 'created';
$string['deleteconfirm'] = 'Delete this note?';
$string['deletenotes'] = 'All notes';
$string['editnote'] = 'Edit note';
$string['enablenotes'] = 'Enable notes';
$string['eventnotecreated'] = 'Note created';
$string['eventnoteupdated'] = 'Note updated';
$string['eventnotedeleted'] = 'Note deleted';
$string['eventnotesviewed'] = 'Notes viewed';
$string['invalidid'] = 'Invalid note ID specified';
$string['invaliduserid'] = 'Invalid user id: {$a}';
$string['myprofileownnotes'] = 'My notes';
$string['nocontent'] = 'Note content can not be empty';
$string['nonotes'] = 'There are no notes of this type yet';
$string['nopermissiontodelete'] = 'You may not delete this note';
$string['note'] = 'Note';
$string['notes'] = 'Notes';
$string['notesdisabled'] = 'Notes are disabled, sorry.';
$string['notesnotvisible'] = 'You are not allowed to view the notes.';
$string['nouser'] = 'You must select a user';
$string['page-notes-x'] = 'Any note pages';
$string['page-notes-index'] = 'Notes main page';
$string['personal'] = 'personal';
$string['personalnotes'] = 'Personal notes';
$string['privacy:metadata:core_notes'] = 'The Notes component stores user notes within the core subsystem.';
$string['privacy:metadata:core_notes:content'] = 'The content of the note.';
$string['privacy:metadata:core_notes:courseid'] = 'The ID of the course associated with the note';
$string['privacy:metadata:core_notes:created'] = 'The creation date/time for the note.';
$string['privacy:metadata:core_notes:lastmodified'] = 'The last modified date/time for the note.';
$string['privacy:metadata:core_notes:publishstate'] = 'The publish state of the note.';
$string['privacy:metadata:core_notes:userid'] = 'The ID of the user associated with the note';
$string['publishstate'] = 'Context';
$string['publishstate_help'] = 'A note\'s context determines who can see the note in everyday use. Users should be aware that all notes, including personal ones, may be disclosed under the laws of their jurisdictions.

* Personal - The note will be visible only to you
* Course - The note will be visible to teachers in this course
* Site - The note will be visible to teachers in all courses';
$string['recipient'] = 'Recipient';
$string['site'] = 'site';
$string['sitenotes'] = 'Site notes';
$string['unknown'] = 'unknown';
