{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_lp/competency_rule_points

    Configuration dialogue for competency points rule.

    Context variables required for this template:
    * children
      * id
      * shortname
      * points
    * requiredpoints

    Example context (json):
    {
        "children": [
            {
                "id": "1",
                "shortname": "Competency",
                "points": 2
            }
        ],
        "requiredpoints": 4
    }
}}
<div class="competency-rule-points">
    <table class="table table-condensed">
        <thead>
            <tr>
                <th scope="col"></th>
                <th scope="col">{{#str}}points, tool_lp{{/str}}</th>
                <th scope="col">{{#str}}required{{/str}}</th>
            </tr>
        </thead>
        <tbody>
            {{#children}}
            <tr data-competency="{{id}}">
                <th scope="row">{{{shortname}}}</th>
                <td>
                    <label class="accesshide" for="pointsforcompetency-{{id}}">{{#str}}pointsgivenfor, tool_lp, {{{competency.shortname}}}{{/str}}</label>
                    <input id="pointsforcompetency-{{id}}" type="number" min="0" value="{{points}}" name="points" class="form-control"/>
                </td>
                <td>
                    <label class="accesshide" for="competency-{{id}}-isrequired">{{#str}}aisrequired, tool_lp, {{{competency.shortname}}}{{/str}}</label>
                    <input id="competency-{{id}}-isrequired" type="checkbox" value="1" name="required" {{#required}}checked{{/required}} />
                </td>
            </tr>
            {{/children}}
        </tbody>
        <tfoot>
            <tr>
                <th scope="row">{{#str}}totalrequiredtocomplete, tool_lp{{/str}}</th>
                <td><input type="number" min="1" value="{{requiredpoints}}" name="requiredpoints" aria-label="{{#str}}totalrequiredtocomplete, tool_lp{{/str}}" class="form-control"></td>
                <td> </td>
            </tr>
        </tfoot>
    </table>
</div>
