{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/local/grades/local/grader/user_picker_placeholder

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * none

    Example context (json):
    {
    }
}}
<div class="d-flex align-items-center user-picker-container mb-2 py-2">
    <div class="d-flex align-items-center">
        <div class="bg-pulse-grey rounded-circle d-inline-block flex-shrink-0 userpicture me-2"></div>
        <div class="bg-pulse-grey d-inline-block" style="height: 22px; width: 150px"></div>
    </div>
    <div class="ms-auto flex-shrink-0">
        <nav>
            <ul class="pagination mb-0">
                <li class="page-item">
                    <a class="bg-pulse-grey border-0 page-link icon-no-margin p-0 icon-size-3" href="#"></a>
                </li>
                <li class="page-item">
                    <a class="bg-pulse-grey border-0 page-link icon-no-margin p-0 icon-size-3 ms-2" href="#"></a>
                </li>
            </ul>
        </nav>
    </div>
</div>
