{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_lti/tool_config_modal_footer

    This template provides the layout for the footer of the configuration summary modal for
    a single tool opened from the tool configuration page.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * mailto

    Example context (json):
    {
        "mailto": "mailto:?subject=...&body=..."
    }

}}
<div>
  <button type="button" class="btn btn-primary" onclick="location.href='{{mailto}}';">{{#str}} tooldetailsmodalemail, mod_lti {{/str}}</button>
  <button type="button" class="btn btn-secondary" data-action="hide">{{#str}} cancel {{/str}}</button>
</div>
