{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/cm/bulkselect

    Displays an activity bulk selector.

    Example context (json):
    {
        "activityname": "Activity example",
        "cmid": 42
    }
}}
<div class="bulkselect d-none" data-for="cmBulkSelect">
    <input
        id="cmCheckbox{{cmid}}"
        type="checkbox"
        data-id="{{cmid}}"
        data-action="toggleSelectionCm"
        data-bulkcheckbox="1"
    >
    <label class="visually-hidden" for="cmCheckbox{{cmid}}">
        {{#str}} selectcm, core_courseformat, {{activityname}}{{/str}}
    </label>
</div>
