{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_h5p/h5peditor

    Displays the H5P Editor form.

    Classes required for JS:
    * editor_display

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * none

    Example context (json):
    {
    }
}}
<div class="h5p-editor-wrapper" id="editor-region-{{uniqid}}">
    <div class="h5p-editor">
        {{> core/loading }}
    </div>
</div>
<div class="h5p-editor-upload"></div>
{{#js}}
    require(['core_h5p/editor_display'], function(Editor) {
        Editor.init("editor-region-{{uniqid}}");
    });
{{/js}}